@@ -119,7 +119,7 @@ struct cve_entry {
 	struct oscap_list *products;	/* vulnerable SW list       */
 	char *published;	/* published datetime       */
 	char *modified;		/* last modified datetime   */
-	struct cvss_entry *cvss;	/* [cvss] score definition  */
+	struct cvss_impact *cvss;	/* [cvss] score definition  */
 	char *sec_protection;
 	char *cwe;		/* cwe (0-1)                */
 	struct oscap_list *summaries;	/* summaries                */
@@ -179,7 +179,7 @@ OSCAP_ACCESSOR_STRING(cve_entry, id)
 /* End of XML string variables definitions
  * */
 /***************************************************************************/
-const struct cvss_entry *cve_entry_get_cvss(const struct cve_entry *item)
+const struct cvss_impact *cve_entry_get_cvss(const struct cve_entry *item)
 {
 
 	if (item == NULL)
@@ -236,7 +236,7 @@ struct cve_entry * cve_entry_clone(struct cve_entry * old_entry)
     new_entry->cwe = oscap_strdup(old_entry->cwe);
     new_entry->sec_protection = oscap_strdup(old_entry->sec_protection);
 
-    new_entry->cvss = cvss_entry_clone(old_entry->cvss);
+    new_entry->cvss = cvss_impact_clone(old_entry->cvss);
 
     new_entry->products = oscap_list_clone(old_entry->products, (oscap_clone_func) cve_product_clone);
     new_entry->references = oscap_list_clone(old_entry->references, (oscap_clone_func) cve_reference_clone);
@@ -571,11 +571,13 @@ struct cve_entry *cve_entry_parse(xmlTextReaderPtr reader)
 		    xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
 			/* here will come function to parse test-expr */
 			conf = cve_configuration_new();
-			conf->id = (char *)xmlTextReaderGetAttribute(reader, ATTR_CVE_ID_STR);
-			xmlTextReaderNextElement(reader);
-			conf->expr = cpe_testexpr_parse(reader);
-			if (conf)
-				oscap_list_add(ret->configurations, conf);
+
+                        if (conf) {
+                                conf->id = (char *)xmlTextReaderGetAttribute(reader, ATTR_CVE_ID_STR);
+                                xmlTextReaderNextElement(reader);
+                                conf->expr = cpe_testexpr_parse(reader);
+                                oscap_list_add(ret->configurations, conf);
+                        }
                         continue;
 		} else
 		    if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_VULNERABLE_SOFTWARE_LIST_STR) &&
@@ -586,15 +588,11 @@ struct cve_entry *cve_entry_parse(xmlTextReaderPtr reader)
 				if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_PRODUCT_STR) &&
 				    xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
 					product = cve_product_new();
-					product->value = (char *)xmlTextReaderReadString(reader);
-					/*if (!data) { 
-					   cve_entry_free(ret);
-					   return NULL;
-					   }
-					   product->value = cpe_name_new(data);
-					   oscap_free(data); */
-					if (product)
+
+                                        if (product) {
+                                                product->value = (char *)xmlTextReaderReadString(reader);
 						oscap_list_add(ret->products, product);
+                                        }
 				}
 				xmlTextReaderNextNode(reader);
 			}
@@ -613,7 +611,7 @@ struct cve_entry *cve_entry_parse(xmlTextReaderPtr reader)
 		} else
 		    if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CVSS_STR) &&
 			xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
-			ret->cvss = cvss_entry_parse(reader);
+            if (ret->cvss == NULL) ret->cvss = cvss_impact_new_from_xml(reader);
 		} else
 		    if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_SECURITY_PROTECTION_STR) &&
 			xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
@@ -626,14 +624,16 @@ struct cve_entry *cve_entry_parse(xmlTextReaderPtr reader)
 		    if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_REFERENCES_STR) &&
 			xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
 			refer = cve_reference_new();
-			refer->type = (char *)xmlTextReaderGetAttribute(reader, ATTR_REFERENCE_TYPE_STR);
-			refer->lang = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "xml:lang");
-			xmlTextReaderNextNode(reader);
-			while (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_REFERENCES_STR) != 0) {
+
+			if (refer) {
+			    refer->type = (char *)xmlTextReaderGetAttribute(reader, ATTR_REFERENCE_TYPE_STR);
+			    refer->lang = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "xml:lang");
+			    xmlTextReaderNextNode(reader);
+			    while (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_REFERENCES_STR) != 0) {
 
 				if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_SOURCE_STR) &&
 				    xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
-					refer->source = (char *)xmlTextReaderReadString(reader);
+				    refer->source = (char *)xmlTextReaderReadString(reader);
 				} else
 				    if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_REFERENCE_STR) &&
 					xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
@@ -641,18 +641,20 @@ struct cve_entry *cve_entry_parse(xmlTextReaderPtr reader)
 					    (char *)xmlTextReaderGetAttribute(reader, ATTR_REFERENCE_HREF_STR);
 					refer->value = (char *)xmlTextReaderReadString(reader);
 
-				}
+				    }
 				xmlTextReaderNextNode(reader);
+			    }
+			    oscap_list_add(ret->references, refer);
 			}
-			if (refer)
-				oscap_list_add(ret->references, refer);
 		} else
 		    if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_SUMMARY_STR) &&
 			xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
 			summary = cve_summary_new();
-			summary->summary = (char *)xmlTextReaderReadString(reader);
-			if (summary)
+
+                        if (summary) {
+                                summary->summary = (char *)xmlTextReaderReadString(reader);
 				oscap_list_add(ret->summaries, summary);
+                        }
 		} else if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
 			oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_EXMLELEM, "Unknown XML element in CVE entry");
 		}
@@ -808,7 +810,7 @@ void cve_entry_export(const struct cve_entry *entry, xmlTextWriterPtr writer)
 		xmlTextWriterWriteElementNS(writer, NULL, TAG_LAST_MODIFIED_DATETIME_STR, CVE_NS, BAD_CAST entry->modified);
 
 	if ((entry->cvss) != NULL) {
-		cvss_entry_export(entry->cvss, writer);
+		cvss_impact_export(entry->cvss, writer);
 	}
 	if ((entry->sec_protection) != NULL)
 		xmlTextWriterWriteElementNS(writer, NULL, TAG_SECURITY_PROTECTION_STR, CVE_NS, BAD_CAST entry->sec_protection);
@@ -919,7 +921,7 @@ void cve_entry_free(struct cve_entry *entry)
 	xmlFree(entry->modified);
 	xmlFree(entry->sec_protection);
 	xmlFree(entry->cwe);
-	cvss_entry_free(entry->cvss);
+	cvss_impact_free(entry->cvss);
 	oscap_list_free(entry->products, (oscap_destruct_func) cve_product_free);
 	oscap_list_free(entry->references, (oscap_destruct_func) cve_reference_free);
 	oscap_list_free(entry->summaries, (oscap_destruct_func) cve_summary_free);
