@@ -256,7 +256,7 @@ static void oval_consume_varref(char *varref, void *user)
 	__attribute__nonnull__(user);
 
 	struct oval_consume_varref_context *ctx = user;
-	*(ctx->variable) = oval_variable_get_new((struct oval_definition_model *)ctx->model, varref, OVAL_VARIABLE_UNKNOWN);
+	*(ctx->variable) = oval_definition_model_get_new_variable((struct oval_definition_model *)ctx->model, varref, OVAL_VARIABLE_UNKNOWN);
 	*(ctx->value) = oval_value_new(OVAL_DATATYPE_STRING, varref);
 }
 
@@ -304,11 +304,14 @@ int oval_entity_parse_tag(xmlTextReaderPtr reader,
 		if (datatype == OVAL_DATATYPE_RECORD) {
 			value = NULL;
 		} else {
-			return_code = oval_value_parse_tag(reader, context, &oval_consume_value, &value);
+			if (oval_parser_boolean_attribute(reader, "xsi:nil", 0))
+				value = NULL;
+			else
+				return_code = oval_value_parse_tag(reader, context, &oval_consume_value, &value);
 		}
 	} else {
 		struct oval_definition_model *model = context->definition_model;
-		variable = oval_variable_get_new(model, varref, OVAL_VARIABLE_UNKNOWN);
+		variable = oval_definition_model_get_new_variable(model, varref, OVAL_VARIABLE_UNKNOWN);
 		varref_type = OVAL_ENTITY_VARREF_ATTRIBUTE;
 		value = NULL;
 		oscap_free(varref);
@@ -332,11 +335,9 @@ int oval_entity_parse_tag(xmlTextReaderPtr reader,
 	return return_code;
 }
 
-
 xmlNode *oval_entity_to_dom(struct oval_entity *entity, xmlDoc * doc, xmlNode * parent) {
 	
-	xmlNsPtr *ns_parent = xmlGetNsList(doc, parent);
-	xmlNs *ent_ns;
+	xmlNsPtr ent_ns = parent->ns;
 	xmlNodePtr root_node = xmlDocGetRootElement(doc);
 
 	xmlNode *entity_node = NULL;
@@ -355,8 +356,6 @@ xmlNode *oval_entity_to_dom(struct oval_entity *entity, xmlDoc * doc, xmlNode *
 	char *tagname = oval_entity_get_name(entity);
 	bool mask = oval_entity_get_mask(entity);
 
-	ent_ns = ns_parent ? ns_parent[0] : NULL;
-
 	/* omit the value and operation used for testing in oval_results if mask=true */
 	if(mask && !xmlStrcmp(root_node->name, (const xmlChar *) "oval_results")){
 		entity_node = xmlNewTextChild(parent, ent_ns, BAD_CAST tagname, BAD_CAST "");
@@ -376,8 +375,5 @@ xmlNode *oval_entity_to_dom(struct oval_entity *entity, xmlDoc * doc, xmlNode *
 	if (vtype == OVAL_ENTITY_VARREF_ATTRIBUTE)
 		xmlNewProp(entity_node, BAD_CAST "var_ref", BAD_CAST oval_variable_get_id(variable));
 
-	if(ns_parent)
-		xmlFree(ns_parent);
-
 	return entity_node;
 }
