@@ -206,6 +206,8 @@ static const struct oscap_string_map OVAL_OPERATION_MAP[] = {
 	{OVAL_OPERATION_BITWISE_AND, "bitwise and"},
 	{OVAL_OPERATION_BITWISE_OR, "bitwise or"},
 	{OVAL_OPERATION_PATTERN_MATCH, "pattern match"},
+	{OVAL_OPERATION_SUBSET_OF, "subset of"},
+	{OVAL_OPERATION_SUPERSET_OF, "superset of"},
 	{OVAL_ENUMERATION_INVALID, NULL}
 };
 
@@ -245,6 +247,8 @@ static const struct oscap_string_map OVAL_DATATYPE_MAP[] = {
 	{OVAL_DATATYPE_FILESET_REVISION, "fileset_revision"},
 	{OVAL_DATATYPE_FLOAT, "float"},
 	{OVAL_DATATYPE_IOS_VERSION, "ios_version"},
+	{OVAL_DATATYPE_IPV4ADDR, "ipv4_address"},
+	{OVAL_DATATYPE_IPV6ADDR, "ipv6_address"},
 	{OVAL_DATATYPE_VERSION, "version"},
 	{OVAL_DATATYPE_INTEGER, "int"},
 	{OVAL_DATATYPE_STRING, "string"},
@@ -422,7 +426,9 @@ static const struct oscap_string_map OVAL_SUBTYPE_INDEPENDENT_MAP[] = {
 	{OVAL_INDEPENDENT_FAMILY, "family"},
 	{OVAL_INDEPENDENT_FILE_MD5, "filemd5"},
 	{OVAL_INDEPENDENT_FILE_HASH, "filehash"},
+	{OVAL_INDEPENDENT_FILE_HASH58, "filehash58"},
 	{OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE, "environmentvariable"},
+	{OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE58, "environmentvariable58"},
 	{OVAL_INDEPENDENT_SQL57, "sql57"},
 	{OVAL_INDEPENDENT_SQL, "sql"},
 	{OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54"},
@@ -690,3 +696,22 @@ const char *oval_result_get_text(oval_result_t result)
 }
 
 
+static const struct oscap_string_map OVAL_DEFINITION_CLASS_MAP[] = {
+        {OVAL_CLASS_COMPLIANCE, "compliance"},
+        {OVAL_CLASS_INVENTORY, "inventory"},
+        {OVAL_CLASS_MISCELLANEOUS, "miscellaneous"},
+        {OVAL_CLASS_PATCH, "patch"},
+        {OVAL_CLASS_VULNERABILITY, "vulnerability"},
+        {OVAL_CLASS_UNKNOWN, NULL}
+};
+
+oval_definition_class_t oval_definition_class_enum(char *class)
+{
+        return oscap_string_to_enum(OVAL_DEFINITION_CLASS_MAP, class);
+}
+
+const char *oval_definition_class_text(oval_definition_class_t class)
+{
+        return oscap_enum_to_string(OVAL_DEFINITION_CLASS_MAP, class);
+}
+
