@@ -37,7 +37,7 @@
 
 #include "common/util.h"
 #include "common/debug_priv.h"
-#include "oval_agent_api_impl.h"
+#include "oval_definitions_impl.h"
 
 struct oval_generator {
 	char *product_name;
