@@ -27,9 +27,14 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_agent_api_impl.h"
 #include "oval_system_characteristics_impl.h"
 #include "oval_collection_impl.h"
@@ -52,18 +57,6 @@ struct oval_message *oval_message_new()
 	return message;
 }
 
-bool oval_message_is_valid(struct oval_message * message)
-{
-        oscap_dlprintf(DBG_W, "NOOP.\n");
-	return true;		//TODO
-}
-
-bool oval_message_is_locked(struct oval_message * message)
-{
-        //oscap_dlprintf(DBG_W, "NOOP.\n");
-	return false;		//TODO
-}
-
 struct oval_message *oval_message_clone(struct oval_message *old_message)
 {
 	struct oval_message *new_message = oval_message_new();
@@ -122,20 +115,16 @@ oval_message_level_t oval_message_get_level(struct oval_message * message)
 
 void oval_message_set_text(struct oval_message *message, char *text)
 {
-	if (message && !oval_message_is_locked(message)) {
-		if (message->text != NULL)
-			oscap_free(message->text);
-		message->text = (text == NULL) ? NULL : oscap_strdup(text);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(message);
+	if (message->text != NULL)
+		oscap_free(message->text);
+	message->text = (text == NULL) ? NULL : oscap_strdup(text);
 }
 
 void oval_message_set_level(struct oval_message *message, oval_message_level_t level)
 {
-	if (message && !oval_message_is_locked(message)) {
-		message->level = level;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(message);
+	message->level = level;
 }
 
 static void oval_message_parse_tag_consumer(char *text, void *message)
@@ -143,48 +132,21 @@ static void oval_message_parse_tag_consumer(char *text, void *message)
 	oval_message_set_text(message, text);
 }
 
-int oval_message_parse_tag(xmlTextReaderPtr reader,
-			   struct oval_parser_context *context, oscap_consumer_func consumer, void *client)
+int oval_message_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, oscap_consumer_func consumer, void *client)
 {
-	int return_code = 1;
+	int return_code = 0;
 	struct oval_message *message = oval_message_new();
-	{			/*message->lever */
-		oval_message_set_level(message, oval_message_level_parse(reader, "level", OVAL_MESSAGE_LEVEL_INFO));
-	}
-	{			/*message->text */
-		return_code = oval_parser_text_value(reader, context, &oval_message_parse_tag_consumer, message);
-	}
-	if (return_code != 1) {
-		oscap_dlprintf(DBG_W, "Return code is not 1: %d.\n", return_code);
-	} else {
-		oscap_dlprintf(DBG_I, "message->level: %d, message->text: %s.\n",
-			       oval_message_get_level(message), oval_message_get_text(message));
+	/* message->level */
+	oval_message_set_level(message, oval_message_level_parse(reader, "level", OVAL_MESSAGE_LEVEL_INFO));
+	/* message->text */
+	return_code = oval_parser_text_value(reader, context, &oval_message_parse_tag_consumer, message);
 
+        if (return_code == 0)
 		(*consumer) (message, client);
-	}
+
 	return return_code;
 }
 
-void oval_message_to_print(struct oval_message *message, char *indent, int idx)
-{
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sMESSAGE.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sMESSAGE[%d].", indent, idx);
-
-	/*
-	   char*                   text;
-	   oval_message_level_enum level;
-	 */
-	oscap_dprintf("%sLEVEL = %d\n", nxtindent, oval_message_get_level(message));
-	oscap_dprintf("%sTEXT  = %s\n", nxtindent, oval_message_get_text(message));
-}
-
 void oval_message_to_dom(struct oval_message *message, xmlDoc * doc, xmlNode * tag_parent)
 {
 
