@@ -46,70 +46,60 @@
 
 #include "_oval_probe_session.h"
 #include "_oval_probe_handler.h"
+#include "oval_probe_meta.h"
 
-/* Sorted by subtype (first column) */
-oval_subtypedsc_t __s2n_tbl[] = {
-        /*     2 */ {OVAL_SUBTYPE_SYSINFO,                  "system_info"         },
-	/*  7001 */ {OVAL_INDEPENDENT_FAMILY,               "family"              },
-        /*  7002 */ {OVAL_INDEPENDENT_FILE_MD5,             "filemd5"             },
-        /*  7003 */ {OVAL_INDEPENDENT_FILE_HASH,            "filehash"            },
-        /*  7004 */ {OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE, "environmentvariable" },
-	/*  7005 */ {OVAL_INDEPENDENT_SQL,                  "sql"                 },
-	/*  7006 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54"   },
-	/*  7007 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT,    "textfilecontent"     },
-        /*  7009 */ {OVAL_INDEPENDENT_VARIABLE,             "variable"            },
-	/*  7010 */ {OVAL_INDEPENDENT_XML_FILE_CONTENT,     "xmlfilecontent"      },
-	/*  7013 */ {OVAL_INDEPENDENT_SQL57,                "sql57"               },
-	/*  9001 */ {OVAL_LINUX_DPKG_INFO,                  "dpkginfo"            },
-	/*  9002 */ {OVAL_LINUX_INET_LISTENING_SERVERS,     "inetlisteningservers"},
-	/*  9003 */ {OVAL_LINUX_RPM_INFO,                   "rpminfo"             },
-        /*  9005 */ {OVAL_LINUX_PARTITION,                  "partition",          },
-	/* 13001 */ {OVAL_UNIX_FILE,                        "file"                },
-	/* 13003 */ {OVAL_UNIX_INTERFACE,                   "interface"           },
-	/* 13004 */ {OVAL_UNIX_PASSWORD,                    "password"            },
-	/* 13005 */ {OVAL_UNIX_PROCESS,                     "process"             },
-	/* 13006 */ {OVAL_UNIX_RUNLEVEL,                    "runlevel"            },
-	/* 13008 */ {OVAL_UNIX_SHADOW,                      "shadow"              },
-	/* 13009 */ {OVAL_UNIX_UNAME,                       "uname"               },
-        /* 13012 */ {OVAL_UNIX_SYSCTL,                      "sysctl"              }
+oval_probe_meta_t OSCAP_GSYM(__probe_meta)[] = {
+        { OVAL_SUBTYPE_SYSINFO, "system_info", &oval_probe_sys_handler, OVAL_PROBEMETA_EXTERNAL, "probe_system_info" },
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_FAMILY, "family"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_FILE_MD5, "filemd5"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_FILE_HASH, "filehash"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE, "environmentvariable"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_SQL, "sql"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_TEXT_FILE_CONTENT, "textfilecontent"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_VARIABLE, "variable"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_XML_FILE_CONTENT, "xmlfilecontent"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_SQL57, "sql57"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE58, "environmentvariable58"),
+        OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_FILE_HASH58, "filehash58"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_DPKG_INFO, "dpkginfo"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_INET_LISTENING_SERVERS, "inetlisteningservers"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_RPM_INFO, "rpminfo"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_PARTITION, "partition"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_IFLISTENERS, "iflisteners"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_RPMVERIFY, "rpmverify"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_SELINUXBOOLEAN, "selinuxboolean"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_SELINUXSECURITYCONTEXT, "selinuxsecuritycontext"),
+        OVAL_PROBE_EXTERNAL(OVAL_SOLARIS_ISAINFO, "isainfo"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_FILE, "file"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_INTERFACE, "interface"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_PASSWORD, "password"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_PROCESS, "process"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_RUNLEVEL, "runlevel"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_SHADOW, "shadow"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_UNAME, "uname"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_XINETD, "xinetd"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_SYSCTL, "sysctl"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_PROCESS58, "process58"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_FILEEXTENDEDATTRIBUTE, "fileextendedattribute"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_GCONF, "gconf"),
+        OVAL_PROBE_EXTERNAL(OVAL_UNIX_ROUTINGTABLE, "routingtable")
 };
 
-#define __s2n_tbl_count (sizeof __s2n_tbl / sizeof(oval_subtypedsc_t))
+#define __PROBE_META_COUNT (sizeof OSCAP_GSYM(__probe_meta)/sizeof OSCAP_GSYM(__probe_meta)[0])
+
+size_t OSCAP_GSYM(__probe_meta_count) = __PROBE_META_COUNT;
+oval_subtypedsc_t OSCAP_GSYM(__s2n_tbl)[__PROBE_META_COUNT];
+oval_subtypedsc_t OSCAP_GSYM(__n2s_tbl)[__PROBE_META_COUNT];
+
+#define __s2n_tbl_count OSCAP_GSYM(__probe_meta_count)
+#define __n2s_tbl_count OSCAP_GSYM(__probe_meta_count)
 
 static int __s2n_tbl_cmp(oval_subtype_t *type, oval_subtypedsc_t *dsc)
 {
         return (*type - dsc->type);
 }
 
-/* Sorted by name (second column) */
-oval_subtypedsc_t __n2s_tbl[] = {
-	/*  9001 */ {OVAL_LINUX_DPKG_INFO,                  "dpkginfo"            },
-        /*  7004 */ {OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE, "environmentvariable" },
-	/*  7001 */ {OVAL_INDEPENDENT_FAMILY,               "family"              },
-	/* 13001 */ {OVAL_UNIX_FILE,                        "file"                },
-        /*  7003 */ {OVAL_INDEPENDENT_FILE_HASH,            "filehash"            },
-        /*  7002 */ {OVAL_INDEPENDENT_FILE_MD5,             "filemd5"             },
-	/*  9002 */ {OVAL_LINUX_INET_LISTENING_SERVERS,     "inetlisteningservers"},
-	/* 13003 */ {OVAL_UNIX_INTERFACE,                   "interface"           },
-	/* 13004 */ {OVAL_UNIX_PASSWORD,                    "password"            },
-        /*  9005 */ {OVAL_LINUX_PARTITION,                  "partition"           },
-	/* 13005 */ {OVAL_UNIX_PROCESS,                     "process"             },
-	/*  9003 */ {OVAL_LINUX_RPM_INFO,                   "rpminfo"             },
-	/* 13006 */ {OVAL_UNIX_RUNLEVEL,                    "runlevel"            },
-	/* 13008 */ {OVAL_UNIX_SHADOW,                      "shadow"              },
-	/*  7005 */ {OVAL_INDEPENDENT_SQL,                  "sql"                 },
-	/*  7013 */ {OVAL_INDEPENDENT_SQL57,                "sql57"               },
-        /* 13012 */ {OVAL_UNIX_SYSCTL,                      "sysctl"              },
-        /*     2 */ {OVAL_SUBTYPE_SYSINFO,                  "system_info"         },
-	/*  7007 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT,    "textfilecontent"     },
-	/*  7006 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54"   },
-	/* 13009 */ {OVAL_UNIX_UNAME,                       "uname"               },
-        /*  7009 */ {OVAL_INDEPENDENT_VARIABLE,             "variable"            },
-	/*  7010 */ {OVAL_INDEPENDENT_XML_FILE_CONTENT,     "xmlfilecontent"      }
-};
-
-#define __n2s_tbl_count (sizeof __n2s_tbl / sizeof(oval_subtypedsc_t))
-
 static int __n2s_tbl_cmp(const char *name, oval_subtypedsc_t *dsc)
 {
         return strcmp(name, dsc->name);
@@ -122,27 +112,80 @@ static int __n2s_tbl_cmp(const char *name, oval_subtypedsc_t *dsc)
  * of memory used by this cache is done at exit using a hook
  * registered with atexit().
  */
-probe_ncache_t  *OSCAP_GSYM(encache) = NULL;
+probe_ncache_t  *OSCAP_GSYM(ncache) = NULL;
 struct id_desc_t OSCAP_GSYM(id_desc);
 #endif
 
+#if defined(OSCAP_THREAD_SAFE)
+# include <pthread.h>
+static pthread_once_t __oval_probe_init_once = PTHREAD_ONCE_INIT;
+#else
+static volatile int __oval_probe_init_once = 0;
+#endif
+
 #define __ERRBUF_SIZE 128
 
-const char *oval_subtype2str(oval_subtype_t subtype)
+static int __s2n_tbl_sortcmp(oval_subtypedsc_t *a, oval_subtypedsc_t *b)
+{
+        return (a->type - b->type);
+}
+
+static int __n2s_tbl_sortcmp(oval_subtypedsc_t *a, oval_subtypedsc_t *b)
+{
+        return strcmp(a->name, b->name);
+}
+
+void oval_probe_tblinit(void)
+{
+        register size_t i;
+
+        for(i = 0; i < OSCAP_GSYM(__probe_meta_count); ++i) {
+                OSCAP_GSYM(__s2n_tbl)[i].type = OSCAP_GSYM(__probe_meta)[i].otype;
+                OSCAP_GSYM(__n2s_tbl)[i].type = OSCAP_GSYM(__probe_meta)[i].otype;
+                OSCAP_GSYM(__s2n_tbl)[i].name = OSCAP_GSYM(__probe_meta)[i].stype;
+                OSCAP_GSYM(__n2s_tbl)[i].name = OSCAP_GSYM(__probe_meta)[i].stype;
+        }
+
+        qsort(OSCAP_GSYM(__s2n_tbl), OSCAP_GSYM(__probe_meta_count), sizeof (oval_subtypedsc_t),
+              (int(*)(const void *, const void *))__s2n_tbl_sortcmp);
+
+        qsort(OSCAP_GSYM(__n2s_tbl), OSCAP_GSYM(__probe_meta_count), sizeof (oval_subtypedsc_t),
+              (int(*)(const void *, const void *))__n2s_tbl_sortcmp);
+}
+
+static void __init_once(void)
+{
+#if defined(OSCAP_THREAD_SAFE)
+        if (pthread_once(&__oval_probe_init_once, &oval_probe_tblinit) != 0)
+                abort();
+#else
+        if (__oval_probe_init_once == 0) {
+                oval_probe_tblinit();
+                __oval_probe_init_once = 1;
+        }
+#endif
+        return;
+}
+
+const char *oval_subtype_to_str(oval_subtype_t subtype)
 {
         oval_subtypedsc_t *d;
 
-        d = oscap_bfind(__s2n_tbl, __s2n_tbl_count, sizeof(oval_subtypedsc_t), &subtype,
+        __init_once();
+
+        d = oscap_bfind(OSCAP_GSYM(__s2n_tbl), __s2n_tbl_count, sizeof(oval_subtypedsc_t), &subtype,
                         (int(*)(void *, void *))__s2n_tbl_cmp);
 
         return (d == NULL ? NULL : d->name);
 }
 
-oval_subtype_t oval_str2subtype(const char *str)
+oval_subtype_t oval_str_to_subtype(const char *str)
 {
         oval_subtypedsc_t *d;
 
-        d = oscap_bfind(__n2s_tbl, __n2s_tbl_count, sizeof(oval_subtypedsc_t), (void *)str,
+        __init_once();
+
+        d = oscap_bfind(OSCAP_GSYM(__n2s_tbl), __n2s_tbl_count, sizeof(oval_subtypedsc_t), (void *)str,
                         (int(*)(void *, void *))__n2s_tbl_cmp);
 
         return (d == NULL ? OVAL_SUBTYPE_UNKNOWN : d->type);
@@ -358,11 +401,11 @@ int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *obj
 				*out_syschar = sysc;
 			return 0;
 		}
-	} else {
+	} else
 		sysc = oval_syschar_new(model, object);
-		if (out_syschar)
-			*out_syschar = sysc;
-	}
+
+	if (out_syschar)
+		*out_syschar = sysc;
 
 	type = oval_object_get_subtype(object);
 	ph = oval_probe_handler_get(psess->ph, type);
@@ -404,6 +447,11 @@ int oval_probe_query_sysinfo(oval_probe_session_t *sess, struct oval_sysinfo **o
 		return(-1);
         }
 
+        if (ph->func == NULL) {
+                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, "OVAL object not correctly defined");
+		return(-1);
+        }
+
         sysinf = NULL;
 
 	ret = ph->func(OVAL_SUBTYPE_SYSINFO, ph->uptr, PROBE_HANDLER_ACT_EVAL, NULL, &sysinf, 0);
@@ -427,8 +475,10 @@ int oval_probe_query_definition(oval_probe_session_t *sess, const char *id) {
         definition_model = oval_syschar_model_get_definition_model(syschar_model);
 	definition = oval_definition_model_get_definition(definition_model, id);
 	if (definition == NULL) {
-		dE("No definition with ID: %s in definition model.", id);
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "Unknown definition.");
+                char msg[100];
+                snprintf(msg, sizeof(msg), "No definition with ID: %s in definition model.", id);
+                dE(msg);
+                oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, msg);
 		return -1;
 	}
 
@@ -442,7 +492,7 @@ int oval_probe_query_definition(oval_probe_session_t *sess, const char *id) {
 }
 
 /**
- * @returns 0 on success
+ * @returns 0 on success; -1 on error; 1 on warning
  */
 static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_criteria_node *cnode) {
 	int ret;
@@ -463,7 +513,7 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
 			return 0;
 		/* probe object */
 		ret = oval_probe_query_object(sess, object, 0, NULL);
-		if (ret != 0)
+		if (ret == -1)
 			return ret;
 		/* probe objects referenced like this: test->state->variable->object */
 		ste_itr = oval_test_get_states(test);
@@ -478,7 +528,7 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
 					struct oval_variable *var = oval_entity_get_variable(entity);
 
 					ret = oval_probe_query_variable(sess, var);
-					if (ret != 0) {
+					if (ret == -1) {
 						oval_state_content_iterator_free(contents);
 						oval_state_iterator_free(ste_itr);
 						return ret;
@@ -540,3 +590,67 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
 	/* we shouldn't get here */
         return -1;
 }
+
+#if 0
+const oval_probe_meta_t * const oval_probe_meta_get(void)
+{
+    return (const oval_probe_meta_t * const)OSCAP_GSYM(__probe_meta);
+}
+#endif
+
+void oval_probe_meta_list(FILE *output, int flags)
+{
+    register size_t i;
+    const char *probe_dir;
+    char probe_path[PATH_MAX];
+    size_t probe_dirlen;
+    oval_probe_meta_t *meta = OSCAP_GSYM(__probe_meta);
+
+    if (output == NULL)
+	output = stdout;
+#if 0
+    fprintf(output, "#\n# %-24s %-30s %s", "object", "probe name", "flags");
+
+    if (flags & OVAL_PROBEMETA_LIST_VERBOSE) {
+	fprintf(output, " %-5s %s\n#\n", "type", "path");
+    } else
+	fprintf(output, "\n#\n");
+#endif
+    probe_dir = oval_probe_ext_getdir();
+    assume_d(probe_dir != NULL, /* void */);
+    probe_dirlen = strlen(probe_dir);
+    assume_r(probe_dirlen + 1 < sizeof probe_path - 1, /* void */);
+    strncpy(probe_path, probe_dir, sizeof probe_path - 1);
+
+    probe_path[probe_dirlen  ] = '/';
+    probe_path[probe_dirlen+1] = '\0';
+
+    for (i = 0; i < OSCAP_GSYM(__probe_meta_count); ++i) {
+	if (meta[i].flags & OVAL_PROBEMETA_EXTERNAL) {
+	    strncpy(probe_path + probe_dirlen + 1,
+		    meta[i].pname,
+		    sizeof probe_path - probe_dirlen + 1);
+
+	    if (flags & OVAL_PROBEMETA_LIST_DYNAMIC) {
+		dI("Checking access to \"%s\"\n", probe_path);
+		if (access(probe_path, X_OK) != 0) {
+		    dW("access: errno=%d, %s\n", errno, strerror(errno));
+		    continue;
+		}
+	    }
+	}
+
+	fprintf(output, "%-32s %-32s %c",
+		meta[i].stype, meta[i].pname,
+		meta[i].flags & OVAL_PROBEMETA_EXTERNAL ? 'E' : '.');
+
+	if (flags & OVAL_PROBEMETA_LIST_VERBOSE) {
+	    fprintf(output, " %-5u %s\n",
+		    meta[i].otype,
+		    meta[i].flags & OVAL_PROBEMETA_EXTERNAL ? probe_path : "");
+	} else
+	    fprintf(output, "\n");
+    }
+
+    return;
+}
