@@ -42,40 +42,13 @@
 #include "probes/public/probe-api.h"
 #include "oval_probe_ext.h"
 #include "oval_sexp.h"
+#include "oval_probe_meta.h"
 
 #define __ERRBUF_SIZE 128
 
 #if defined(__SVR4) && defined (__sun)
 #define __STRING(x)     #x
 #endif
-/* KEEP THIS LIST SORTED! (by subtype) */
-const oval_pdsc_t OSCAP_GSYM(default_pdsc)[] = {
-        {OVAL_SUBTYPE_SYSINFO,                  "system_info",       "probe_system_info"},
-	{OVAL_INDEPENDENT_FAMILY,               "family",            "probe_family"},
-        {OVAL_INDEPENDENT_FILE_MD5,             "filemd5",           "probe_filemd5"},
-        {OVAL_INDEPENDENT_FILE_HASH,            "filehash",          "probe_filehash"},
-        {OVAL_INDEPENDENT_SQL,                  "sql",               "probe_sql"},
-        {OVAL_INDEPENDENT_SQL57,                "sql57",             "probe_sql57"},
-	{OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54", "probe_textfilecontent54"},
-	{OVAL_INDEPENDENT_TEXT_FILE_CONTENT,    "textfilecontent",   "probe_textfilecontent"},
-	{OVAL_INDEPENDENT_XML_FILE_CONTENT,     "xmlfilecontent",    "probe_xmlfilecontent"},
-        {OVAL_INDEPENDENT_LDAP57,               "ldap57",            "probe_ldap57"},
-	{OVAL_LINUX_DPKG_INFO,                  "dpkginfo",          "probe_dpkginfo"},
-	{OVAL_LINUX_INET_LISTENING_SERVERS,     "inetlisteningservers", "probe_inetlisteningservers"},
-	{OVAL_LINUX_RPM_INFO,                   "rpminfo",           "probe_rpminfo"},
-        {OVAL_LINUX_PARTITION,                  "partition",         "probe_partition"},
-	{OVAL_UNIX_FILE,                        "file",              "probe_file"},
-	{OVAL_UNIX_INTERFACE,                   "interface",         "probe_interface"},
-	{OVAL_UNIX_PASSWORD,                    "password",          "probe_password"},
-	{OVAL_UNIX_PROCESS,                     "process",           "probe_process"},
-	{OVAL_UNIX_RUNLEVEL,                    "runlevel",          "probe_runlevel"},
-	{OVAL_UNIX_SHADOW,                      "shadow",            "probe_shadow"},
-	{OVAL_UNIX_UNAME,                       "uname",             "probe_uname"},
-        {OVAL_UNIX_DNSCACHE,                    "dnscache",          "probe_dnscache"},
-        {OVAL_UNIX_SYSCTL,                      "sysctl",            "probe_sysctl"}
-};
-
-#define DEFAULT_PDSC_COUNT (sizeof OSCAP_GSYM(default_pdsc) / sizeof (oval_pdsc_t))
 
 static oval_pdtbl_t *oval_pdtbl_new(void);
 static void          oval_pdtbl_free(oval_pdtbl_t *table);
@@ -320,7 +293,7 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 				return (NULL);
 			}
 
-			ret = oval_state2sexp(pext->sess_ptr, ste, &ste_sexp);
+			ret = oval_state_to_sexp(pext->sess_ptr, ste, &ste_sexp);
 			if (ret !=0) {
 				oscap_dlprintf(DBG_E, "Failed to convert OVAL state to SEXP, id: %s.\n",
 					       id_str);
@@ -802,8 +775,10 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
                 pd  = oval_pdtbl_get(pext->pdtbl, type);
 
                 if (pd == NULL) {
-                        if (oval_probe_sys_handler(type, ptr, PROBE_HANDLER_ACT_OPEN) != 0)
+                        if (oval_probe_sys_handler(type, ptr, PROBE_HANDLER_ACT_OPEN) != 0) {
+				va_end(ap);
                                 return(-1);
+			}
 
                         pd = oval_pdtbl_get(pext->pdtbl, type);
                 }
@@ -893,10 +868,10 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                         probe_dsc = oval_pdsc_lookup(pext->pdsc, pext->pdsc_cnt, oval_object_get_subtype(obj));
 
 			if (probe_dsc == NULL) {
-				snprintf (errmsg, sizeof errmsg, "subtype %u not supported", oval_object_get_subtype(obj));
-				oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
-
-				return (-1);
+				oval_syschar_add_new_message(sys, "OVAL object not supported", OVAL_MESSAGE_LEVEL_WARNING);
+				oval_syschar_set_flag(sys, SYSCHAR_FLAG_NOT_COLLECTED);
+				va_end(ap);
+				return (1);
 			}
 
                         probe_urilen = snprintf(probe_uri, sizeof probe_uri,
@@ -905,24 +880,24 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                         if (probe_urilen >= sizeof probe_uri) {
                                 snprintf (errmsg, sizeof errmsg, "probe URI too long");
                                 oscap_seterr (OSCAP_EFAMILY_GLIBC, ENAMETOOLONG, errmsg);
-
+				va_end(ap);
                                 return (-1);
                         }
 
                         oscap_dlprintf(DBG_I, "URI: %s.\n", probe_uri);
 
                         if (oval_pdtbl_add(pext->pdtbl, oval_object_get_subtype(obj), -1, probe_uri) != 0) {
-
-                                snprintf (errmsg, sizeof errmsg, "%s probe not supported", probe_dsc->name);
-                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
-
-                                return (-1);
+				oval_syschar_add_new_message(sys, "OVAL object not supported", OVAL_MESSAGE_LEVEL_WARNING);
+				oval_syschar_set_flag(sys, SYSCHAR_FLAG_NOT_COLLECTED);
+				va_end(ap);
+                                return (1);
 			}
 
 			pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
 
                         if (pd == NULL) {
                                 oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "internal error");
+				va_end(ap);
                                 return (-1);
                         }
                 }
@@ -949,6 +924,7 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
 			}
 		}
 
+		va_end(ap);
 		return ret;
         }
         case PROBE_HANDLER_ACT_OPEN:
@@ -969,21 +945,23 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                         for (i = 0; i < pext->pdtbl->count; ++i) {
                                 pd  = pext->pdtbl->memb[i];
 
-                                //fprintf(stderr, "Sending reset to %p(%s)...\n", pd, oval_subtype2str(pd->subtype));
-
-
-				if (pd == NULL)
+				if (pd == NULL) {
+					va_end(ap);
 					return(0);
+				}
 
 				if (act == PROBE_HANDLER_ACT_RESET)
 					ret = oval_probe_ext_reset(pext->pdtbl->ctx, pd, pext);
 				else
 					ret = oval_probe_ext_abort(pext->pdtbl->ctx, pd, pext);
 
-				if (ret != 0)
+				if (ret != 0) {
+					va_end(ap);
 					return (ret);
+				}
                         }
 
+			va_end(ap);
                         return(0);
                 } else {
                         /*
@@ -991,7 +969,8 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                          */
                         pd = oval_pdtbl_get(pext->pdtbl, type);
 
-                        if (pd == NULL)
+			va_end(ap);
+                        if (pd == NULL) 
                                 return(0);
 
 			if (act == PROBE_HANDLER_ACT_RESET)
@@ -1036,24 +1015,34 @@ int oval_probe_ext_init(oval_pext_t *pext)
                         goto _ret;
 		}
 
-		pext->pdsc = oscap_alloc(sizeof(oval_pdsc_t) * DEFAULT_PDSC_COUNT);
+		pext->pdsc = oscap_alloc(sizeof(oval_pdsc_t) * OSCAP_GSYM(__probe_meta_count));
 
-		for (r = 0, i = 0; i < DEFAULT_PDSC_COUNT; ++i) {
-			if (stat((OSCAP_GSYM(default_pdsc)[i]).file, &st) != 0) {
-				dW("skipped: %s (stat failed, errno=%d)\n", OSCAP_GSYM(default_pdsc)[i].name, errno);
+                dI("__probe_meta_count = %zu\n", OSCAP_GSYM(__probe_meta_count));
+
+		for (r = 0, i = 0; i < OSCAP_GSYM(__probe_meta_count); ++i) {
+                        if (!(OSCAP_GSYM(__probe_meta)[i].flags & OVAL_PROBEMETA_EXTERNAL)) {
+                                dI("skipped: %s (not an external probe)\n", OSCAP_GSYM(__probe_meta)[i].stype);
+                                continue;
+                        }
+
+			if (stat(OSCAP_GSYM(__probe_meta)[i].pname, &st) != 0) {
+				dW("skipped: %s (stat failed, errno=%d)\n", OSCAP_GSYM(__probe_meta)[i].stype, errno);
 				continue;
 			}
 
 			if (!S_ISREG(st.st_mode)) {
-				dW("skipped: %s (not a regular file)\n", OSCAP_GSYM(default_pdsc)[i].name);
+				dW("skipped: %s (not a regular file)\n", OSCAP_GSYM(__probe_meta)[i].stype);
 				continue;
 			}
 
-			memcpy(pext->pdsc + r, OSCAP_GSYM(default_pdsc) + i, sizeof(oval_pdsc_t));
+                        pext->pdsc[r].type = OSCAP_GSYM(__probe_meta)[i].otype;
+                        pext->pdsc[r].name = OSCAP_GSYM(__probe_meta)[i].stype;
+                        pext->pdsc[r].file = OSCAP_GSYM(__probe_meta)[i].pname;
+
 			++r;
 		}
 
-		if (r < DEFAULT_PDSC_COUNT)
+		if (r < OSCAP_GSYM(__probe_meta_count))
 			pext->pdsc = oscap_realloc(pext->pdsc, sizeof(oval_pdsc_t) * r);
 
 		pext->pdsc_cnt = r;
@@ -1093,7 +1082,7 @@ int oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struc
 	}
 
 	object = oval_syschar_get_object(syschar);
-	ret = oval_object2sexp(pext->sess_ptr, oval_subtype2str(oval_object_get_subtype(object)), syschar, &s_obj);
+	ret = oval_object_to_sexp(pext->sess_ptr, oval_subtype_to_str(oval_object_get_subtype(object)), syschar, &s_obj);
 
 	if (ret != 0)
 		return (1);
@@ -1128,7 +1117,7 @@ int oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struc
         /*
 	 * Convert the received S-exp to OVAL system characteristic.
 	 */
-	ret = oval_sexp2sysch(s_sys, syschar);
+	ret = oval_sexp_to_sysch(s_sys, syschar);
 	SEXP_free(s_sys);
 
 	return (ret);
@@ -1185,3 +1174,18 @@ int oval_probe_ext_abort(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext)
 
 	return (0);
 }
+
+const char *oval_probe_ext_getdir(void)
+{
+    const char *probe_dir;
+
+#if defined(OVAL_PROBEDIR_ENV)
+    probe_dir = getenv("OVAL_PROBE_DIR");
+#else
+    probe_dir = NULL;
+#endif
+    if (probe_dir == NULL)
+	probe_dir = OVAL_PROBE_DIR;
+
+    return (probe_dir);
+}
