@@ -36,8 +36,8 @@ typedef struct {
 } oval_pd_t;
 
 typedef struct {
-	oval_pd_t *memb;
-	size_t count;
+	oval_pd_t **memb;
+	size_t      count;
 	SEAP_CTX_t *ctx;
 } oval_pdtbl_t;
 
@@ -68,12 +68,15 @@ typedef struct oval_pext oval_pext_t;
 oval_pext_t *oval_pext_new(void);
 void oval_pext_free(oval_pext_t *pext);
 int oval_probe_ext_init(oval_pext_t *pext);
-struct oval_syschar *oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *, struct oval_object *object, int flags);
+int oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struct oval_syschar *syschar, int flags);
 int oval_probe_ext_reset(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext);
+int oval_probe_ext_abort(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext);
 
 int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...);
 int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...);
 
 extern const oval_pdsc_t OSCAP_GSYM(default_pdsc)[];
 
+const char *oval_probe_ext_getdir(void);
+
 #endif /* OVAL_PROBE_EXT_H */
