@@ -60,16 +60,17 @@ OSCAP_HIDDEN_START;
 
 OSCAP_HIDDEN_END;
 
-extern probe_ncache_t *OSCAP_GSYM(encache);
+extern probe_ncache_t *OSCAP_GSYM(ncache);
 
 #endif /* ENABLE_PROBES */
 
 typedef struct {
         oval_subtype_t type;
-        char          *name;
+        const char    *name;
 } oval_subtypedsc_t;
 
-const char *oval_subtype2str(oval_subtype_t subtype);
-oval_subtype_t oval_str2subtype(const char *str);
+void oval_probe_tblinit(void);
+const char *oval_subtype_to_str(oval_subtype_t subtype);
+oval_subtype_t oval_str_to_subtype(const char *str);
 
 #endif /* OVAL_PROBE_IMPL_H */
