@@ -45,13 +45,12 @@
 #include "oval_probe_impl.h"
 #if defined(ENABLE_PROBES)
 # include "oval_probe_ext.h"
-# include "oval_probe_int.h"
-
+# include "oval_probe_meta.h"
 #if defined(OSCAP_THREAD_SAFE)
 # include <pthread.h>
-static pthread_once_t __encache_once = PTHREAD_ONCE_INIT;
+static pthread_once_t __oval_probe_session_init_once = PTHREAD_ONCE_INIT;
 #else
-static volatile int __encache_once = 0;
+static volatile int __oval_probe_session_init_once = 0;
 #endif
 
 /**
@@ -59,22 +58,28 @@ static volatile int __encache_once = 0;
  * atexit(3) during initialization and ensures that the element
  * name cache is freed before exit.
  */
-static void encache_libfree(void)
+static void ncache_libfree(void)
 {
-        probe_ncache_free(OSCAP_GSYM(encache));
+        probe_ncache_free(OSCAP_GSYM(ncache));
 }
 
 /**
  * Initialize element name cache and register an exit hook.
  */
-static void encache_libinit(void)
+static void ncache_libinit(void)
 {
-        if (OSCAP_GSYM(encache) == NULL) {
-                OSCAP_GSYM(encache) = probe_ncache_new();
-                atexit(encache_libfree);
+        if (OSCAP_GSYM(ncache) == NULL) {
+                OSCAP_GSYM(ncache) = probe_ncache_new();
+                atexit(ncache_libfree);
         }
 }
 
+static void oval_probe_session_libinit(void)
+{
+        ncache_libinit();
+        oval_probe_tblinit();
+}
+
 /**
  * Initialize the element name cache. This function can be called repeatedly
  * from various probe system entry points to ensure that the cache is initialized.
@@ -82,15 +87,16 @@ static void encache_libinit(void)
  * is used to ensure that the initialization is done just once. Otherwise a
  * volatile integer flag is used.
  */
-static void encache_once(void)
+static void __init_once(void)
 {
 #if defined(OSCAP_THREAD_SAFE)
-        if (pthread_once(&__encache_once, &encache_libinit) != 0)
+        if (pthread_once(&__oval_probe_session_init_once,
+			 &oval_probe_session_libinit) != 0)
                 abort();
 #else
-        if (__encache_once == 0) {
-                encache_libinit();
-                __encache_once = 1;
+        if (__oval_probe_session_init_once == 0) {
+                oval_probe_session_libinit();
+                __oval_probe_session_init_once = 1;
         }
 #endif
         return;
@@ -100,52 +106,46 @@ static void encache_once(void)
 oval_probe_session_t *oval_probe_session_new(struct oval_syschar_model *model)
 {
         oval_probe_session_t *sess;
-
+#if defined(ENABLE_PROBES)
+        void *handler_arg;
+        register size_t i;
+#endif
         sess = oscap_talloc(oval_probe_session_t);
         sess->ph = oval_phtbl_new();
         sess->sys_model = model;
         sess->flg = 0;
-
 #if defined(ENABLE_PROBES)
         sess->pext = oval_pext_new();
         sess->pext->model    = &sess->sys_model;
         sess->pext->sess_ptr = sess;
 
-        encache_once();
+        __init_once();
 
-        oval_probe_handler_set(sess->ph, OVAL_SUBTYPE_SYSINFO,       oval_probe_sys_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_FAMILY,    oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_FILE_MD5,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_FILE_HASH, oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_TEXT_FILE_CONTENT, oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_XML_FILE_CONTENT,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_LDAP57,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_SQL,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_SQL57,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_LINUX_INET_LISTENING_SERVERS,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_LINUX_DPKG_INFO, oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_LINUX_RPM_INFO,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_LINUX_PARTITION, oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_FILE,       oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_INTERFACE,  oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_PASSWORD,   oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_PROCESS,    oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_RUNLEVEL,   oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_SHADOW,     oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_UNAME,      oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_DNSCACHE,   oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_UNIX_SYSCTL,     oval_probe_ext_handler, sess->pext);
-        oval_probe_handler_set(sess->ph, OVAL_SUBTYPE_ALL,     oval_probe_ext_handler, sess->pext); /* special case for reset */
+        dI("__probe_meta_count = %zu\n", OSCAP_GSYM(__probe_meta_count));
 
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE,  oval_probe_envvar_handler, sess->sys_model);
-        oval_probe_handler_set(sess->ph, OVAL_INDEPENDENT_VARIABLE,              oval_probe_var_handler,    sess);
+        for (i = 0; i < OSCAP_GSYM(__probe_meta_count); ++i) {
+                handler_arg = NULL;
+
+                if (OSCAP_GSYM(__probe_meta)[i].flags & OVAL_PROBEMETA_EXTERNAL)
+                        handler_arg = sess->pext;
+
+                oval_probe_handler_set(sess->ph,
+				       OSCAP_GSYM(__probe_meta)[i].otype,
+				       OSCAP_GSYM(__probe_meta)[i].handler, handler_arg);
+        }
+
+        oval_probe_handler_set(sess->ph, OVAL_SUBTYPE_ALL, oval_probe_ext_handler, sess->pext); /* special case for reset */
 #endif
         return(sess);
 }
 
 void oval_probe_session_destroy(oval_probe_session_t *sess)
 {
+	if (sess == NULL) {
+		dE("Invalid session (NULL)\n");
+		return;
+	}
+
         oval_phtbl_free(sess->ph);
 #if defined(ENABLE_PROBES)
         oval_pext_free(sess->pext);
@@ -164,7 +164,10 @@ int oval_probe_session_reset(oval_probe_session_t *sess, struct oval_syschar_mod
 #if defined(ENABLE_PROBES)
         oval_ph_t *ph;
 
-        ph = oval_probe_handler_get(sess->ph, OVAL_SUBTYPE_ALL);
+        if ((ph = oval_probe_handler_get(sess->ph, OVAL_SUBTYPE_ALL)) == NULL) {
+		dE("No probe handler for OVAL_SUBTYPE_ALL\n");
+		return (-1);
+	}
 
         if (ph->func(OVAL_SUBTYPE_ALL, ph->uptr, PROBE_HANDLER_ACT_RESET) != 0) {
                 return(-1);
@@ -179,7 +182,13 @@ int oval_probe_session_reset(oval_probe_session_t *sess, struct oval_syschar_mod
 int oval_probe_session_abort(oval_probe_session_t *sess)
 {
 #if defined(ENABLE_PROBES)
-	oval_ph_t *ph = oval_probe_handler_get(sess->ph, OVAL_SUBTYPE_ALL);
+	oval_ph_t *ph;
+
+	if ((ph = oval_probe_handler_get(sess->ph, OVAL_SUBTYPE_ALL) ) == NULL) {
+		dE("No probe handler for OVAL_SUBTYPE_ALL\n");
+		return (-1);
+	}
+
         return ph->func(OVAL_SUBTYPE_ALL, ph->uptr, PROBE_HANDLER_ACT_ABORT);
 #else
 	return (-1);
@@ -188,10 +197,16 @@ int oval_probe_session_abort(oval_probe_session_t *sess)
 
 int oval_probe_session_sethandler(oval_probe_session_t *sess, oval_subtype_t type, oval_probe_handler_t handler, void *ptr)
 {
+	dE("Operation not supported\n");
         return(-1);
 }
 
 struct oval_syschar_model *oval_probe_session_getmodel(oval_probe_session_t *sess)
 {
-        return (sess->sys_model);
+	if (sess == NULL) {
+		dE("Invalid session (NULL)\n");
+		return (NULL);
+	}
+
+	return (sess->sys_model);
 }
