@@ -86,8 +86,10 @@ struct oval_result_criteria_node *oval_result_criteria_node_new(struct oval_resu
 			/*(NODETYPE_CRITERIA, negate, operator); */
 			node = (oval_result_criteria_node_t *)
 			    oscap_alloc(sizeof(oval_result_criteria_node_CRITERIA_t));
-			if (node == NULL)
+			if (node == NULL) {
+				va_end(ap);
 				return NULL;
+			}
 
 			oval_result_criteria_node_CRITERIA_t *criteria = (oval_result_criteria_node_CRITERIA_t *) node;
 			criteria->operator =(oval_operator_t) va_arg(ap, int);
@@ -98,8 +100,10 @@ struct oval_result_criteria_node *oval_result_criteria_node_new(struct oval_resu
 			/*(NODETYPE_CRITERION, negate, test, variable_instance); */
 			node = (oval_result_criteria_node_t *)
 			    oscap_alloc(sizeof(oval_result_criteria_node_CRITERION_t));
-			if (node == NULL)
+			if (node == NULL) {
+				va_end(ap);
 				return NULL;
+			}
 
 			oval_result_criteria_node_CRITERION_t *criterion =
 			    (oval_result_criteria_node_CRITERION_t *) node;
@@ -111,8 +115,10 @@ struct oval_result_criteria_node *oval_result_criteria_node_new(struct oval_resu
 			/*(NODETYPE_EXTENDDEF, negate, definition, variable_instance); */
 			node = (oval_result_criteria_node_t *)
 			    oscap_alloc(sizeof(oval_result_criteria_node_EXTENDDEF_t));
-			if (node == NULL)
+			if (node == NULL) {
+				va_end(ap);
 				return NULL;
+			}
 
 			oval_result_criteria_node_EXTENDDEF_t *extenddef =
 			    (oval_result_criteria_node_EXTENDDEF_t *) node;
@@ -454,34 +460,24 @@ static int _oval_result_criteria_parse(xmlTextReaderPtr reader, struct oval_pars
 
 int oval_result_criteria_node_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, struct oval_result_system *sys, oscap_consumer_func consumer, void *client) 
 {
-	int return_code = 0;
 	xmlChar *localName = xmlTextReaderLocalName(reader);
+	int rc = 0;
 
 	struct oval_result_criteria_node *node = NULL;
 	if (strcmp((const char *)localName, "criteria") == 0) {
 		oval_operator_t operator = oval_operator_parse(reader, "operator", OVAL_OPERATOR_UNKNOWN);
 		int negate = oval_parser_boolean_attribute(reader, "negate", false);
 		node = oval_result_criteria_node_new(sys, OVAL_NODETYPE_CRITERIA, negate, operator);
-		return_code = oval_parser_parse_tag(reader, context, _oval_result_criteria_parse, node);
+		rc = oval_parser_parse_tag(reader, context, _oval_result_criteria_parse, node);
 	} else if (strcmp((const char *)localName, "criterion") == 0) {
 		xmlChar *test_ref = xmlTextReaderGetAttribute(reader, BAD_CAST "test_ref");
-		int version = oval_parser_int_attribute(reader, "version", 0);
 		int variable_instance = oval_parser_int_attribute(reader, "variable_instance", 1);
 		int negate = oval_parser_boolean_attribute(reader, "negate", false);
 
 		struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
 		struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
-		struct oval_test *oval_test = oval_definition_model_get_test(definition_model, (char *)test_ref);
-		struct oval_result_test *rslt_test = NULL;
-
-		if (oval_test != NULL) {
-			int tst_ver;
-
-			rslt_test = oval_result_system_get_new_test(sys, oval_test);
-			tst_ver = oval_test_get_version(oval_test);
-			if (tst_ver != version)
-				dW("Unmatched versions: test: %d, criteria: %d.\n", tst_ver, version);
-		}
+		struct oval_test *oval_test = oval_definition_model_get_new_test(definition_model, (char *)test_ref);
+		struct oval_result_test *rslt_test = oval_result_system_get_new_test(sys, oval_test);
 
 		node = oval_result_criteria_node_new(sys, OVAL_NODETYPE_CRITERION, negate, rslt_test, variable_instance);
 		oscap_free(test_ref);
@@ -491,25 +487,28 @@ int oval_result_criteria_node_parse(xmlTextReaderPtr reader, struct oval_parser_
 		int negate = oval_parser_boolean_attribute(reader, "negate", false);
 		struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
 		struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
-		struct oval_definition *oval_definition = oval_definition_model_get_definition(definition_model, (char *)definition_ref);
-		struct oval_result_definition *rslt_definition = (oval_definition) ? oval_result_system_get_new_definition(sys, oval_definition) : NULL;
-		node = (rslt_definition) ? oval_result_criteria_node_new(sys, OVAL_NODETYPE_EXTENDDEF, negate, rslt_definition, variable_instance) : NULL;
+		struct oval_definition *oval_definition = oval_definition_model_get_new_definition(definition_model, (char *)definition_ref);
+		struct oval_result_definition *rslt_definition = oval_result_system_get_new_definition(sys, oval_definition);
+		node = oval_result_criteria_node_new(sys, OVAL_NODETYPE_EXTENDDEF, negate, rslt_definition, variable_instance);
 		oscap_free(definition_ref);
 	} else {
 		dW("unhandled criteria node: <%s>.\n",(char *)localName);
 		oval_parser_skip_tag(reader, context);
-		return_code = 1;
 	}
 
-	if (return_code==0) {
-		oval_result_t result = oval_result_parse(reader, "result", 0);
-		oval_result_criteria_node_set_result(node, result);
+	if (node==NULL || rc==-1) {
+		dE("Can't parse result criteria node.\n");
+		return 1;
 	}
 
+	oval_result_t result = oval_result_parse(reader, "result", 0);
+	oval_result_criteria_node_set_result(node, result);
+
 	(*consumer) (node, client);
 
+
 	oscap_free(localName);
-	return return_code;
+	return rc;
 }
 
 static xmlNode *_oval_result_CRITERIA_to_dom(struct oval_result_criteria_node *node, xmlDocPtr doc, xmlNode * parent) {
