@@ -36,6 +36,7 @@
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
+#include <arpa/inet.h>
 #if defined USE_REGEX_PCRE
 #include <pcre.h>
 #elif defined USE_REGEX_POSIX
@@ -202,7 +203,7 @@ struct oval_result_test *oval_result_test_new(struct oval_result_system *sys, ch
 	struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
 	struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
 	test->system = sys;
-	test->test = oval_test_get_new(definition_model, tstid);
+	test->test = oval_definition_model_get_new_test(definition_model, tstid);
 	test->messages = oval_collection_new();
 	test->result = OVAL_RESULT_NOT_EVALUATED;
 	test->instance = 1;
@@ -401,6 +402,230 @@ static int cmp_float(float a, float b)
 	return r;
 }
 
+static oval_result_t ipv4addr_cmp(char *s1, char *s2, oval_operation_t op)
+{
+	oval_result_t result = OVAL_RESULT_ERROR;
+	char *s, *pfx;
+	int nm1, nm2;
+	struct in_addr addr1, addr2;
+
+	s = strdup(s1);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		int cnt;
+		char nm[4];
+
+		*pfx++ = '\0';
+		cnt = sscanf(pfx, "%hhu.%hhu.%hhu.%hhu", &nm[0], &nm[1], &nm[2], &nm[3]);
+		if (cnt > 1) { /* netmask */
+			nm1 = (nm[0] << 24) + (nm[1] << 16) + (nm[2] << 8) + nm[3];
+		} else { /* prefix */
+			nm1 = (~0) << (32 - nm[0]);
+		}
+	} else {
+		nm1 = ~0;
+	}
+	if (inet_pton(AF_INET, s, &addr1) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	oscap_free(s);
+	s = strdup(s2);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		int cnt;
+		char nm[4];
+
+		*pfx++ = '\0';
+		cnt = sscanf(pfx, "%hhu.%hhu.%hhu.%hhu", &nm[0], &nm[1], &nm[2], &nm[3]);
+		if (cnt > 1) { /* netmask */
+			nm2 = (nm[0] << 24) + (nm[1] << 16) + (nm[2] << 8) + nm[3];
+		} else { /* prefix */
+			nm2 = (~0) << (32 - nm[0]);
+		}
+	} else {
+		nm2 = ~0;
+	}
+	if (inet_pton(AF_INET, s, &addr2) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	switch (op) {
+	case OVAL_OPERATION_EQUALS:
+		if (!memcmp(&addr1, &addr2, sizeof(struct in_addr)) && nm1 == nm2)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_NOT_EQUAL:
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) || nm1 != nm2)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUBSET_OF:
+		if (nm1 <= nm2) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_GREATER_THAN:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) > 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_GREATER_THAN_OR_EQUAL:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) >= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUPERSET_OF:
+		if (nm1 >= nm2) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_LESS_THAN:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) < 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_LESS_THAN_OR_EQUAL:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) <= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	default:
+		dE("Unexpected compare operation: %d.\n", op);
+	}
+
+ cleanup:
+	oscap_free(s);
+
+	return OVAL_RESULT_ERROR;
+}
+
+static void mask_v6_addrs(struct in6_addr *addr1, int p1len, struct in6_addr *addr2, int p2len)
+{
+	int i;
+
+	for (i = 0; i < 128; ++i) {
+		addr1->s6_addr[i / 8] &= (((i < p1len) ? 1 : 0) << (i % 8));
+		addr2->s6_addr[i / 8] &= (((i < p2len) ? 1 : 0) << (i % 8));
+	}
+}
+
+static oval_result_t ipv6addr_cmp(char *s1, char *s2, oval_operation_t op)
+{
+	oval_result_t result = OVAL_RESULT_ERROR;
+	char *s, *pfx;
+	int p1len, p2len;
+	struct in6_addr addr1, addr2;
+
+	s = strdup(s1);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		*pfx++ = '\0';
+		p1len = strtol(pfx, NULL, 10);
+	} else {
+		p1len = 128;
+	}
+	if (inet_pton(AF_INET6, s, &addr1) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	oscap_free(s);
+	s = strdup(s2);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		*pfx++ = '\0';
+		p2len = strtol(pfx, NULL, 10);
+	} else {
+		p2len = 128;
+	}
+	if (inet_pton(AF_INET6, s, &addr2) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	switch (op) {
+	case OVAL_OPERATION_EQUALS:
+		if (!memcmp(&addr1, &addr2, sizeof(struct in6_addr)) && p1len == p2len)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_NOT_EQUAL:
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) || p1len != p2len)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUBSET_OF:
+		if (p1len <= p2len) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_GREATER_THAN:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) > 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_GREATER_THAN_OR_EQUAL:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) >= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUPERSET_OF:
+		if (p1len >= p2len) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_LESS_THAN:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) < 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_LESS_THAN_OR_EQUAL:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) <= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	default:
+		dE("Unexpected compare operation: %d.\n", op);
+	}
+
+ cleanup:
+	oscap_free(s);
+
+	return result;
+}
+
 // finally, we have gotten to the point of comparing system data with a state
 static oval_result_t evaluate(char *sys_data, char *state_data, oval_datatype_t sys_data_type,
 			      oval_datatype_t state_data_type, oval_operation_t operation)
@@ -581,6 +806,10 @@ static oval_result_t evaluate(char *sys_data, char *state_data, oval_datatype_t
 		} else if (operation == OVAL_OPERATION_LESS_THAN_OR_EQUAL) {
 			return (OVAL_RESULT_TRUE);
 		}		// we have already filtered out the invalid ones
+	} else if (state_data_type == OVAL_DATATYPE_IPV4ADDR) {
+		return ipv4addr_cmp(state_data, sys_data, operation);
+	} else if (state_data_type == OVAL_DATATYPE_IPV6ADDR) {
+		return ipv6addr_cmp(state_data, sys_data, operation);
 	} else if (state_data_type == OVAL_DATATYPE_FILESET_REVISION
 		   || state_data_type == OVAL_DATATYPE_IOS_VERSION) {
 		dW("Unsupported data type: %s.\n", oval_datatype_get_text(state_data_type));
@@ -674,6 +903,10 @@ oval_result_t ores_get_result_bychk(struct oresults *ores, oval_check_t check)
 			result = OVAL_RESULT_NOT_EVALUATED;
 		}
 		break;
+	case OVAL_CHECK_NONE_EXIST:
+		dW("The 'none exist' CheckEnumeration value has been deprecated. "
+		   "Converted to check='none satisfy'.\n");
+		/* FALLTHROUGH */
 	case OVAL_CHECK_NONE_SATISFY:
 		if (ores->true_cnt > 0) {
 			result = OVAL_RESULT_FALSE;
@@ -840,6 +1073,7 @@ static oval_result_t eval_item(struct oval_syschar_model *syschar_model, struct
 		oval_result_t ste_ent_res;
 		struct oval_sysent_iterator *item_entities_itr;
 		struct oresults ent_ores;
+		bool found_matching_item;
 
 		if ((content = oval_state_content_iterator_next(state_contents_itr)) == NULL) {
 			oscap_dlprintf(DBG_E, "Found NULL state content.\n");
@@ -890,6 +1124,7 @@ static oval_result_t eval_item(struct oval_syschar_model *syschar_model, struct
 		}
 
 		ores_clear(&ent_ores);
+		found_matching_item = false;
 
 		item_entities_itr = oval_sysitem_get_sysents(cur_sysitem);
 		while (oval_sysent_iterator_has_more(item_entities_itr)) {
@@ -910,6 +1145,8 @@ static oval_result_t eval_item(struct oval_syschar_model *syschar_model, struct
 			if (strcmp(item_entity_name, state_entity_name))
 				continue;
 
+			found_matching_item = true;
+
 			/* copy mask attribute from state to item */
 			if (oval_entity_get_mask(state_entity))
 				oval_sysent_set_mask(item_entity,1);
@@ -977,6 +1214,10 @@ static oval_result_t eval_item(struct oval_syschar_model *syschar_model, struct
 		}
 		oval_sysent_iterator_free(item_entities_itr);
 
+		if (!found_matching_item)
+			dW("Entity name '%s' from state (id: '%s') not found in item (id: '%s').\n",
+			   state_entity_name, oval_state_get_id(state), oval_sysitem_get_id(cur_sysitem));
+
 		ste_ent_res = ores_get_result_bychk(&ent_ores, entity_check);
 		ores_add_res(&ste_ores, ste_ent_res);
 	}
@@ -1339,6 +1580,8 @@ oval_result_t oval_result_test_eval(struct oval_result_test *rtest)
 			rtest->result = OVAL_RESULT_UNKNOWN;
 	}
 
+        dI("\t%s => %s\n", oval_test_get_id(oval_result_test_get_test(rtest)), oval_result_get_text(rtest->result));
+
 	return rtest->result;
 }
 
@@ -1423,7 +1666,7 @@ static int _oval_result_test_binding_parse(xmlTextReaderPtr reader, struct oval_
 	struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(SYSTEM);
 	struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
 
-	struct oval_variable *variable = oval_variable_get_new
+	struct oval_variable *variable = oval_definition_model_get_new_variable
 	    (definition_model, (char *)variable_id, OVAL_VARIABLE_UNKNOWN);
 
 	xmlChar *value = xmlTextReaderValue(reader);
@@ -1467,7 +1710,7 @@ int oval_result_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_conte
 	xmlChar *test_id = xmlTextReaderGetAttribute(reader, BAD_CAST "test_id");
 
 	dmod = context->definition_model;
-	dtst = oval_test_get_new(dmod, (char *) test_id);
+	dtst = oval_definition_model_get_new_test(dmod, (char *) test_id);
 	test = oval_result_system_get_new_test(sys, dtst);
 	if (test == NULL)
 		return -1;
