@@ -282,7 +282,7 @@ static void oval_consume_object_ref(char *objref, void *user)
 
 	struct oval_set_context *ctx = user;
 	struct oval_definition_model *model = ctx->context->definition_model;
-	struct oval_object *object = oval_object_get_new(model, objref);
+	struct oval_object *object = oval_definition_model_get_new_object(model, objref);
 	oval_setobject_add_object(ctx->set, object);
 }
 
@@ -357,10 +357,6 @@ int oval_set_parse_tag(xmlTextReaderPtr reader,
 xmlNode *oval_set_to_dom(struct oval_setobject *set, xmlDoc * doc, xmlNode * parent) {
 	xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
 	xmlNode *set_node = xmlNewTextChild(parent, ns_definitions, BAD_CAST "set", NULL);
-	if (ns_definitions == NULL) {
-		ns_definitions = xmlNewNs(set_node, OVAL_DEFINITIONS_NAMESPACE, NULL);
-		xmlSetNs(set_node, ns_definitions);
-	}
 
 	oval_setobject_operation_t operation = oval_setobject_get_operation(set);
 	if (operation != OVAL_SET_OPERATION_UNION)
