@@ -59,30 +59,13 @@ SEXP_t *oval_value_to_sexp(struct oval_value *val, oval_datatype_t dtype)
 	SEXP_t *val_sexp = NULL;
         char   *val_rptr = NULL;
 
-	/* todo: remove calls to SEXP_datatype_set() */
-
 	switch (dtype) {
+	case OVAL_DATATYPE_EVR_STRING:
+	case OVAL_DATATYPE_IPV4ADDR:
+	case OVAL_DATATYPE_IPV6ADDR:
+	case OVAL_DATATYPE_STRING:
 	case OVAL_DATATYPE_VERSION:
                 val_rptr = oval_value_get_text (val);
-
-                if (val_rptr != NULL) {
-                        val_sexp = SEXP_string_newf("%s", val_rptr);
-                        SEXP_datatype_set(val_sexp, "version");
-                }
-
-		break;
-        case OVAL_DATATYPE_EVR_STRING:
-                val_rptr = oval_value_get_text (val);
-
-                if (val_rptr != NULL) {
-                        val_sexp = SEXP_string_newf("%s", val_rptr);
-                        SEXP_datatype_set(val_sexp, "evr_str");
-                }
-
-		break;
-	case OVAL_DATATYPE_STRING:
-                val_rptr = oval_value_get_text (val);
-
                 if (val_rptr != NULL) {
                         val_sexp = SEXP_string_newf("%s", val_rptr);
                 }
@@ -120,44 +103,42 @@ static SEXP_t *oval_entity_to_sexp(struct oval_entity *ent)
 	SEXP_t *elm, *elm_name;
 	SEXP_t *r0, *r1, *r2;
 	oval_datatype_t datatype;
+	oval_entity_varref_type_t vr_type;
 
 	elm_name = SEXP_list_new(r0 = SEXP_string_newf("%s", oval_entity_get_name(ent)),
 				 /* operation */
 				 r1 = SEXP_string_new(":operation", 10),
 				 r2 = SEXP_number_newu_32(oval_entity_get_operation(ent)), NULL);
-
-	SEXP_free(r0);
-	SEXP_free(r1);
-	SEXP_free(r2);
+	SEXP_vfree(r0, r1, r2, NULL);
 
 	elm = SEXP_list_new(NULL);
 	datatype = oval_entity_get_datatype(ent);
 	probe_ent_setdatatype(elm, datatype);
 
-	if (oval_entity_get_varref_type(ent) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
+	vr_type = oval_entity_get_varref_type(ent);
+	if (vr_type == OVAL_ENTITY_VARREF_ATTRIBUTE
+	    || vr_type == OVAL_ENTITY_VARREF_ELEMENT) {
 		/* var_ref */
 		struct oval_variable *var;
 
 		var = oval_entity_get_variable(ent);
-
 		SEXP_list_add(elm_name, r0 = SEXP_string_new(":var_ref", 8));
-		SEXP_free(r0);
-
-		SEXP_list_add(elm_name, r0 = SEXP_string_newf("%s", oval_variable_get_id(var)));
-		SEXP_free(r0);
-
+		SEXP_list_add(elm_name, r1 = SEXP_string_newf("%s", oval_variable_get_id(var)));
 		SEXP_list_add(elm, elm_name);
-		SEXP_free(elm_name);
-
+		SEXP_vfree(r0, r1, elm_name, NULL);
 	} else {
 		/* value */
+		struct oval_value *val;
+
 		SEXP_list_add(elm, elm_name);
 		SEXP_free(elm_name);
+		val = oval_entity_get_value(ent);
 
-		if (datatype != OVAL_DATATYPE_RECORD) {
+		if (datatype != OVAL_DATATYPE_RECORD
+		    && val != NULL) {
 			SEXP_t *val_sexp;
 
-			val_sexp = oval_value_to_sexp(oval_entity_get_value(ent), datatype);
+			val_sexp = oval_value_to_sexp(val, datatype);
 			if (val_sexp != NULL) {
 				SEXP_list_add(elm, val_sexp);
 				SEXP_free(val_sexp);
@@ -168,7 +149,7 @@ static SEXP_t *oval_entity_to_sexp(struct oval_entity *ent)
 	return (elm);
 }
 
-static int oval_varref_to_sexp(void *sess, struct oval_entity *entity, struct oval_syschar *syschar, SEXP_t **out_sexp)
+static int oval_varref_attr_to_sexp(void *sess, struct oval_entity *entity, struct oval_syschar *syschar, SEXP_t **out_sexp)
 {
 	unsigned int val_cnt = 0;
 	SEXP_t *val_lst, *val_sexp, *varref, *id_sexp, *val_cnt_sexp;
@@ -193,6 +174,7 @@ static int oval_varref_to_sexp(void *sess, struct oval_entity *entity, struct ov
 		vit = oval_variable_get_values(var);
 		if (oval_value_iterator_has_more(vit))
 			break;
+		oval_value_iterator_free(vit);
 		/* fall through */
 	case SYSCHAR_FLAG_DOES_NOT_EXIST:
 		snprintf(msg, sizeof(msg), "Referenced variable has no values (%s).", oval_variable_get_id(var));
@@ -244,6 +226,50 @@ static int oval_varref_to_sexp(void *sess, struct oval_entity *entity, struct ov
 	return 0;
 }
 
+static int oval_varref_elm_to_sexp(void *sess, struct oval_variable *var, oval_datatype_t dt, SEXP_t **out_sexp)
+{
+	SEXP_t *val_lst;
+	struct oval_value_iterator *val_itr;
+	oval_syschar_collection_flag_t flag;
+
+	if (oval_probe_query_variable(sess, var) != 0)
+		return -1;
+
+	flag = oval_variable_get_collection_flag(var);
+	if (flag != SYSCHAR_FLAG_COMPLETE
+	    && flag != SYSCHAR_FLAG_INCOMPLETE) {
+		*out_sexp = SEXP_list_new(NULL);
+		return 0;
+	}
+
+	val_lst = SEXP_list_new(NULL);
+
+	val_itr = oval_variable_get_values(var);
+	while (oval_value_iterator_has_more(val_itr)) {
+		struct oval_value *val;
+		SEXP_t *vs;
+
+		val = oval_value_iterator_next(val_itr);
+		vs = oval_value_to_sexp(val, dt);
+		if (vs == NULL) {
+			oscap_dlprintf(DBG_E, "Failed to convert OVAL value to SEXP: "
+				       "datatype: %s, text: %s.\n",
+				       oval_datatype_get_text(dt),
+				       oval_value_get_text(val));
+			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Failed to convert OVAL value to SEXP");
+			oval_value_iterator_free(val_itr);
+			SEXP_free(val_lst);
+			return -1;
+		}
+		SEXP_list_add(val_lst, vs);
+		SEXP_free(vs);
+	}
+	oval_value_iterator_free(val_itr);
+
+	*out_sexp = val_lst;
+	return 0;
+}
+
 static SEXP_t *oval_filter_to_sexp(struct oval_filter *filter)
 {
 	SEXP_t *elm, *attr, *r0, *r1;
@@ -367,7 +393,7 @@ static SEXP_t *oval_behaviors_to_sexp(struct oval_behavior_iterator *bit)
 	return (r0);
 }
 
-int oval_object2sexp(void *sess, const char *typestr, struct oval_syschar *syschar, SEXP_t **out_sexp)
+int oval_object_to_sexp(void *sess, const char *typestr, struct oval_syschar *syschar, SEXP_t **out_sexp)
 {
 	unsigned int ent_cnt, varref_cnt;
 	int ret = -1;
@@ -407,6 +433,7 @@ int oval_object2sexp(void *sess, const char *typestr, struct oval_syschar *sysch
 	cit = oval_object_get_object_contents(object);
 	while (oval_object_content_iterator_has_more(cit)) {
 		oval_check_t ochk;
+		oval_entity_varref_type_t vr_type;
 
 		content = oval_object_content_iterator_next(cit);
 		elm = NULL;
@@ -416,7 +443,6 @@ int oval_object2sexp(void *sess, const char *typestr, struct oval_syschar *sysch
 		case OVAL_OBJECTCONTENT_ENTITY:
 			entity = oval_object_content_get_entity(content);
 			elm = oval_entity_to_sexp(entity);
-
 			if (elm == NULL)
 				break;
 
@@ -427,9 +453,9 @@ int oval_object2sexp(void *sess, const char *typestr, struct oval_syschar *sysch
 				SEXP_free(r0);
 			}
 
-			if (oval_entity_get_varref_type(entity) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
-				ret = oval_varref_to_sexp(sess, entity, syschar, &stmp);
-
+			vr_type = oval_entity_get_varref_type(entity);
+			if (vr_type == OVAL_ENTITY_VARREF_ATTRIBUTE) {
+				ret = oval_varref_attr_to_sexp(sess, entity, syschar, &stmp);
 				if (ret != 0) {
 					SEXP_free(elm);
 					elm = NULL;
@@ -446,6 +472,23 @@ int oval_object2sexp(void *sess, const char *typestr, struct oval_syschar *sysch
 
 				lst = obj_sexp;
 				++ent_cnt;
+			} else if (vr_type == OVAL_ENTITY_VARREF_ELEMENT) {
+				SEXP_t *val_lst;
+				struct oval_variable *var;
+				oval_datatype_t dt;
+
+				var = oval_entity_get_variable(entity);
+				dt = oval_entity_get_datatype(entity);
+
+				ret = oval_varref_elm_to_sexp(sess, var, dt, &val_lst);
+				if (ret != 0) {
+					SEXP_free(elm);
+					elm = NULL;
+					break;
+				}
+
+				SEXP_list_add(elm, val_lst);
+				SEXP_free(val_lst);
 			}
 			break;
 
@@ -550,14 +593,14 @@ static SEXP_t *oval_record_field_STATE_to_sexp(struct oval_record_field *rf)
 	return rf_sexp;
 }
 
-static struct oval_sysent *oval_sysent_from_sexp(struct oval_syschar_model *model, SEXP_t * sexp);
+static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model, struct oval_sysitem *item, SEXP_t * sexp);
 
 static struct oval_record_field *oval_record_field_ITEM_from_sexp(SEXP_t *sexp)
 {
 	struct oval_sysent *sysent;
 	struct oval_record_field *rf;
 
-	sysent = oval_sysent_from_sexp(NULL, sexp);
+	sysent = oval_sexp_to_sysent(NULL, NULL, sexp);
 	if (sysent == NULL)
 		return NULL;
 
@@ -573,7 +616,7 @@ static struct oval_record_field *oval_record_field_ITEM_from_sexp(SEXP_t *sexp)
 	return rf;
 }
 
-int oval_state2sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
+int oval_state_to_sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
 {
 	SEXP_t *ste, *ste_name, *ste_ent;
 	SEXP_t *r0, *r1, *r2, *r3, *r4;
@@ -582,7 +625,7 @@ int oval_state2sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
 	const char *subtype_name;
 	struct oval_state_content_iterator *contents;
 
-        subtype_name = oval_subtype2str(oval_state_get_subtype(state));
+        subtype_name = oval_subtype_to_str(oval_state_get_subtype(state));
 
 	if (subtype_name == NULL) {
 		_D("FAIL: unknown subtype: %d\n", oval_state_get_subtype(state));
@@ -605,6 +648,7 @@ int oval_state2sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
 	contents = oval_state_get_contents(state);
 	while (oval_state_content_iterator_has_more(contents)) {
 		oval_check_t ochk;
+		oval_entity_varref_type_t vr_type;
 		struct oval_entity *ent;
 		struct oval_state_content *content = oval_state_content_iterator_next(contents);
 		struct oval_record_field_iterator *rf_itr;
@@ -640,46 +684,18 @@ int oval_state2sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
 			SEXP_free(r0);
 		}
 
-		if (oval_entity_get_varref_type(ent) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
+		vr_type = oval_entity_get_varref_type(ent);
+		if (vr_type == OVAL_ENTITY_VARREF_ATTRIBUTE
+		    || vr_type == OVAL_ENTITY_VARREF_ELEMENT) {
 			SEXP_t *val_lst;
 			struct oval_variable *var;
-			struct oval_value_iterator *val_itr;
 			oval_datatype_t dt;
-			oval_syschar_collection_flag_t flag;
 
 			var = oval_entity_get_variable(ent);
-			if (oval_probe_query_variable(sess, var) != 0) {
-				goto fail;
-			}
-
 			dt = oval_entity_get_datatype(ent);
-			val_lst = SEXP_list_new(NULL);
 
-			flag = oval_variable_get_collection_flag(var);
-			if (flag == SYSCHAR_FLAG_COMPLETE
-			    || flag == SYSCHAR_FLAG_INCOMPLETE) {
-				val_itr = oval_variable_get_values(var);
-				while (oval_value_iterator_has_more(val_itr)) {
-					struct oval_value *val;
-					SEXP_t *vs;
-
-					val = oval_value_iterator_next(val_itr);
-					vs = oval_value_to_sexp(val, dt);
-					if (vs == NULL) {
-						oscap_dlprintf(DBG_E, "Failed to convert OVAL value to SEXP: "
-							       "datatype: %s, text: %s.\n",
-							       oval_datatype_get_text(dt),
-							       oval_value_get_text(val));
-						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Failed to convert OVAL value to SEXP");
-						oval_value_iterator_free(val_itr);
-						SEXP_free(val_lst);
-						goto fail;
-					}
-					SEXP_list_add(val_lst, vs);
-					SEXP_free(vs);
-				}
-				oval_value_iterator_free(val_itr);
-			}
+			if (oval_varref_elm_to_sexp(sess, var, dt, &val_lst) != 0)
+				goto fail;
 
 			SEXP_list_add(ste_ent, val_lst);
 			SEXP_free(val_lst);
@@ -699,7 +715,7 @@ int oval_state2sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
 	return (-1);
 }
 
-static struct oval_message *oval_sexp2msg(const SEXP_t *msg)
+static struct oval_message *oval_sexp_to_msg(const SEXP_t *msg)
 {
 	struct oval_message *message;
 	SEXP_t *r0;
@@ -720,7 +736,7 @@ static struct oval_message *oval_sexp2msg(const SEXP_t *msg)
 	return message;
 }
 
-static struct oval_sysent *oval_sysent_from_sexp(struct oval_syschar_model *model, SEXP_t * sexp)
+static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model, struct oval_sysitem *item, SEXP_t * sexp)
 {
 	char *key;
 	oval_syschar_status_t status;
@@ -731,6 +747,31 @@ static struct oval_sysent *oval_sysent_from_sexp(struct oval_syschar_model *mode
 	if (!key)
 		return NULL;
 
+	if (strcmp("message", key) == 0 && item != NULL) {
+	    struct oval_message *msg;
+	    oval_message_level_t lvl;
+	    SEXP_t *lvl_sexp, *txt_sexp;
+	    char txt[1024];
+
+	    lvl_sexp = probe_obj_getattrval(sexp, "level");
+	    lvl = SEXP_number_getu_32(lvl_sexp);
+
+	    txt_sexp = probe_ent_getval(sexp);
+	    SEXP_string_cstr_r(txt_sexp, txt, sizeof txt);
+
+	    SEXP_vfree(lvl_sexp, txt_sexp);
+
+	    /* TODO: sanity checks */
+
+	    msg = oval_message_new();
+
+	    oval_message_set_level(msg, lvl);
+	    oval_message_set_text(msg, txt);
+	    oval_sysitem_add_message(item, msg);
+
+	    return (NULL);
+	}
+
 	status = probe_ent_getstatus(sexp);
 	dt = probe_ent_getdatatype(sexp);
 
@@ -740,7 +781,7 @@ static struct oval_sysent *oval_sysent_from_sexp(struct oval_syschar_model *mode
 	oval_sysent_set_datatype(ent, dt);
 	oval_sysent_set_mask(ent, probe_ent_getmask(sexp));
 
-	if (status != OVAL_STATUS_EXISTS)
+	if (status != SYSCHAR_STATUS_EXISTS)
 		return ent;
 
 	if (dt == OVAL_DATATYPE_RECORD) {
@@ -791,8 +832,11 @@ static struct oval_sysent *oval_sysent_from_sexp(struct oval_syschar_model *mode
 				break;
 			}
 			break;
+		case OVAL_DATATYPE_EVR_STRING:
+		case OVAL_DATATYPE_IPV4ADDR:
+		case OVAL_DATATYPE_IPV6ADDR:
 		case OVAL_DATATYPE_STRING:
-                case OVAL_DATATYPE_EVR_STRING:
+		case OVAL_DATATYPE_VERSION:
 			valp = SEXP_string_cstr(sval);
 			break;
 		default:
@@ -811,20 +855,31 @@ static struct oval_sysent *oval_sysent_from_sexp(struct oval_syschar_model *mode
 	return ent;
 }
 
-static struct oval_sysitem *oval_sysitem_from_sexp(struct oval_syschar_model *model, SEXP_t * sexp)
+static struct oval_sysitem *oval_sexp_to_sysitem(struct oval_syschar_model *model, SEXP_t * sexp)
 {
 	_A(sexp);
 
-	static int id_counter = 1;	/* TODO better ID generator */
-
-	char *name;
+	char *name, *id;
+	SEXP_t *id_sexp;
 	struct oval_sysitem *sysitem = NULL;
 
+	id_sexp = probe_ent_getattrval(sexp, "id");
+	id = SEXP_string_cstr(id_sexp);
+	SEXP_free(id_sexp);
+
+	sysitem = oval_syschar_model_get_sysitem(model, id);
+
+	if (sysitem) {
+                oscap_free(id);
+		return sysitem;
+        }
+
 	name = probe_ent_getname(sexp);
 
-	if (name == NULL)
+	if (name == NULL) {
+                oscap_free(id);
 		return NULL;
-	else {
+        } else {
 		char *endptr = strrchr(name, '_');
 
 		if (strcmp(endptr, "_item") != 0)
@@ -833,36 +888,35 @@ static struct oval_sysitem *oval_sysitem_from_sexp(struct oval_syschar_model *mo
 		*endptr = '\0';	// cut off the '_item' part
 	}
 
-	int type = oval_str2subtype(name);
+	int type = oval_str_to_subtype(name);
 
 	_D("Syschar entry type: %d '%s' => %s\n", type, name, (type ? "OK" : "FAILED to decode"));
-
-	char id[16];
+#ifndef NDEBUG
+	if (!type)
+		abort();
+#endif
 	SEXP_t *sub;
 	struct oval_sysent *sysent;
 
 	int status = probe_ent_getstatus(sexp);
 
-	sprintf(id, "%d", id_counter++);
-	sysitem = oval_sysitem_get_new(model, id);
+	sysitem = oval_sysitem_new(model, id);
 	oval_sysitem_set_status(sysitem, status);
 	oval_sysitem_set_subtype(sysitem, type);
-	//oval_sysitem_set_subtype_name(sysitem, name);
 
-	if (status == OVAL_STATUS_EXISTS) {
-		for (int i = 2; (sub = SEXP_list_nth(sexp, i)) != NULL; ++i) {
-			if ((sysent = oval_sysent_from_sexp(model, sub)) != NULL)
-				oval_sysitem_add_sysent(sysitem, sysent);
-			SEXP_free(sub);
-		}
+	for (int i = 2; (sub = SEXP_list_nth(sexp, i)) != NULL; ++i) {
+	    if ((sysent = oval_sexp_to_sysent(model, sysitem, sub)) != NULL)
+		    oval_sysitem_add_sysent(sysitem, sysent);
+		SEXP_free(sub);
 	}
 
  cleanup:
+        oscap_free(id);
 	oscap_free(name);
 	return sysitem;
 }
 
-int oval_sexp2sysch(const SEXP_t *cobj, struct oval_syschar *syschar)
+int oval_sexp_to_sysch(const SEXP_t *cobj, struct oval_syschar *syschar)
 {
 	oval_syschar_collection_flag_t flag;
 	SEXP_t *messages, *msg, *items, *item;
@@ -877,7 +931,7 @@ int oval_sexp2sysch(const SEXP_t *cobj, struct oval_syschar *syschar)
 	SEXP_list_foreach(msg, messages) {
 		struct oval_message *omsg;
 
-		omsg = oval_sexp2msg(msg);
+		omsg = oval_sexp_to_msg(msg);
 		if (omsg != NULL)
 			oval_syschar_add_message(syschar, omsg);
 	}
@@ -888,7 +942,7 @@ int oval_sexp2sysch(const SEXP_t *cobj, struct oval_syschar *syschar)
 	SEXP_list_foreach(item, items) {
 		struct oval_sysitem *sysitem;
 
-		sysitem = oval_sysitem_from_sexp(model, item);
+		sysitem = oval_sexp_to_sysitem(model, item);
 		if (sysitem != NULL)
 			oval_syschar_add_sysitem(syschar, sysitem);
 	}
