@@ -250,7 +250,7 @@ int oval_syschar_model_import(struct oval_syschar_model *model, const char *file
         context.syschar_model = model;
         context.user_data = NULL;
 	xmlTextReaderSetErrorHandler(reader, &libxml_error_handler, &context);
-	/* jump into oval_definitions */
+	/* jump into oval_system_characteristics */
 	xmlTextReaderRead(reader);
 	/* make sure this is syschar */
 	char *tagname = (char *)xmlTextReaderLocalName(reader);
@@ -295,7 +295,7 @@ struct oval_sysitem *oval_syschar_model_get_sysitem(struct oval_syschar_model *m
 }
 
 
-struct oval_syschar *oval_syschar_get_new(struct oval_syschar_model *model, struct oval_object *object)
+struct oval_syschar *oval_syschar_model_get_new_syschar(struct oval_syschar_model *model, struct oval_object *object)
 {
 	char *object_id = oval_object_get_id(object);
 	struct oval_syschar *syschar = oval_syschar_model_get_syschar(model, object_id);
@@ -305,7 +305,7 @@ struct oval_syschar *oval_syschar_get_new(struct oval_syschar_model *model, stru
 	return syschar;
 }
 
-struct oval_sysitem *oval_sysitem_get_new(struct oval_syschar_model *model, const char *id)
+struct oval_sysitem *oval_syschar_model_get_new_sysitem(struct oval_syschar_model *model, const char *id)
 {
 	struct oval_sysitem *sysitem = oval_syschar_model_get_sysitem(model, id);
 	if (sysitem == NULL) {
@@ -330,10 +330,16 @@ xmlNode *oval_syschar_model_to_dom(struct oval_syschar_model * syschar_model, xm
 
 	xmlNs *ns_common = xmlNewNs(root_node, OVAL_COMMON_NAMESPACE, BAD_CAST "oval");
 	xmlNs *ns_xsi = xmlNewNs(root_node, OVAL_XMLNS_XSI, BAD_CAST "xsi");
+	xmlNs *ns_unix = xmlNewNs(root_node, OVAL_SYSCHAR_UNIX_NS, BAD_CAST "unix-sys");
+	xmlNs *ns_ind = xmlNewNs(root_node, OVAL_SYSCHAR_IND_NS, BAD_CAST "ind-sys");
+	xmlNs *ns_lin = xmlNewNs(root_node, OVAL_SYSCHAR_LIN_NS, BAD_CAST "lin-sys");
 	xmlNs *ns_syschar = xmlNewNs(root_node, OVAL_SYSCHAR_NAMESPACE, NULL);
 
 	xmlSetNs(root_node, ns_common);
 	xmlSetNs(root_node, ns_xsi);
+	xmlSetNs(root_node, ns_unix);
+	xmlSetNs(root_node, ns_ind);
+	xmlSetNs(root_node, ns_lin);
 	xmlSetNs(root_node, ns_syschar);
 
         /* Always report the generator */
