@@ -329,7 +329,7 @@ static int _oval_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_cont
 		char *object_ref = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "object_ref");
 		if (object_ref != NULL) {
 			struct oval_definition_model *model = context->definition_model;
-			struct oval_object *object = oval_object_get_new(model, object_ref);
+			struct oval_object *object = oval_definition_model_get_new_object(model, object_ref);
 			oscap_free(object_ref);
 			object_ref = NULL;
 			oval_test_set_object(test, object);
@@ -338,7 +338,7 @@ static int _oval_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_cont
 		char *state_ref = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "state_ref");
 		if (state_ref != NULL) {
 			struct oval_definition_model *model = context->definition_model;
-			struct oval_state *state = oval_state_get_new(model, state_ref);
+			struct oval_state *state = oval_definition_model_get_new_state(model, state_ref);
 			oval_test_add_state(test, state);
 			oscap_free(state_ref);
 			state_ref = NULL;
@@ -361,7 +361,7 @@ int oval_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *con
 	struct oval_definition_model *model = context->definition_model;
 
 	char *id = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
-	struct oval_test *test = oval_test_get_new(model, id);
+	struct oval_test *test = oval_definition_model_get_new_test(model, id);
 
 	oval_subtype_t subtype = oval_subtype_parse(reader);
         if ( subtype == OVAL_SUBTYPE_UNKNOWN) {
@@ -371,14 +371,22 @@ int oval_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *con
         }
 	oval_test_set_subtype(test, subtype);
 
-	oval_existence_t existence = oval_existence_parse(reader, "check_existence", OVAL_AT_LEAST_ONE_EXISTS);
-	oval_test_set_existence(test, existence);
-
 	oval_operator_t ste_operator = oval_operator_parse(reader, "state_operator", OVAL_OPERATOR_AND);
 	oval_test_set_state_operator(test, ste_operator);
 
 	oval_check_t check = oval_check_parse(reader, "check", OVAL_CHECK_UNKNOWN);
-	oval_test_set_check(test, check);
+	if (check == OVAL_CHECK_NONE_EXIST) {
+		dW("The 'none exist' CheckEnumeration value has been deprecated. "
+		   "Converted to check='none satisfy' and check_existence='none exist'.\n");
+		oval_test_set_check(test, OVAL_CHECK_NONE_SATISFY);
+		oval_test_set_existence(test, OVAL_NONE_EXIST);
+	} else {
+		oval_existence_t existence;
+
+		oval_test_set_check(test, check);
+		existence = oval_existence_parse(reader, "check_existence", OVAL_AT_LEAST_ONE_EXISTS);
+		oval_test_set_existence(test, existence);
+	}
 
 	comm = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "comment");
 	if (comm != NULL) {
@@ -413,21 +421,20 @@ xmlNode *oval_test_to_dom(struct oval_test *test, xmlDoc * doc, xmlNode * parent
 		return test_node;
 	}
 
-	struct oval_state_iterator *ste_itr;
+	/* get test name */
 	const char *subtype_text = oval_subtype_get_text(subtype);
 	char test_name[strlen(subtype_text) + 6];
-	*test_name = '\0';
-	strcat(strcat(test_name, subtype_text), "_test");
-	test_node = xmlNewTextChild(parent, NULL, BAD_CAST test_name, NULL);
+	sprintf(test_name, "%s_test", subtype_text);
 
+	/* get family URI */
 	oval_family_t family = oval_test_get_family(test);
 	const char *family_text = oval_family_get_text(family);
 	char family_uri[strlen((const char *)OVAL_DEFINITIONS_NAMESPACE) + strlen(family_text) + 2];
-	*family_uri = '\0';
-	strcat(strcat(strcat(family_uri, (const char *)OVAL_DEFINITIONS_NAMESPACE), "#"), family_text);
-	xmlNs *ns_family = xmlNewNs(test_node, BAD_CAST family_uri, NULL);
+	sprintf(family_uri,"%s#%s", OVAL_DEFINITIONS_NAMESPACE, family_text);
 
-	xmlSetNs(test_node, ns_family);
+	/* search namespace & create child */
+	xmlNs *ns_family = xmlSearchNsByHref(doc, parent, BAD_CAST family_uri);
+	test_node = xmlNewTextChild(parent, ns_family, BAD_CAST test_name, NULL);
 
 	char *id = oval_test_get_id(test);
 	xmlNewProp(test_node, BAD_CAST "id", BAD_CAST id);
@@ -472,7 +479,7 @@ xmlNode *oval_test_to_dom(struct oval_test *test, xmlDoc * doc, xmlNode * parent
 		xmlNewProp(object_node, BAD_CAST "object_ref", BAD_CAST oval_object_get_id(object));
 	}
 
-	ste_itr = oval_test_get_states(test);
+	struct oval_state_iterator *ste_itr = oval_test_get_states(test);
 	while (oval_state_iterator_has_more(ste_itr)) {
 		struct oval_state *state;
 
