@@ -20,13 +20,18 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 #include <time.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include "../public/sm_alloc.h"
+
+#include "public/sm_alloc.h"
 #include "common.h"
 #include "bitmap.h"
 
