@@ -22,6 +22,10 @@
 #ifndef RBT_COMMON_H
 #define RBT_COMMON_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdint.h>
 #include <stdbool.h>
 
@@ -29,6 +33,12 @@
 # include <pthread.h>
 #endif
 
+#ifndef HAVE_POSIX_MEMALIGN
+# ifdef HAVE_MEMALIGN
+int posix_memalign(void **memptr, size_t alignment, size_t size);
+# endif /* HAVE_MEMALIGN */
+#endif /* HAVE_POSIX_MEMALIGN */
+
 typedef enum {
         RBT_GENKEY,
         RBT_STRKEY,
