@@ -41,6 +41,7 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include <sexp-types.h>
+#include <helpers.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -403,6 +404,11 @@ SEXP_t *SEXP_list_push (SEXP_t *list, const SEXP_t *s_exp);
  */
 SEXP_t *SEXP_list_pop (SEXP_t *list);
 
+/**
+ * Sort a list using `compare' as the comparison function.
+ */
+SEXP_t *SEXP_list_sort(SEXP_t *list, int(*compare)(const SEXP_t *, const SEXP_t *));
+
 /**
  * Replace the n-th element of a list.
  * This function increments element's reference count.
@@ -454,6 +460,12 @@ SEXP_t    *SEXP_listit_length (SEXP_it_t *it);
 SEXP_t    *SEXP_listit_seek (SEXP_it_t *it, uint32_t n);
 void       SEXP_listit_free (SEXP_it_t *it);
 
+typedef struct SEXP_list_it SEXP_list_it;
+
+SEXP_list_it *SEXP_list_it_new(SEXP_t *list);
+SEXP_t *SEXP_list_it_next(SEXP_list_it *it);
+void SEXP_list_it_free(SEXP_list_it *it);
+
 #if __STDC_VERSION__ >= 199901L
 # include <common/util.h>
 
@@ -464,7 +476,7 @@ void       SEXP_listit_free (SEXP_it_t *it);
  * @param list the list the loop iterates through
  */
 #define SEXP_list_foreach(var, list)                                    \
-        for (uint32_t OSCAP_CONCAT(i,__LINE__) = 1; ((var) = SEXP_list_nth (list, OSCAP_CONCAT(i,__LINE__))) != NULL; ++OSCAP_CONCAT(i,__LINE__), SEXP_free (var))
+        for (uint32_t OSCAP_CONCAT(i,__LINE__) = 1; ((var) = SEXP_list_nth (list, OSCAP_CONCAT(i,__LINE__))) != NULL; ++OSCAP_CONCAT(i,__LINE__), SEXP_free (var), (var) = NULL)
 
 /**
  * Iterate through a sublist, assigning each element to a variable.
@@ -474,7 +486,7 @@ void       SEXP_listit_free (SEXP_it_t *it);
  * @param end the index of the last element of the sublist
  */
 #define SEXP_sublist_foreach(var, list, beg, end)                       \
-        for (uint32_t OSCAP_CONCAT(i,__LINE__) = (beg); OSCAP_CONCAT(i,__LINE__) <= ((size_t)(end)) && ((var) = SEXP_list_nth (list, OSCAP_CONCAT(i,__LINE__))) != NULL; ++OSCAP_CONCAT(i,__LINE__), SEXP_free (var))
+        for (uint32_t OSCAP_CONCAT(i,__LINE__) = (beg); OSCAP_CONCAT(i,__LINE__) <= ((size_t)(end)) && ((var) = SEXP_list_nth (list, OSCAP_CONCAT(i,__LINE__))) != NULL; ++OSCAP_CONCAT(i,__LINE__), SEXP_free (var), (var) = NULL)
 
 #define SEXP_LIST_END (UINT32_MAX - 1)
 
@@ -515,6 +527,13 @@ uint32_t SEXP_refs (const SEXP_t *ref);
 
 bool SEXP_eq (const SEXP_t *a, const SEXP_t *b);
 
+/**
+ * Compare reference pointers.
+ */
+int SEXP_refcmp(const SEXP_t *a, const SEXP_t *b);
+
+bool SEXP_deepcmp(const SEXP_t *a, const SEXP_t *b);
+
 #if defined(NDEBUG)
 /**
  * Free a sexp object.
@@ -528,12 +547,13 @@ void     SEXP_free (SEXP_t *s_exp);
  * @param s_exp the object to be freed
  * @param ... arbitrary number of objects to be freed
  */
-void     SEXP_vfree (SEXP_t *s_exp, ...);
+void     __SEXP_vfree (int n, SEXP_t *s_exp, ...);
+# define SEXP_vfree(...) __SEXP_vfree(PP_NARG(__VA_ARGS__), __VA_ARGS__)
 #else
 # define SEXP_free(ptr) __SEXP_free (ptr, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 void     __SEXP_free (SEXP_t *s_exp, const char *file, uint32_t line, const char *func);
-# define SEXP_vfree(...) __SEXP_vfree (__FILE__, __LINE__, __PRETTY_FUNCTION__, __VA_ARGS__)
-void     __SEXP_vfree (const char *file, uint32_t line, const char *func, SEXP_t *s_exp, ...);
+# define SEXP_vfree(...) __SEXP_vfree (__FILE__, __LINE__, __PRETTY_FUNCTION__, PP_NARG(__VA_ARGS__), __VA_ARGS__)
+void     __SEXP_vfree (const char *file, uint32_t line, const char *func, int n, SEXP_t *s_exp, ...);
 #endif
 
 /**
