@@ -25,13 +25,20 @@
 #define SM_ALLOC_H
 
 #include <seap-debug.h>
+#include "config.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#undef  __P
-#define __P __attribute__ ((unused)) static
+#define __ATTRIB __attribute__ ((unused)) static
+
+#if !defined(HAVE_POSIX_MEMALIGN)
+# if defined (HAVE_MEMALIGN)
+extern int posix_memalign (void ** __memptr, size_t __alignment, size_t __size);
+
+# endif /* HAVE_MEMALIGN */
+#endif /* HAVE_POSIX_MEMALIGN */
 
 #if defined(NDEBUG)
 void *sm_alloc (size_t s);
@@ -42,22 +49,22 @@ int   sm_memalign (void **p, size_t a, size_t s);
 void  sm_free (void *p);
 #else
 void *  __sm_alloc_dbg (size_t s, const char *f, size_t l);
-__P void *sm_alloc     (size_t s) { return __sm_alloc_dbg (s, __FUNCTION__, 0); }
+__ATTRIB void *sm_alloc     (size_t s) { return __sm_alloc_dbg (s, __FUNCTION__, 0); }
 
 void *  __sm_calloc_dbg (size_t n, size_t s, const char *f, size_t l);
-__P void *sm_calloc     (size_t n, size_t s) { return __sm_calloc_dbg (n, s, __FUNCTION__, 0); }
+__ATTRIB void *sm_calloc     (size_t n, size_t s) { return __sm_calloc_dbg (n, s, __FUNCTION__, 0); }
 
 void *  __sm_realloc_dbg (void *p, size_t s, const char *f, size_t l);
-__P void *sm_realloc     (void *p, size_t s) { return __sm_realloc_dbg (p, s, __FUNCTION__, 0); }
+__ATTRIB void *sm_realloc     (void *p, size_t s) { return __sm_realloc_dbg (p, s, __FUNCTION__, 0); }
 
 void *  __sm_reallocf_dbg (void *p, size_t s, const char *f, size_t l);
-__P void *sm_reallocf     (void *p, size_t s) { return __sm_reallocf_dbg (p, s, __FUNCTION__, 0); }
+__ATTRIB void *sm_reallocf     (void *p, size_t s) { return __sm_reallocf_dbg (p, s, __FUNCTION__, 0); }
 
 int     __sm_memalign_dbg (void **p, size_t a, size_t s, const char *f, size_t l);
-__P int __sm_memalign     (void **p, size_t a, size_t s) { return __sm_memalign_dbg (p, a, s, __FUNCTION__, 0); }
+__ATTRIB int __sm_memalign     (void **p, size_t a, size_t s) { return __sm_memalign_dbg (p, a, s, __FUNCTION__, 0); }
 
 void   __sm_free_dbg (void *p, const char *f, size_t l);
-__P void sm_free     (void *p) { __sm_free_dbg (p, __FUNCTION__, 0); }
+__ATTRIB void sm_free     (void *p) { __sm_free_dbg (p, __FUNCTION__, 0); }
 
 # define sm_alloc(s)          __sm_alloc_dbg (s, __PRETTY_FUNCTION__, __LINE__)
 # define sm_calloc(n, s)      __sm_calloc_dbg (n, s, __PRETTY_FUNCTION__, __LINE__)
