@@ -20,6 +20,10 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -36,7 +40,7 @@
 #include <fcntl.h>
 #include <common/assume.h>
 #include <errno.h>
-#include <config.h>
+
 #include "generic/common.h"
 #include "public/sm_alloc.h"
 #include "public/strbuf.h"
@@ -62,8 +66,8 @@ extern char **environ;
 
 static char *get_exec_path (const char *uri, uint32_t flags)
 {
-        char   *path;
-        size_t  plen, ulen;
+        char  *path;
+        size_t ulen;
 
         ulen = strlen (uri);
 
@@ -74,7 +78,6 @@ static char *get_exec_path (const char *uri, uint32_t flags)
         else {
                 uri  += 2;
                 ulen -= 2;
-                plen  = ulen;
         }
 
         path = sm_alloc (sizeof (char) * (PATH_MAX + 1));
@@ -270,6 +273,7 @@ int sch_pipe_openfd2 (SEAP_desc_t *desc, int ifd, int ofd, uint32_t flags)
 ssize_t sch_pipe_recv (SEAP_desc_t *desc, void *buf, size_t len, uint32_t flags)
 {
         sch_pipedata_t *data;
+	ssize_t         ret;
 
         assume_d (desc != NULL, -1, errno = EFAULT;);
         assume_d (buf  != NULL, -1, errno = EFAULT;);
@@ -278,9 +282,13 @@ ssize_t sch_pipe_recv (SEAP_desc_t *desc, void *buf, size_t len, uint32_t flags)
 
         assume_r (data != NULL, -1, errno = EBADF;);
 
-        if (check_child (data->pid, 0) == 0)
-                return read (data->pfd, buf, len);
-        else
+        if (check_child (data->pid, 0) == 0) {
+                if ((ret = read (data->pfd, buf, len)) == 0)
+			if (check_child(data->pid, 0))
+				return (-1);
+
+		return (ret);
+        } else
                 return (-1);
 }
 
