@@ -20,7 +20,12 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
+
 #include "_sexp-types.h"
 #include "_seap-types.h"
 #include "_seap-message.h"
