@@ -37,8 +37,10 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include <errno.h>
+#include <math.h>
 
 #include "common/assume.h"
+#include "common/bfind.h"
 #include "public/sm_alloc.h"
 #include "_sexp-types.h"
 #include "_sexp-value.h"
@@ -423,9 +425,10 @@ int64_t SEXP_number_geti_64 (const SEXP_t *s_exp)
         case SEXP_NUM_INT16:
                 return ((int64_t)SEXP_NCASTP(u16,v_dsc.mem)->n);
         case SEXP_NUM_UINT8:
-        case SEXP_NUM_INT8:
         case SEXP_NUM_BOOL:
                 return ((int64_t)SEXP_NCASTP(u8,v_dsc.mem)->n);
+        case SEXP_NUM_INT8:
+                return ((int64_t)SEXP_NCASTP(i8,v_dsc.mem)->n);
         default:
                 abort ();
         }
@@ -545,10 +548,27 @@ SEXP_t *SEXP_number_newf (double n)
 
 double SEXP_number_getf (const SEXP_t *s_exp)
 {
+        SEXP_val_t v_dsc;
+        SEXP_numtype_t t;
+
         _LOGCALL_;
-        abort ();
+
+        if (s_exp == NULL) {
+                errno = EFAULT;
+                return (NAN);
+        }
+
         SEXP_VALIDATE(s_exp);
-        return (0);
+
+        SEXP_val_dsc (&v_dsc, s_exp->s_valp);
+        t = SEXP_rawval_number_type (&v_dsc);
+
+        if (t != SEXP_NUM_DOUBLE) {
+                errno = EDOM;
+                return (NAN);
+        }
+
+	return SEXP_NCASTP(f,v_dsc.mem)->n;
 }
 
 int SEXP_number_get (const SEXP_t *s_exp, void *dst, SEXP_numtype_t type)
@@ -1071,54 +1091,12 @@ SEXP_t *SEXP_listref_first (SEXP_t *list)
 
 SEXP_t *SEXP_list_rest  (const SEXP_t *list)
 {
-        SEXP_t    *rest;
-        SEXP_val_t v_dsc_o, v_dsc_r;
-        struct SEXP_val_lblk *lblk;
+	SEXP_t *sexp;
 
-        _LOGCALL_;
+	sexp = SEXP_new();
+	sexp = SEXP_list_rest_r(sexp, list);
 
-        if (list == NULL) {
-                errno = EINVAL;
-                return (NULL);
-        }
-
-        SEXP_VALIDATE(list);
-
-        SEXP_val_dsc (&v_dsc_o, list->s_valp);
-
-        if (v_dsc_o.type != SEXP_VALTYPE_LIST) {
-                errno = EINVAL;
-                return (NULL);
-        }
-
-        if (SEXP_val_new (&v_dsc_r, sizeof (void *) + sizeof (uint16_t),
-                          SEXP_VALTYPE_LIST) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        SEXP_LCASTP(v_dsc_r.mem)->offset = SEXP_LCASTP(v_dsc_o.mem)->offset + 1;
-        SEXP_LCASTP(v_dsc_r.mem)->b_addr = SEXP_LCASTP(v_dsc_o.mem)->b_addr;
-
-        lblk = SEXP_VALP_LBLK(SEXP_LCASTP(v_dsc_r.mem)->b_addr);
-
-        if (lblk != NULL) {
-                if (SEXP_LCASTP(v_dsc_r.mem)->offset == lblk->real) {
-                        SEXP_LCASTP(v_dsc_r.mem)->offset = 0;
-                        SEXP_LCASTP(v_dsc_r.mem)->b_addr = SEXP_VALP_LBLK(lblk->nxsz);
-                }
-
-                if (SEXP_VALP_LBLK(SEXP_LCASTP(v_dsc_r.mem)->b_addr) != NULL)
-                        SEXP_LCASTP(v_dsc_r.mem)->b_addr = (void *)SEXP_rawval_lblk_incref ((uintptr_t) SEXP_LCASTP(v_dsc_r.mem)->b_addr);
-        }
-
-        rest = SEXP_new ();
-        rest->s_valp = SEXP_val_ptr (&v_dsc_r);
-
-        SEXP_VALIDATE(rest);
-
-        return (rest);
+	return (sexp);
 }
 
 SEXP_t *SEXP_listref_rest (SEXP_t *list)
@@ -1406,6 +1384,184 @@ SEXP_t *SEXP_list_pop (SEXP_t *list)
         return (s_ref);
 }
 
+#define SEXP_LISTIT_ARRAY_INC  32
+#define SEXP_LISTIT_ARRAY_INIT 32
+
+struct SEXP_list_it{
+        struct SEXP_val_lblk *block;
+        uint16_t index;
+        uint16_t count;
+};
+
+SEXP_list_it *SEXP_list_it_new(SEXP_t *list)
+{
+        SEXP_val_t v_dsc;
+        SEXP_list_it *it;
+
+        if (list == NULL) {
+                errno = EFAULT;
+                return (NULL);
+        }
+
+        SEXP_val_dsc(&v_dsc, list->s_valp);
+
+        if (v_dsc.type != SEXP_VALTYPE_LIST) {
+                errno = EINVAL;
+                return (NULL);
+        }
+
+        it = sm_talloc(SEXP_list_it);
+        it->block = SEXP_LCASTP(v_dsc.mem)->b_addr;
+        it->index = SEXP_LCASTP(v_dsc.mem)->offset;
+        it->count = it->block != NULL ? it->block->real : 0;
+
+        return (it);
+}
+
+SEXP_t *SEXP_list_it_next(SEXP_list_it *it)
+{
+        SEXP_t *item;
+
+        if (it->block == NULL)
+                return (NULL);
+
+        item = it->block->memb + it->index;
+
+        ++it->index;
+
+        if (it->index == it->count) {
+                it->block = SEXP_VALP_LBLK(it->block->nxsz);
+                it->index = 0;
+                it->count = it->block != NULL ? it->block->real : 0;
+        }
+
+        return (item);
+}
+
+void SEXP_list_it_free(SEXP_list_it *it)
+{
+        sm_free(it);
+}
+
+SEXP_t *SEXP_list_sort(SEXP_t *list, int(*compare)(const SEXP_t *, const SEXP_t *))
+{
+        SEXP_val_t v_dsc;
+        SEXP_list_it *list_it;
+        size_t list_it_count, list_it_alloc;
+        register size_t i, j;
+
+        if (list == NULL || compare == NULL) {
+                errno = EFAULT;
+                return (NULL);
+        }
+
+        SEXP_val_dsc(&v_dsc, list->s_valp);
+
+        if (v_dsc.type != SEXP_VALTYPE_LIST) {
+                errno = EINVAL;
+                return (NULL);
+        }
+
+        /*
+         * TODO: check reference counts and make copies of list
+         * blocks if needed
+         */
+
+        /*
+         * PASS #1: Sort each block and build the iterator array
+         */
+        list_it_count = 1;
+        list_it_alloc = SEXP_LISTIT_ARRAY_INIT;
+        list_it = sm_alloc(sizeof(SEXP_list_it) * list_it_alloc);
+
+        list_it[0].block = SEXP_LCASTP(v_dsc.mem)->b_addr;
+
+        _I("Sorting blocks & building iterator array\n");
+
+        while (list_it[list_it_count - 1].block != NULL) {
+                /* initialize the rest of the iterator */
+                list_it[list_it_count - 1].index = list_it_count == 1 ? SEXP_LCASTP(v_dsc.mem)->offset : 0;
+                list_it[list_it_count - 1].count = list_it[list_it_count - 1].block->real;
+
+                /* sort */
+                qsort(list_it[list_it_count - 1].block->memb,
+                      list_it[list_it_count - 1].count, sizeof(SEXP_t),
+                      (int(*)(const void *, const void *))compare);
+
+                /* reallocate the iterator array if needed */
+                if (list_it_count == list_it_alloc) {
+                        _I("Reallocating iterator array: %z -> %z\n",
+                           list_it_alloc, list_it_alloc + SEXP_LISTIT_ARRAY_INC);
+
+                        list_it_alloc += SEXP_LISTIT_ARRAY_INC;
+                        list_it = sm_realloc(list_it, sizeof(SEXP_list_it) * list_it_alloc);
+                }
+
+                /* skip to the next block */
+                list_it[list_it_count].block = SEXP_VALP_LBLK(list_it[list_it_count - 1].block->nxsz);
+                ++list_it_count;
+        }
+
+        --list_it_count;
+        _I("Iterator count = %zu\n", list_it_count);
+
+        if (list_it_count > 0) {
+                /*
+                 * PASS #2: Mergesort all blocks
+                 */
+                for (i = 0; i < list_it_count - 1; ++i) {
+                        while(list_it[i].index < list_it[i].count) {
+                                SEXP_t *min_v, tmp_v, *first_v;
+                                size_t  min_i = i;
+
+                                /* search for minimal s_valp value */
+                                min_v = first_v = list_it[i].block->memb + list_it[i].index;
+
+                                for (j = i + 1; j < list_it_count; ++j) {
+                                        if (compare(list_it[j].block->memb + list_it[j].index, min_v) < 0) {
+                                                min_v = list_it[j].block->memb + list_it[j].index;
+                                                min_i = j;
+                                        }
+                                }
+
+                                if (min_v != first_v) {
+                                        size_t dst_i = 0;
+
+                                        /* save the old value & move the minval to it's place */
+                                        tmp_v  = list_it[i].block->memb[list_it[i].index];
+                                        list_it[i].block->memb[list_it[i].index] = *min_v;
+
+					if (list_it[min_i].count > 1) {
+						/*
+						 * find a new place for the old values in the min.
+						 * value source block
+						 */
+						(void)oscap_bfind_i(list_it[min_i].block->memb + 1,
+								    list_it[min_i].count - 1, sizeof(SEXP_t), (void *)&tmp_v,
+								    (int(*)(void *, void *))compare, &dst_i);
+
+						_I("dst_i = %zu\n", dst_i);
+
+						/* make place for the old value in the min. value source block */
+						memmove(list_it[min_i].block->memb, list_it[min_i].block->memb + 1,
+							sizeof(SEXP_t) * dst_i);
+					}
+
+                                        list_it[min_i].block->memb[dst_i] = tmp_v;
+                                }
+
+                                ++list_it[i].index;
+                        }
+                }
+        }
+        /*
+         * Cleanup
+         */
+        sm_free(list_it);
+
+        return (list);
+}
+
 void SEXP_lstack_init (SEXP_lstack_t *stack)
 {
         stack->p_list = SEXP_list_new (NULL);
@@ -1670,6 +1826,84 @@ bool SEXP_eq (const SEXP_t *a, const SEXP_t *b)
                 return (false);
         return (a->s_valp == b->s_valp);
 }
+#include <inttypes.h>
+int SEXP_refcmp(const SEXP_t *a, const SEXP_t *b)
+{
+        if (a->s_valp < b->s_valp)
+                return (-1);
+        if (a->s_valp > b->s_valp)
+                return (1);
+        return (0);
+}
+
+bool SEXP_deepcmp(const SEXP_t *a, const SEXP_t *b)
+{
+        SEXP_valtype_t type;
+
+        if (a == NULL || b == NULL)
+                return (a == b);
+        if ((type = SEXP_typeof(a)) != SEXP_typeof(b))
+                return (false);
+        if (!SEXP_listp(a)) {
+                /* compare simple objects */
+                switch(type) {
+                case SEXP_VALTYPE_STRING:
+                        return (SEXP_string_cmp(a, b) == 0);
+                case SEXP_VALTYPE_NUMBER: {
+                        SEXP_numtype_t ntype_a, ntype_b;
+
+                        ntype_a = SEXP_number_type(a);
+                        ntype_b = SEXP_number_type(b);
+
+                        if (ntype_a == SEXP_NUM_DOUBLE) {
+                                if (ntype_b != SEXP_NUM_DOUBLE)
+                                        return (false);
+                                else {
+                                        double na, nb;
+
+                                        na = SEXP_number_getf(a);
+                                        nb = SEXP_number_getf(b);
+
+                                        return (na == nb);
+                                }
+                        } else {
+                                uint64_t na, nb;
+
+                                na = SEXP_number_getu_64(a);
+                                nb = SEXP_number_getu_64(b);
+
+                                return (na == nb);
+                        }
+                        /* NOTREACHED */
+                }
+                default:
+#ifndef NDEBUG
+                        abort();
+#endif
+                        return (false);
+                }
+        } else {
+                /* compare lists */
+                register SEXP_list_it *it_a, *it_b;
+		register SEXP_t *ia, *ib;
+		register bool ret = false;
+
+                it_a = SEXP_list_it_new(a);
+                it_b = SEXP_list_it_new(b);
+
+                do {
+                        ret = SEXP_deepcmp(ia = SEXP_list_it_next(it_a),
+                                           ib = SEXP_list_it_next(it_b));
+                } while(ret && ib != NULL && ia != NULL);
+
+                SEXP_list_it_free(it_a);
+                SEXP_list_it_free(it_b);
+
+                return (ret);
+        }
+        /* NOTREACHED */
+        return (false);
+}
 
 /*
  * SEXP_free for list members. The difference between
@@ -1741,21 +1975,18 @@ void __SEXP_free (SEXP_t *s_exp, const char *file, uint32_t line, const char *fu
 #endif
 
 #if defined(NDEBUG)
-void SEXP_vfree (SEXP_t *s_exp, ...)
+void __SEXP_vfree (int n, SEXP_t *s_exp, ...)
 #else
-void __SEXP_vfree (const char *file, uint32_t line, const char *func, SEXP_t *s_exp, ...)
+void __SEXP_vfree (const char *file, uint32_t line, const char *func, int n, SEXP_t *s_exp, ...)
 #endif
 {
         va_list ap;
 
-#if !defined(NDEBUG) && defined(SEAP_VERBOSE_DEBUG)
-        _D("s_exp=%p (%s:%u:%s)\n", s_exp, file, line, func);
-#endif
-
         va_start (ap, s_exp);
 
-        for (; s_exp != NULL; s_exp = va_arg (ap, SEXP_t *))
-                SEXP_free (s_exp);
+        for (; n > 1; --n, s_exp = va_arg (ap, SEXP_t *))
+		if (s_exp != NULL)
+			SEXP_free (s_exp);
 
         va_end (ap);
 }
@@ -1933,57 +2164,6 @@ size_t SEXP_sizeof (const SEXP_t *s_exp)
         return (sz);
 }
 
-#if 0
-int SEXP_structprint (FILE *fp, const SEXP_t *s_exp)
-{
-        SEXP_val_t v_dsc;
-
-        SEXP_val_dsc (&v_dsc, s_exp->s_valp);
-
-        /*
-         *  print S-exp flags, type, value pointer
-         */
-
-        fprintf (fp,
-                 "=> s-exp@%p ... type: %s (%x)\n"
-                 "            ... flgs: %c%c%c (%x)\n"
-                 "            ...  ptr: %p\n",
-                 SEXP_strtype (s_exp),
-		 SEXP_softrefp(s_exp) ? 's' : '-',
-		 '-',
-		 '-',
-/*
-                 (s_exp->s_flgs & SEXP_FLAG_SREF  ? 's' : '-'),
-                 (s_exp->s_flgs & SEXP_FLAG_INVAL ? 'i' : '-'),
-                 (s_exp->s_flgs & SEXP_FLAG_UNFIN ? 'u' : '-'),
-*/
-                 (void *)s_exp->s_valp);
-
-        switch (v_dsc.type) {
-        case SEXP_VALTYPE_LIST:
-        {
-                struct SEXP_val_lblk *lblk;
-
-                /* print list blocks */
-
-                /* recursion */
-
-        }       break;
-        case SEXP_VALTYPE_NUMBER:
-
-
-                break;
-        case SEXP_VALTYPE_STRING:
-
-                break;
-        default:
-                abort ();
-        }
-
-        return (0);
-}
-#endif /* 0 */
-
 #if !defined(NDEBUG) || defined(VALIDATE_SEXP)
 typedef struct {
         const char *file;
