@@ -19,11 +19,17 @@
  * Authors:
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-#include <config.h>
+
 #include "common/assume.h"
 #include "generic/common.h"
 #include "public/sm_alloc.h"
@@ -63,6 +69,7 @@ SEXP_pstate_t *SEXP_pstate_new (void)
 
         pstate->v_bool[0] = 0;
         pstate->v_bool[1] = 0;
+	pstate->p_error   = SEXP_PRET_EUNDEF;
 
         return (pstate);
 }
@@ -1213,6 +1220,10 @@ SEXP_t *SEXP_parse (const SEXP_psetup_t *psetup, char *buffer, size_t buflen, SE
                 e_dsc.p_bufoff += e_dsc.p_explen;
                 e_dsc.p_explen  = 0;
 
+                e_dsc.p_numclass = SEXP_NUMCLASS_INV;
+                e_dsc.p_numbase  = 0;
+                e_dsc.p_numstage = 255;
+
                 continue;
         } /* for (;;) */
 
@@ -1235,6 +1246,7 @@ SKIP_LOOP:
 
                         return (s_list);
                 }
+                /* FALLTHROUGH */
         case SEXP_PRET_EUNFIN:
                 /*
                  * The last S-expression parsed was considered incomplete and we
@@ -1259,14 +1271,17 @@ SKIP_LOOP:
                 state->p_numclass = e_dsc.p_numclass;
                 state->p_numbase  = e_dsc.p_numbase;
                 state->p_numstage = e_dsc.p_numstage;
+		state->p_error    = SEXP_PRET_EUNFIN;
 
                 return (NULL);
         case SEXP_PRET_EINVAL:
+		state->p_error = SEXP_PRET_EINVAL;
                 /*
                  * The parser encoutered an invalid sequence of octets
                  */
                 return (NULL);
         case SEXP_PRET_EUNDEF:
+		state->p_error = SEXP_PRET_EUNDEF;
                 /*
                  * Undefined error (i.e. we don't know how to handle the error
                  * that caused the undefined state or we do not expect that such
@@ -1690,17 +1705,24 @@ found:
                                 b_encfree = true;
                         }
 
-                        if (spb_pick (dsc->p_buffer, dsc->p_bufoff + 1, dsc->p_explen - 2, b_enc) != 0)
+			if (spb_pick (dsc->p_buffer, dsc->p_bufoff + 1, dsc->p_explen - 2, b_enc) != 0) {
+				if (b_encfree)
+					sm_free (b_enc);
                                 return (SEXP_PRET_EUNDEF);
+			}
                 } else
                         b_encfree = false;
 
+                b_dec    = NULL;
                 b_declen = base64_decode (b_enc, dsc->p_explen - 2, &b_dec);
 
                 if (b_declen == 0) {
                         if (b_encfree)
                                 sm_free (b_enc);
 
+                        if (b_dec != NULL)
+                                sm_free (b_dec);
+
                         return (SEXP_PRET_EINVAL);
                 }
 
@@ -1749,17 +1771,24 @@ __PARSE_RT SEXP_parse_kl_string_b64 (__PARSE_PT(dsc))
                         b_encfree = true;
                 }
 
-                if (spb_pick (dsc->p_buffer, dsc->p_bufoff + 1, dsc->p_explen, b_enc) != 0)
+		if (spb_pick (dsc->p_buffer, dsc->p_bufoff + 1, dsc->p_explen, b_enc) != 0) {
+			if (b_encfree)
+				sm_free (b_enc);
                         return (SEXP_PRET_EUNDEF);
+		}
         } else
                 b_encfree = false;
 
+        b_dec    = NULL;
         b_declen = base64_decode (b_enc, dsc->p_explen, &b_dec);
 
         if (b_declen == 0) {
                 if (b_encfree)
                         sm_free (b_enc);
 
+                if (b_dec != NULL)
+                        sm_free (b_dec);
+
                 return (SEXP_PRET_EINVAL);
         }
 
@@ -1927,3 +1956,17 @@ __PARSE_RT SEXP_parse_bool (__PARSE_PT(dsc), bool val)
 
         return (SEXP_PRET_SUCCESS);
 }
+
+bool SEXP_pstate_errorp(SEXP_pstate_t *pstate)
+{
+	if (pstate == NULL)
+		return (true);
+
+	switch (pstate->p_error) {
+	case SEXP_PRET_SUCCESS:
+	case SEXP_PRET_EUNFIN:
+		return (false);
+	default:
+		return (true);
+	}
+}
