@@ -20,8 +20,12 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
+//#ifdef HAVE_CONFIG_H
 #include <config.h>
+//#endif
+
 #include <stdint.h>
+
 #include "_sexp-atomic.h"
 #include "_sexp-value.h"
 #include "public/sm_alloc.h"
@@ -42,14 +46,14 @@ int SEXP_val_new (SEXP_val_t *dst, size_t vmemsize, SEXP_type_t type)
         dst->hdr->size = vmemsize;
         dst->type      = type;
         dst->ptr       = SEXP_val_ptr (dst);
-
+#if defined(SEAP_VERBOSE_DEBUG)
         _D("\n"
            "new value: hdr->refs = %u\n"
            "           hdr->size = %zu\n"
            "                type = %hhu\n"
            "                 ptr = %p\n",
            dst->hdr->refs, dst->hdr->size, dst->type, (void *)dst->ptr);
-
+#endif
         return (0);
 }
 
@@ -302,7 +306,6 @@ SEXP_t *SEXP_rawval_lblk_nth (uintptr_t lblkp, uint32_t n)
         register struct SEXP_val_lblk *lblk;
 
         _LOGCALL_;
-        _D("n = %u\n", n);
 
         lblk = SEXP_VALP_LBLK(lblkp);
 
@@ -366,6 +369,9 @@ uintptr_t SEXP_rawval_lblk_replace (uintptr_t lblkp, uint32_t n, const SEXP_t *n
                         SEXP_rawval_lblk_decref ((uintptr_t)lblk);
                         memb = SEXP_rawval_lblk_nth (lb_ptr, n);
 
+			if (memb == NULL)
+				abort();
+
                         goto replace;
                 }
         }
@@ -374,7 +380,6 @@ uintptr_t SEXP_rawval_lblk_replace (uintptr_t lblkp, uint32_t n, const SEXP_t *n
 
         memb = lblk->memb + (n - 1);
 replace:
-
         _A(lb_head != 0);
         _A(memb != NULL);
 
@@ -460,7 +465,6 @@ uintptr_t SEXP_rawval_lblk_copy (uintptr_t lblkp, uint16_t n_skip)
         uint16_t  off_n;  /* offset in the new block */
         uint16_t  off_o;  /* offset in the old block */
         uint8_t  cur_sz;  /* size of the new block */
-        uint8_t  old_sz;  /* size of the old block */
 
         lb_head = 0;
         off_n   = 0;
@@ -470,7 +474,6 @@ uintptr_t SEXP_rawval_lblk_copy (uintptr_t lblkp, uint16_t n_skip)
         if (lb_old == NULL)
                 return ((uintptr_t) NULL);
 
-        old_sz  = lb_old->nxsz & SEXP_LBLKS_MASK;
         cur_sz  = 0;
         lb_new  = (struct SEXP_val_lblk *)SEXP_rawval_lblk_new (cur_sz);
         lb_head = (uintptr_t)lb_new;
@@ -486,7 +489,6 @@ uintptr_t SEXP_rawval_lblk_copy (uintptr_t lblkp, uint16_t n_skip)
                                 break;
 
                         off_o  = 0;
-                        old_sz = lb_old->nxsz & SEXP_LBLKS_MASK;
                 }
 
                 /*
