@@ -72,11 +72,9 @@ static int mem2hex (uint8_t *mem, size_t mlen, char *str, size_t slen)
         return (0);
 }
 
-static int filehash_cb (const char *p, const char *f, void *ptr)
+static int filehash_cb (const char *p, const char *f, probe_ctx *ctx)
 {
-        SEXP_t *itm, *r0, *r1;
-        SEXP_t *res = (SEXP_t *) ptr;
-
+        SEXP_t *itm;
         char   pbuf[PATH_MAX+1];
         size_t plen, flen;
 
@@ -110,31 +108,20 @@ static int filehash_cb (const char *p, const char *f, void *ptr)
         fd = open (pbuf, O_RDONLY);
 
         if (fd < 0) {
-                SEXP_t *at;
-
                 strerror_r (errno, pbuf, PATH_MAX);
                 pbuf[PATH_MAX] = '\0';
 
-                fd = open (p, O_RDONLY);
-                at = probe_attr_creat ("status",  r0 = SEXP_number_newi_32 (OVAL_STATUS_ERROR), /* XXX: don't use newi_32 directly */
-                                       "message", r1 = SEXP_string_newf ("%s", pbuf),
-                                       NULL);
+		itm = probe_item_create(OVAL_INDEPENDENT_FILE_HASH, NULL,
+				"filepath", OVAL_DATATYPE_STRING, pbuf,
+				"path",     OVAL_DATATYPE_STRING, p,
+				"filename", OVAL_DATATYPE_STRING, f,
+				NULL
+				);
+		probe_item_add_msg(itm, OVAL_MESSAGE_LEVEL_ERROR,
+				"Can't open \"%s\": errno=%d, %s.", pbuf, errno, strerror (errno));
+		probe_item_setstatus(itm, SYSCHAR_STATUS_ERROR);
 
-                SEXP_vfree (r0, r1, NULL);
-
-                if (fd < 0) {
-                        itm = probe_item_creat ("filehash_item", NULL,
-                                                "path", at, NULL,
-                                                NULL);
-                } else {
-                        close (fd);
-                        itm = probe_item_creat ("filehash_item", NULL,
-                                                "filename", at, NULL,
-                                                NULL);
-                }
-
-                SEXP_free (at);
-        } else {
+       } else {
                 uint8_t md5_dst[16];
                 size_t  md5_dstlen = sizeof md5_dst;
                 char    md5_str[(sizeof md5_dst * 2) + 1];
@@ -147,8 +134,8 @@ static int filehash_cb (const char *p, const char *f, void *ptr)
                  * Compute hash values
                  */
                 if (crapi_mdigest_fd (fd, 2,
-                                      CRAPI_DIGEST_MD5,  &md5_dst,  &md5_dstlen,
-                                      CRAPI_DIGEST_SHA1, &sha1_dst, &sha1_dstlen) != 0)
+                                      CRAPI_DIGEST_MD5,  md5_dst,  &md5_dstlen,
+                                      CRAPI_DIGEST_SHA1, sha1_dst, &sha1_dstlen) != 0)
                 {
                         close (fd);
                         return (-1);
@@ -171,8 +158,7 @@ static int filehash_cb (const char *p, const char *f, void *ptr)
                                         NULL);
         }
 
-	probe_cobj_add_item(res, itm);
-        SEXP_free (itm);
+        probe_item_collect(ctx, itm);
 
         return (0);
 }
@@ -212,25 +198,21 @@ void probe_fini (void *arg)
         return;
 }
 
-int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex, SEXP_t *filters)
+int probe_main (probe_ctx *ctx, void *mutex)
 {
-        SEXP_t *path, *filename, *behaviors, *filepath;
+        SEXP_t *path, *filename, *behaviors, *filepath, *probe_in;
 
 	OVAL_FTS    *ofts;
 	OVAL_FTSENT *ofts_ent;
 
-        (void)filters;
-
-	if (probe_in == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
-
         if (mutex == NULL) {
 		return (PROBE_EINIT);
         }
 
         _A(mutex == &__filehash_probe_mutex);
 
+        probe_in  = probe_ctx_getobject(ctx);
+
         path      = probe_obj_getent (probe_in, "path",      1);
         filename  = probe_obj_getent (probe_in, "filename",  1);
         behaviors = probe_obj_getent (probe_in, "behaviors", 1);
@@ -270,7 +252,7 @@ int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex, SEXP_t *filter
 
 	if ((ofts = oval_fts_open(path, filename, filepath, behaviors)) != NULL) {
 		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
-			filehash_cb(ofts_ent->path, ofts_ent->file, probe_out);
+			filehash_cb(ofts_ent->path, ofts_ent->file, ctx);
 			oval_ftsent_free(ofts_ent);
 		}
 
