@@ -243,7 +243,7 @@ __fail:
 }
 
 static int dbSQL_eval(const char *engine, const char *version,
-                      const char *conn, const char *sql, SEXP_t *probe_out)
+                      const char *conn, const char *sql, probe_ctx *ctx)
 {
 	int err = -1;
 	dbURIInfo_t uriInfo = { .host = NULL,
@@ -332,8 +332,7 @@ static int dbSQL_eval(const char *engine, const char *version,
 				odbx_result_finish(sql_dbr);
 			}
 
-			probe_cobj_add_item(probe_out, item);
-			SEXP_free(item);
+                        probe_item_collect(ctx, item);
 		}
 
 		if (odbx_finish(sql_dbh) != ODBX_ERR_SUCCESS) {
@@ -348,10 +347,11 @@ __exit:
 	return (err);
 }
 
-int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *arg)
 {
-	char       *engine, *version, *conn, *sqlexp;
+	char *engine, *version, *conn, *sqlexp;
 	int err;
+        SEXP_t *probe_in;
 
 #define get_string(dst, obj, ent_name)					\
 	do {								\
@@ -376,15 +376,13 @@ int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 		SEXP_free(__sval);					\
 	} while (0)
 
-	if (probe_in == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
-
 	engine  = NULL;
 	version = NULL;
 	conn    = NULL;
 	sqlexp  = NULL;
 
+        probe_in = probe_ctx_getobject(ctx);
+
 	get_string(engine,  probe_in, "engine");
 	get_string(version, probe_in, "version");
 	get_string(conn,    probe_in, "connection_string");
@@ -393,7 +391,7 @@ int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 	/*
 	 * evaluate the SQL statement
 	 */
-	err = dbSQL_eval(engine, version, conn, sqlexp, probe_out);
+	err = dbSQL_eval(engine, version, conn, sqlexp, ctx);
 __exit:
 	if (sqlexp != NULL) {
 		__clearmem(sqlexp, strlen(sqlexp));
