@@ -210,7 +210,7 @@ static SEXP_t *create_item(const char *path, const char *filename, char *pattern
 struct pfdata {
 	char *pattern;
 	SEXP_t *filename_ent;
-	SEXP_t *cobj;
+        probe_ctx *ctx;
 };
 
 static int process_file(const char *path, const char *filename, void *arg)
@@ -272,8 +272,9 @@ static int process_file(const char *path, const char *filename, void *arg)
 			++cur_inst;
 			item = create_item(path, filename, pfd->pattern,
 					   cur_inst, substrs, substr_cnt);
-			probe_cobj_add_item(pfd->cobj, item);
-			SEXP_free(item);
+
+                        probe_item_collect(pfd->ctx, item);
+
 			for (k = 0; k < substr_cnt; ++k)
 				free(substrs[k]);
 			free(substrs);
@@ -295,20 +296,17 @@ static int process_file(const char *path, const char *filename, void *arg)
 	return ret;
 }
 
-int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *arg)
 {
-	SEXP_t *path_ent, *filename_ent, *line_ent, *behaviors_ent, *filepath_ent;
+	SEXP_t *path_ent, *filename_ent, *line_ent, *behaviors_ent, *filepath_ent, *probe_in;
 	char *pattern;
 
 	OVAL_FTS    *ofts;
 	OVAL_FTSENT *ofts_ent;
 
         (void)arg;
-        (void)filters;
 
-	if (probe_in == NULL || probe_out == NULL) {
-		return(PROBE_EINVAL);
-	}
+        probe_in = probe_ctx_getobject(ctx);
 
 	path_ent = probe_obj_getent(probe_in, "path",     1);
 	filename_ent = probe_obj_getent(probe_in, "filename", 1);
@@ -345,7 +343,7 @@ int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 
 	pfd.pattern = pattern;
 	pfd.filename_ent = filename_ent;
-	pfd.cobj = probe_out;
+	pfd.ctx = ctx;
 
 	if ((ofts = oval_fts_open(path_ent, filename_ent, filepath_ent, behaviors_ent)) != NULL) {
 		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
