@@ -23,12 +23,58 @@
 #define OVAL_FTS_H
 
 #include <sexp.h>
+#if defined(__SVR4) && defined(__sun)
+#include "fts_sun.h"
+#else
 #include <fts.h>
+#endif
 #include <pcre.h>
 #include "fsdev.h"
 
+#define ENT_GET_AREF(ent, dst, attr_name, mandatory)			\
+	do {								\
+		if (((dst) = probe_ent_getattrval(ent, attr_name)) == NULL) { \
+			if (mandatory) {				\
+				_F("Attribute `%s' is missing!\n", attr_name); \
+				return (NULL);				\
+			}						\
+		}							\
+	} while(0)
+
+#define ENT_GET_STRVAL(ent, dst, dstlen, zerolen_exp)			\
+	do {							\
+		SEXP_t *___r;					\
+								\
+		if ((___r = probe_ent_getval(ent)) == NULL) {	\
+			_W("entity has no value!\n");		\
+			return (NULL);				\
+		} else {					\
+			if (!SEXP_stringp(___r)) {		\
+				_F("invalid type\n");		\
+				SEXP_free(___r);		\
+				return (NULL);			\
+			}					\
+			if (SEXP_string_length(___r) == 0) {	\
+				SEXP_free(___r);		\
+				zerolen_exp;			\
+			} else {				\
+				SEXP_string_cstr_r(___r, dst, dstlen); \
+				SEXP_free(___r);		\
+			}					\
+		}						\
+	} while (0)
+
 typedef struct {
-	FTS    *ofts_fts;
+	/* oval_fts_read_match_path() state */
+	FTS *ofts_match_path_fts;
+	FTSENT *ofts_match_path_fts_ent;
+	/* oval_fts_read_recurse_path() state */
+	FTS *ofts_recurse_path_fts;
+	int ofts_recurse_path_fts_opts;
+	int ofts_recurse_path_curdepth;
+	char *ofts_recurse_path_pthcpy;
+	char *ofts_recurse_path_curpth;
+	dev_t ofts_recurse_path_devid;
 
 	char   **ofts_st_path;       /**< starting paths - NULL terminated */
 	uint16_t ofts_st_path_count; /**< number of starting paths */
