@@ -41,11 +41,12 @@
 #include "_probe-api.h"
 #include "assume.h"
 #include "memusage.h"
-#include "sysinfo.h"
+#include "oscap_sysinfo.h"
 #include "oval_probe_impl.h"
+#include "probe/entcmp.h"
 
 extern probe_rcache_t  *OSCAP_GSYM(pcache);
-extern probe_ncache_t  *OSCAP_GSYM(encache);
+extern probe_ncache_t  *OSCAP_GSYM(ncache);
 extern struct id_desc_t OSCAP_GSYM(id_desc);
 
 /*
@@ -67,7 +68,7 @@ SEXP_t *probe_item_creat(const char *name, SEXP_t * attrs, ...)
 		attrs = va_arg(ap, SEXP_t *);
 		val = va_arg(ap, SEXP_t *);
 
-                ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
+                ns  = probe_ncache_ref (OSCAP_GSYM(ncache), name);
 		ent = SEXP_list_new(NULL);
 
 		if (attrs != NULL) {
@@ -92,6 +93,8 @@ SEXP_t *probe_item_creat(const char *name, SEXP_t * attrs, ...)
 		name = va_arg(ap, const char *);
 	}
 
+	va_end(ap);
+
 	return (itm);
 }
 
@@ -101,8 +104,13 @@ SEXP_t *probe_item_new(const char *name, SEXP_t * attrs)
 
 	_LOGCALL_;
 
-	sid = probe_item_newid(&(OSCAP_GSYM(id_desc)));
+        /*
+         * Allocate space for the ID which will be generated
+         * in the icache worker
+         */
+	sid  = SEXP_string_new("", 0);
 	attr = probe_attr_creat("id", sid, NULL);
+
 	if (attrs != NULL) {
 		attrs = SEXP_list_join(attr, attrs);
 		SEXP_free(attr);
@@ -211,28 +219,85 @@ int probe_itement_setstatus(SEXP_t *obj, const char *name, uint32_t n, oval_sysc
         return (0);
 }
 
-SEXP_t *probe_item_newid(struct id_desc_t * id_desc)
-{
-	int id_ctr;
-	SEXP_t *sid;
-
-#if defined(HAVE_ATOMIC_FUNCTIONS)
-	id_ctr = __sync_fetch_and_add(&(id_desc->item_id_ctr), 1);
-#else
-	pthread_mutex_lock(&(id_desc->item_id_ctr_lock));
-	id_ctr = id_desc->item_id_ctr++;
-	pthread_mutex_unlock(&(id_desc->item_id_ctr_lock));
-#endif
-	sid = SEXP_string_newf("%d", id_ctr);
-
-	return sid;
-}
-
 void probe_item_resetidctr(struct id_desc_t *id_desc)
 {
 	id_desc->item_id_ctr = 1;
 }
 
+bool probe_item_filtered(const SEXP_t *item, const SEXP_t *filters)
+{
+	bool filtered = false;
+	SEXP_t *filter, *ste;
+
+	SEXP_list_foreach(filter, filters) {
+		SEXP_t *felm, *ste_res, *r0;
+		oval_result_t ores;
+		oval_operator_t oopr;
+		oval_filter_action_t ofact;
+
+		r0 = SEXP_list_first(filter);
+		ofact = SEXP_number_getu(r0);
+		SEXP_free(r0);
+		ste = SEXP_list_nth(filter, 2);
+		ste_res = SEXP_list_new(NULL);
+
+		SEXP_sublist_foreach(felm, ste, 2, SEXP_LIST_END) {
+			SEXP_t *ielm, *elm_res;
+			char *elm_name;
+			oval_check_t ochk;
+			int i;
+
+			elm_res = SEXP_list_new(NULL);
+			elm_name = probe_ent_getname(felm);
+
+			for (i = 1;; ++i) {
+				ielm = probe_obj_getent(item, elm_name, i);
+
+				if (ielm == NULL)
+					break;
+
+				ores = probe_entste_cmp(felm, ielm);
+				SEXP_list_add(elm_res, r0 = SEXP_number_newi_32(ores));
+
+				SEXP_free(ielm);
+				SEXP_free(r0);
+			}
+
+			oscap_free(elm_name);
+			r0 = probe_ent_getattrval(felm, "entity_check");
+
+			if (r0 == NULL)
+				ochk = OVAL_CHECK_ALL;
+			else
+				ochk = SEXP_number_geti_32(r0);
+
+			SEXP_free(r0);
+
+			ores = probe_ent_result_bychk(elm_res, ochk);
+			SEXP_list_add(ste_res, r0 = SEXP_number_newi_32(ores));
+			SEXP_free(r0);
+			SEXP_free(elm_res);
+		}
+
+		r0 = probe_ent_getattrval(ste, "operator");
+		if (r0 == NULL)
+			oopr = OVAL_OPERATOR_AND;
+		else
+			oopr = SEXP_number_geti_32(r0);
+		ores = probe_ent_result_byopr(ste_res, oopr);
+		SEXP_vfree(ste, ste_res, r0, NULL);
+
+		if ((ores == OVAL_RESULT_TRUE && ofact == OVAL_FILTER_ACTION_EXCLUDE)
+		    || (ores == OVAL_RESULT_FALSE && ofact == OVAL_FILTER_ACTION_INCLUDE)) {
+			filtered = true;
+			SEXP_free(filter);
+			break;
+		}
+	}
+
+	return filtered;
+}
+
 /*
  * attributes
  */
@@ -288,7 +353,7 @@ SEXP_t *probe_obj_creat(const char *name, SEXP_t * attrs, ...)
 		attrs = va_arg(ap, SEXP_t *);
 		val = va_arg(ap, SEXP_t *);
 
-                ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
+                ns  = probe_ncache_ref (OSCAP_GSYM(ncache), name);
 		ent = SEXP_list_new(NULL);
 
 		if (attrs != NULL) {
@@ -313,6 +378,7 @@ SEXP_t *probe_obj_creat(const char *name, SEXP_t * attrs, ...)
 		name = va_arg(ap, const char *);
 	}
 
+	va_end(ap);
 	return (obj);
 }
 
@@ -323,7 +389,7 @@ SEXP_t *probe_obj_new(const char *name, SEXP_t * attrs)
 	_LOGCALL_;
 
 	obj = SEXP_list_new(NULL);
-	ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
+	ns  = probe_ncache_ref (OSCAP_GSYM(ncache), name);
 
 	if (attrs != NULL) {
 		SEXP_t *nl, *nj;
@@ -513,15 +579,6 @@ int probe_obj_setstatus(SEXP_t * obj, oval_syschar_status_t status)
 	return (-1);
 }
 
-#if 0
-static int probe_obj_setentstatus(SEXP_t * obj, const char *name, uint32_t n, oval_syschar_status_t status)
-{
-	_LOGCALL_;
-	/* TBI */
-	return (-1);
-}
-#endif
-
 char *probe_obj_getname(const SEXP_t * obj)
 {
 	return probe_ent_getname(obj);
@@ -576,6 +633,7 @@ int probe_cobj_add_item(SEXP_t *cobj, const SEXP_t *item)
 
 	lst = SEXP_listref_nth(cobj, 3);
 	item_cnt = SEXP_list_length(lst);
+
 	if (probe_cobj_memcheck(item_cnt) != 0) {
 		SEXP_t *msg;
 
@@ -903,6 +961,8 @@ SEXP_t *probe_ent_creat(const char *name, SEXP_t * attrs, SEXP_t * val, ...)
 		val = va_arg(ap, SEXP_t *);
 	}
 
+	va_end(ap);
+
 	return (ent_list);
 }
 
@@ -913,7 +973,7 @@ SEXP_t *probe_ent_creat1(const char *name, SEXP_t * attrs, SEXP_t * val)
 	_LOGCALL_;
 
 	ent = SEXP_list_new(NULL);
-	ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
+	ns  = probe_ncache_ref (OSCAP_GSYM(ncache), name);
 
 	if (attrs != NULL) {
 		SEXP_t *nl, *nj;
@@ -1153,7 +1213,7 @@ oval_syschar_status_t probe_ent_getstatus(const SEXP_t * ent)
                 sta = (oval_syschar_status_t) SEXP_number_geti_32 (val);
                 SEXP_free (val);
         } else {
-                sta = OVAL_STATUS_EXISTS;
+                sta = SYSCHAR_STATUS_EXISTS;
         }
 
 	return (sta);
@@ -1353,7 +1413,7 @@ static int probe_cobj_memcheck(size_t item_cnt)
 		if (memusage (&mu) != 0)
 			return (-1);
 
-		if (sysinfo (&si) != 0)
+		if (oscap_sysinfo (&si) != 0)
 			return (-1);
 
 		c_ratio = (double)mu.mu_data/(double)((si.totalram * si.mem_unit) / 1024);
@@ -1391,9 +1451,10 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
         double  value_flt;
         bool    value_bool;
         bool    free_value = true;
+        bool    multiplied;
         int     value_i, multiply;
 
-        subtype_name = oval_subtype2str(item_subtype);
+        subtype_name = oval_subtype_to_str(item_subtype);
 
         if (subtype_name == NULL) {
                 dE("Invalid/Unknown subtype: %d\n", (int)item_subtype);
@@ -1417,6 +1478,7 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
         while (value_name != NULL) {
                 value_i    = 0;
                 multiply   = 1;
+                multiplied = false;
 		value_type = va_arg(ap, oval_datatype_t);
 
                 switch (value_type) {
@@ -1431,6 +1493,7 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
                 case OVAL_DATATYPE_STRING_M:
                         value_type = OVAL_DATATYPE_STRING;
                         value_stra = va_arg(ap, char **);
+                        multiplied = true;
                         multiply   = 0;
 
                         if (value_stra == NULL)
@@ -1475,6 +1538,8 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
                 case OVAL_DATATYPE_EVR_STRING:
                 case OVAL_DATATYPE_FILESET_REVISION:
                 case OVAL_DATATYPE_IOS_VERSION:
+		case OVAL_DATATYPE_IPV4ADDR:
+		case OVAL_DATATYPE_IPV6ADDR:
                 case OVAL_DATATYPE_VERSION:
                         value_str  = va_arg(ap, char *);
                         value_sexp = SEXP_string_new_r(&value_sexp_mem, value_str, strlen(value_str));
@@ -1490,7 +1555,7 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
                         return (NULL);
                 }
 
-                name_sexp = probe_ncache_ref(OSCAP_GSYM(encache), value_name);
+                name_sexp = probe_ncache_ref(OSCAP_GSYM(ncache), value_name);
 
                 while(value_i < multiply) {
                         entity = SEXP_list_new_r(&entity_mem, name_sexp, value_sexp + value_i, NULL);
@@ -1503,10 +1568,11 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
                                 if (free_value) {
                                         while(value_i < multiply)
                                                 SEXP_free_r(value_sexp + value_i++);
-                                        if (multiply > 1)
+                                        if (multiplied)
                                                 oscap_free(value_sexp);
                                 }
 
+				va_end(ap);
                                 return (NULL);
                         }
 
@@ -1524,12 +1590,79 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
 
                 SEXP_free(name_sexp);
 
-                if (multiply > 1)
+                if (multiplied)
                         oscap_free(value_sexp);
         skip:
                 value_name = va_arg(ap, const char *);
 		free_value = true;
         }
 
+	va_end(ap);
         return (item);
 }
+
+oval_operation_t probe_ent_getoperation(SEXP_t *entity, oval_operation_t default_op)
+{
+        oval_operation_t ret;
+        SEXP_t *aval;
+
+        if (entity == NULL) {
+                dW("entity == NULL\n");
+                return (OVAL_OPERATION_UNKNOWN);
+        }
+
+        aval = probe_ent_getattrval(entity, "operation");
+
+        if (aval == NULL) {
+                dW("Attribute \"operation\" not found. Using default.\n");
+                return (default_op);
+        }
+
+        if (!SEXP_numberp(aval)) {
+                dW("Invalid type\n");
+                SEXP_free(aval);
+                return (OVAL_OPERATION_UNKNOWN);
+        }
+
+        ret = SEXP_number_geti_32(aval);
+        SEXP_free(aval);
+
+        return (ret);
+}
+
+int probe_item_add_msg(SEXP_t *item, oval_message_level_t msglvl, char *msgfmt, ...)
+{
+    va_list ap;
+    SEXP_t  lvl_sexp, msg_sexp;
+    SEXP_t *attrs;
+    char    msg_buffer[1024];
+    int     msg_length;
+
+    va_start(ap, msgfmt);
+
+    msg_length = vsnprintf(msg_buffer, sizeof msg_buffer, msgfmt, ap);
+
+    if (msg_length < 0) {
+	dE("vsnprintf failed! errno=%u, %s.\n", errno, strerror(errno));
+	return (-1);
+    }
+
+    if ((size_t)msg_length >= sizeof msg_buffer) {
+	dE("message too long!\n");
+	return (-1);
+    }
+
+    va_end(ap);
+
+    SEXP_number_newu_32_r(&lvl_sexp, msglvl);
+    SEXP_string_new_r(&msg_sexp, msg_buffer, msg_length);
+
+    attrs = probe_attr_creat("level", &lvl_sexp, NULL);
+    probe_item_ent_add(item, "message", attrs, &msg_sexp);
+
+    SEXP_free_r(&lvl_sexp);
+    SEXP_free_r(&msg_sexp);
+    SEXP_free(attrs);
+
+    return (0);
+}
