@@ -41,6 +41,7 @@
 #elif defined USE_REGEX_POSIX
 #include <regex.h>
 #endif
+#include <arpa/inet.h>
 
 #include "common/debug_priv.h"
 #include "entcmp.h"
@@ -476,6 +477,233 @@ oval_result_t probe_ent_cmp_string(SEXP_t * val1, SEXP_t * val2, oval_operation_
 	return result;
 }
 
+static oval_result_t probe_ent_cmp_ipv4addr(SEXP_t *val1, SEXP_t *val2, oval_operation_t op)
+{
+	oval_result_t result = OVAL_RESULT_ERROR;
+	char *s, *pfx;
+	int nm1, nm2;
+	struct in_addr addr1, addr2;
+
+        memset(&addr1, 0, sizeof (struct in_addr));
+        memset(&addr2, 0, sizeof (struct in_addr));
+
+	s = SEXP_string_cstr(val1);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		int cnt;
+		char nm[4];
+
+		*pfx++ = '\0';
+		cnt = sscanf(pfx, "%hhu.%hhu.%hhu.%hhu", &nm[0], &nm[1], &nm[2], &nm[3]);
+		if (cnt > 1) { /* netmask */
+			nm1 = (nm[0] << 24) + (nm[1] << 16) + (nm[2] << 8) + nm[3];
+		} else { /* prefix */
+			nm1 = (~0) << (32 - nm[0]);
+		}
+	} else {
+		nm1 = ~0;
+	}
+	if (inet_pton(AF_INET, s, &addr1) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	oscap_free(s);
+	s = SEXP_string_cstr(val2);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		int cnt;
+		char nm[4];
+
+		*pfx++ = '\0';
+		cnt = sscanf(pfx, "%hhu.%hhu.%hhu.%hhu", &nm[0], &nm[1], &nm[2], &nm[3]);
+		if (cnt > 1) { /* netmask */
+			nm2 = (nm[0] << 24) + (nm[1] << 16) + (nm[2] << 8) + nm[3];
+		} else { /* prefix */
+			nm2 = (~0) << (32 - nm[0]);
+		}
+	} else {
+		nm2 = ~0;
+	}
+	if (inet_pton(AF_INET, s, &addr2) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	switch (op) {
+	case OVAL_OPERATION_EQUALS:
+		if (!memcmp(&addr1, &addr2, sizeof(struct in_addr)) && nm1 == nm2)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_NOT_EQUAL:
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) || nm1 != nm2)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUBSET_OF:
+		if (nm1 <= nm2) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_GREATER_THAN:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) > 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_GREATER_THAN_OR_EQUAL:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) >= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUPERSET_OF:
+		if (nm1 >= nm2) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_LESS_THAN:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) < 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_LESS_THAN_OR_EQUAL:
+		addr1.s_addr &= nm1;
+		addr2.s_addr &= nm2;
+		if (memcmp(&addr1, &addr2, sizeof(struct in_addr)) <= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	default:
+		dE("Unexpected compare operation: %d.\n", op);
+	}
+
+ cleanup:
+	oscap_free(s);
+
+	return (result);
+}
+
+static void mask_v6_addrs(struct in6_addr *addr1, int p1len, struct in6_addr *addr2, int p2len)
+{
+	int i;
+
+	for (i = 0; i < 128; ++i) {
+		addr1->s6_addr[i / 8] &= (((i < p1len) ? 1 : 0) << (i % 8));
+		addr2->s6_addr[i / 8] &= (((i < p2len) ? 1 : 0) << (i % 8));
+	}
+}
+
+static oval_result_t probe_ent_cmp_ipv6addr(SEXP_t *val1, SEXP_t *val2, oval_operation_t op)
+{
+	oval_result_t result = OVAL_RESULT_ERROR;
+	char *s, *pfx;
+	int p1len, p2len;
+	struct in6_addr addr1, addr2;
+
+	s = SEXP_string_cstr(val1);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		*pfx++ = '\0';
+		p1len = strtol(pfx, NULL, 10);
+	} else {
+		p1len = 128;
+	}
+	if (inet_pton(AF_INET6, s, &addr1) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	oscap_free(s);
+	s = SEXP_string_cstr(val2);
+	pfx = strchr(s, '/');
+	if (pfx) {
+		*pfx++ = '\0';
+		p2len = strtol(pfx, NULL, 10);
+	} else {
+		p2len = 128;
+	}
+	if (inet_pton(AF_INET6, s, &addr2) <= 0) {
+		dW("inet_pton() failed.\n");
+		goto cleanup;
+	}
+
+	switch (op) {
+	case OVAL_OPERATION_EQUALS:
+		if (!memcmp(&addr1, &addr2, sizeof(struct in6_addr)) && p1len == p2len)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_NOT_EQUAL:
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) || p1len != p2len)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUBSET_OF:
+		if (p1len <= p2len) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_GREATER_THAN:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) > 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_GREATER_THAN_OR_EQUAL:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) >= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_SUPERSET_OF:
+		if (p1len >= p2len) {
+			result = OVAL_RESULT_FALSE;
+			break;
+		}
+		/* FALLTHROUGH */
+	case OVAL_OPERATION_LESS_THAN:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) < 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	case OVAL_OPERATION_LESS_THAN_OR_EQUAL:
+		mask_v6_addrs(&addr1, p1len, &addr2, p2len);
+		if (memcmp(&addr1, &addr2, sizeof(struct in6_addr)) <= 0)
+			result = OVAL_RESULT_TRUE;
+		else
+			result = OVAL_RESULT_FALSE;
+		break;
+	default:
+		dE("Unexpected compare operation: %d.\n", op);
+	}
+
+ cleanup:
+	oscap_free(s);
+
+	return result;
+}
+
 static oval_result_t probe_ent_cmp(SEXP_t * ent, SEXP_t * val2)
 {
 	oval_operation_t op;
@@ -501,12 +729,12 @@ static oval_result_t probe_ent_cmp(SEXP_t * ent, SEXP_t * val2)
 		is_var = 0;
 	}
 
+	dtype = probe_ent_getdatatype(ent);
 	stmp = probe_ent_getattrval(ent, "operation");
 	if (stmp == NULL)
 		op = OVAL_OPERATION_EQUALS;
 	else
 		op = SEXP_number_geti_32(stmp);
-
         SEXP_free(stmp);
 	res_lst = SEXP_list_new(NULL);
 
@@ -522,8 +750,6 @@ static oval_result_t probe_ent_cmp(SEXP_t * ent, SEXP_t * val2)
 			return OVAL_RESULT_ERROR;
 		}
 
-		dtype = probe_ent_getdatatype(ent);
-
 		switch (dtype) {
 		case OVAL_DATATYPE_BINARY:
 			ores = probe_ent_cmp_binary(val1, val2, op);
@@ -552,6 +778,12 @@ static oval_result_t probe_ent_cmp(SEXP_t * ent, SEXP_t * val2)
 		case OVAL_DATATYPE_STRING:
 			ores = probe_ent_cmp_string(val1, val2, op);
 			break;
+		case OVAL_DATATYPE_IPV4ADDR:
+			ores = probe_ent_cmp_ipv4addr(val1, val2, op);
+			break;
+		case OVAL_DATATYPE_IPV6ADDR:
+			ores = probe_ent_cmp_ipv6addr(val1, val2, op);
+			break;
 		default:
 			ores = OVAL_RESULT_ERROR;
 			_D("Unexpected data type: %d\n", dtype);
@@ -690,10 +922,10 @@ oval_result_t probe_entste_cmp(SEXP_t * ent_ste, SEXP_t * ent_itm)
 
 	item_status = probe_ent_getstatus(ent_itm);
 	switch (item_status) {
-	case OVAL_STATUS_DOESNOTEXIST:
+	case SYSCHAR_STATUS_DOES_NOT_EXIST:
 		return OVAL_RESULT_FALSE;
-	case OVAL_STATUS_ERROR:
-	case OVAL_STATUS_NOTCOLLECTED:
+	case SYSCHAR_STATUS_ERROR:
+	case SYSCHAR_STATUS_NOT_COLLECTED:
 		return OVAL_RESULT_ERROR;
 	default:
 		break;
@@ -734,7 +966,9 @@ oval_result_t probe_entobj_cmp(SEXP_t * ent_obj, SEXP_t * val)
 	}
 
 	ores = probe_ent_cmp(ent_obj, val);
+#if defined(OSCAP_VERBOSE_DEBUG)
 	dI("ores: %d, '%s'.\n", ores, oval_result_get_text(ores));
+#endif
 	if (ores == OVAL_RESULT_NOT_EVALUATED)
 		return OVAL_RESULT_FALSE;
 	return ores;
@@ -828,6 +1062,10 @@ oval_result_t probe_ent_result_bychk(SEXP_t * res_lst, oval_check_t check)
 			result = OVAL_RESULT_NOT_EVALUATED;
 		}
 		break;
+	case OVAL_CHECK_NONE_EXIST:
+		dW("The 'none exist' CheckEnumeration value has been deprecated. "
+		   "Converted to check='none satisfy'.\n");
+		/* FALLTHROUGH */
 	case OVAL_CHECK_NONE_SATISFY:
 		if (ores.true_cnt > 0) {
 			result = OVAL_RESULT_FALSE;
