@@ -64,10 +64,18 @@ void *probe_worker_runfn(void *arg)
 
                 return (NULL);
 	} else {
+                SEXP_t *items;
+
 		dI("probe thread deleted\n");
 
 		obj = SEAP_msg_get(pair->pth->msg);
 		oid = probe_obj_getattrval(obj, "id");
+                items = probe_cobj_get_items(probe_res);
+
+                if (items != NULL) {
+                        SEXP_list_sort(items, SEXP_refcmp);
+                        SEXP_free(items);
+                }
 
 		if (probe_rcache_sexp_add(pair->probe->rcache, oid, probe_res) != 0) {
 			/* TODO */
@@ -131,67 +139,6 @@ probe_worker_t *probe_worker_new(void)
 	return (pth);
 }
 
-#if 0
-/**
- * Create a new thread for handling a message with ID `sid'. This thread will be registered by
- * the thread manager. The handler is defined by the function `function' and `arg' is the user
- * pointer which will be passed to this function. The `arg_free' function is called only if the
- * thread is canceled before returing from the handler function.
- */
-int probe_worker_create(probe_t *probe, SEAP_msgid_t sid, SEXP_t * (*msg_handler)(SEAP_msg_t *, int *), SEAP_msg_t *msg)
-{
-	pthread_attr_t   pth_attr;
-	probe_pmpair_t  *pair;
-
-	if (mgr == NULL || msg_handler == NULL)
-		return (-1);
-
-	if (pthread_attr_init(&pth_attr) != 0)
-		return (-1);
-
-	if (pthread_attr_setdetachstate(&pth_attr, PTHREAD_CREATE_DETACHED) != 0) {
-		pthread_attr_destroy(&pth_attr);
-		return (-1);
-	}
-
-	pair = oscap_talloc(probe_pmpair_t);
-
-	pair->mgr      = mgr;
-	pair->pth      = probe_thread_new();
-	pair->pth->sid = sid;
-	pair->pth->msg = msg;
-	pair->pth->msg_handler = msg_handler;
-
-	if (rbt_i32_add(mgr->threads, sid, pair->pth, NULL) != 0) {
-		/*
-		 * Getting here means that there is already a
-		 * thread handling the message with the given
-		 * ID.
-		 */
-		pthread_attr_destroy(&pth_attr);
-		oscap_free(pair->pth);
-		oscap_free(pair);
-
-		return (-1);
-	}
-
-	if (pthread_create(&pair->pth->tid, &pth_attr, &probe_thread_runfn, pair) != 0) {
-		pthread_attr_destroy(&pth_attr);
-
-		if (rbt_i32_del(pair->mgr->threads, pair->pth->sid, NULL) != 0) {
-			/* ... do something ... */
-		}
-
-		oscap_free(pair);
-		return (-1);
-	}
-
-	pthread_attr_destroy(&pth_attr);
-
-	return (0);
-}
-#endif /* 0 */
-
 struct probe_varref_ctx {
 	SEXP_t *pi2;
 	unsigned int ent_cnt;
@@ -459,80 +406,6 @@ static SEXP_t *probe_prepare_filters(probe_t *probe, SEXP_t *obj)
 	return filters;
 }
 
-bool probe_item_filtered(SEXP_t *item, SEXP_t *filters)
-{
-	bool filtered = false;
-	SEXP_t *filter, *ste;
-
-	SEXP_list_foreach(filter, filters) {
-		SEXP_t *felm, *ste_res, *r0;
-		oval_result_t ores;
-		oval_operator_t oopr;
-		oval_filter_action_t ofact;
-
-		r0 = SEXP_list_first(filter);
-		ofact = SEXP_number_getu(r0);
-		SEXP_free(r0);
-		ste = SEXP_list_nth(filter, 2);
-		ste_res = SEXP_list_new(NULL);
-
-		SEXP_sublist_foreach(felm, ste, 2, SEXP_LIST_END) {
-			SEXP_t *ielm, *elm_res;
-			char *elm_name;
-			oval_check_t ochk;
-			int i;
-
-			elm_res = SEXP_list_new(NULL);
-			elm_name = probe_ent_getname(felm);
-
-			for (i = 1;; ++i) {
-				ielm = probe_obj_getent(item, elm_name, i);
-
-				if (ielm == NULL)
-					break;
-
-				ores = probe_entste_cmp(felm, ielm);
-				SEXP_list_add(elm_res, r0 = SEXP_number_newi_32(ores));
-
-				SEXP_free(ielm);
-				SEXP_free(r0);
-			}
-
-			oscap_free(elm_name);
-			r0 = probe_ent_getattrval(felm, "entity_check");
-
-			if (r0 == NULL)
-				ochk = OVAL_CHECK_ALL;
-			else
-				ochk = SEXP_number_geti_32(r0);
-
-			SEXP_free(r0);
-
-			ores = probe_ent_result_bychk(elm_res, ochk);
-			SEXP_list_add(ste_res, r0 = SEXP_number_newi_32(ores));
-			SEXP_free(r0);
-			SEXP_free(elm_res);
-		}
-
-		r0 = probe_ent_getattrval(ste, "operator");
-		if (r0 == NULL)
-			oopr = OVAL_OPERATOR_AND;
-		else
-			oopr = SEXP_number_geti_32(r0);
-		ores = probe_ent_result_byopr(ste_res, oopr);
-		SEXP_vfree(ste, ste_res, r0, NULL);
-
-		if ((ores == OVAL_RESULT_TRUE && ofact == OVAL_FILTER_ACTION_EXCLUDE)
-		    || (ores == OVAL_RESULT_FALSE && ofact == OVAL_FILTER_ACTION_INCLUDE)) {
-			filtered = true;
-			SEXP_free(filter);
-			break;
-		}
-	}
-
-	return filtered;
-}
-
 /**
  * Combine two collections of items using an operation.
  * @param cobj1 item collection
@@ -540,119 +413,125 @@ bool probe_item_filtered(SEXP_t *item, SEXP_t *filters)
  * @param op operation
  * @return the result of the operation
  */
-static SEXP_t *probe_set_combine(SEXP_t *cobj1, SEXP_t *cobj2, oval_setobject_operation_t op)
+static SEXP_t *probe_set_combine(SEXP_t *cobj0, SEXP_t *cobj1, oval_setobject_operation_t op)
 {
-	bool append;
-	SEXP_t *res_items, *item1, *item2, *id1, *id2;
-	SEXP_t *res_cobj, *item_lst1, *item_lst2;
-	oval_syschar_collection_flag_t flag1, flag2, res_flag;
+        SEXP_t *set0, *set1, *res_cobj;
+        register int cmp;
+        register SEXP_t *item0, *item1, *res;
+        register SEXP_list_it *sit0, *sit1;
+	oval_syschar_collection_flag_t res_flag;
 
-	_LOGCALL_;
-
-	if (cobj1 == NULL)
-		return SEXP_ref(cobj2);
-	if (cobj2 == NULL)
+	if (cobj0 == NULL)
 		return SEXP_ref(cobj1);
+	if (cobj1 == NULL)
+		return SEXP_ref(cobj0);
 
-	item_lst1 = probe_cobj_get_items(cobj1);
-	item_lst2 = probe_cobj_get_items(cobj2);
-	flag1 = probe_cobj_get_flag(cobj1);
-	flag2 = probe_cobj_get_flag(cobj2);
+        set0 = probe_cobj_get_items(cobj0);
+        set1 = probe_cobj_get_items(cobj1);
 
-	res_flag = probe_cobj_combine_flags(flag1, flag2, op);
-	res_items = SEXP_list_new(NULL);
+        /* prepare storage for results */
+        res = SEXP_list_new(NULL);
+        res_flag = probe_cobj_combine_flags(probe_cobj_get_flag(cobj0),
+                                            probe_cobj_get_flag(cobj1), op);
 
-	switch (op) {
-	case OVAL_SET_OPERATION_INTERSECTION:
-		SEXP_list_foreach(item1, item_lst1) {
-			id1 = probe_obj_getentval(item1, "id", 1);
-			append = false;
+        /* prepare iterators & first items */
+        sit0  = SEXP_list_it_new(set0);
+        sit1  = SEXP_list_it_new(set1);
+        item0 = SEXP_list_it_next(sit0);
+        item1 = SEXP_list_it_next(sit1);
 
-			SEXP_list_foreach(item2, item_lst2) {
-				id2 = probe_obj_getentval(item2, "id", 1);
+        /* perform the set operation */
+        switch(op) {
+        case OVAL_SET_OPERATION_UNION:
+                while (item0 != NULL && item1 != NULL) {
+                        cmp = SEXP_refcmp(item0, item1);
 
-				if (!SEXP_string_cmp(id1, id2)) {
-					append = true;
+                        if (cmp < 0) {
+                                SEXP_list_add(res, item0);
+                                item0 = SEXP_list_it_next(sit0);
+                        } else if (cmp > 0) {
+                                SEXP_list_add(res, item1);
+                                item1 = SEXP_list_it_next(sit1);
+                        } else {
+                                SEXP_list_add(res, item0);
+                                item0 = SEXP_list_it_next(sit0);
+                                item1 = SEXP_list_it_next(sit1);
+                        }
+                }
 
-					SEXP_free(id2);
-					SEXP_free(item2);
+                if (item0 != NULL) {
+                        do {
+                                SEXP_list_add(res, item0);
+                        } while((item0 = SEXP_list_it_next(sit0)) != NULL);
+                } else if (item1 != NULL) {
+                        do {
+                                SEXP_list_add(res, item1);
+                        } while((item1 = SEXP_list_it_next(sit1)) != NULL);
+                }
 
-					break;
-				}
+                break;
+        case OVAL_SET_OPERATION_INTERSECTION:
+                while (item0 != NULL && item1 != NULL) {
+                        cmp = SEXP_refcmp(item0, item1);
 
-				SEXP_free(id2);
-			}
+                        if (cmp < 0)
+                                item0 = SEXP_list_it_next(sit0);
+                        else if (cmp > 0)
+                                item1 = SEXP_list_it_next(sit1);
+                        else {
+                                SEXP_list_add(res, item0);
+                                item0 = SEXP_list_it_next(sit0);
+                                item1 = SEXP_list_it_next(sit1);
+                        }
+                }
 
-			if (append) {
-				SEXP_list_add(res_items, item1);
-			}
+                break;
+        case OVAL_SET_OPERATION_COMPLEMENT:
+                while (item0 != NULL && item1 != NULL) {
+                        cmp = SEXP_refcmp(item0, item1);
 
-			SEXP_free(id1);
-		}
-		break;
-	case OVAL_SET_OPERATION_UNION:
-		{
-			SEXP_t *lj;
+                        if (cmp < 0) {
+                                SEXP_list_add(res, item0);
+                                item0 = SEXP_list_it_next(sit0);
+                        } else if (cmp > 0) {
+                                item1 = SEXP_list_it_next(sit1);
+                        } else {
+                                item0 = SEXP_list_it_next(sit0);
+                                item1 = SEXP_list_it_next(sit1);
+                        }
+                }
 
-			lj = SEXP_list_join(res_items, item_lst2);
-			SEXP_free(res_items);
-			res_items = lj;
-			/* fall through */
-		}
-	case OVAL_SET_OPERATION_COMPLEMENT:
-		SEXP_list_foreach(item1, item_lst1) {
-			id1 = probe_obj_getentval(item1, "id", 1);
-			append = true;
+                if (item0 != NULL) {
+                        do {
+                                SEXP_list_add(res, item0);
+                        } while((item0 = SEXP_list_it_next(sit0)) != NULL);
+                }
 
-			SEXP_list_foreach(item2, item_lst2) {
-				id2 = probe_obj_getentval(item2, "id", 1);
+                break;
+        default:
+                dE("Unknown set operation: %d\n", op);
+                abort();
+        }
 
-				if (!SEXP_string_cmp(id1, id2)) {
-					append = false;
-
-					SEXP_free(id2);
-					SEXP_free(item2);
-
-					break;
-				}
-
-				SEXP_free(id2);
-			}
-
-			if (append) {
-				SEXP_list_add(res_items, item1);
-			}
-
-			SEXP_free(id1);
-		}
-		break;
-	default:
-		{
-			SEXP_t *r0, *r1;
-
-                        r0 = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                              "Unexpected set operation: %d.", op);
-			r1 = SEXP_list_new(r0, NULL);
-			res_cobj = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
-			SEXP_vfree(item_lst1, item_lst2, res_items, r0, r1, NULL);
-			return res_cobj;
-		}
-	}
+        SEXP_list_it_free(sit0);
+        SEXP_list_it_free(sit1);
 
 	/*
 	 * If the collected information is complete but all the items are
 	 * removed, the flag is set to SYSCHAR_FLAG_DOES_NOT_EXIST
 	 */
-	if (res_flag == SYSCHAR_FLAG_COMPLETE
-	    && SEXP_list_length(res_items) == 0)
+	if (res_flag == SYSCHAR_FLAG_COMPLETE && SEXP_list_length(res) == 0)
 		res_flag = SYSCHAR_FLAG_DOES_NOT_EXIST;
 
-	res_cobj = probe_cobj_new(res_flag, NULL, res_items);
-	SEXP_vfree(item_lst1, item_lst2, res_items, NULL);
+	res_cobj = probe_cobj_new(res_flag, NULL, res);
+
+	SEXP_free(set0);
+	SEXP_free(set1);
+	SEXP_free(res);
 
 	// todo: variables
 
-	return res_cobj;
+	return (res_cobj);
 }
 
 /**
@@ -677,11 +556,11 @@ static SEXP_t *probe_set_apply_filters(SEXP_t *cobj, SEXP_t *filters)
 		item_status = probe_ent_getstatus(item);
 
 		switch (item_status) {
-		case OVAL_STATUS_DOESNOTEXIST:
+		case SYSCHAR_STATUS_DOES_NOT_EXIST:
 			continue;
-		case OVAL_STATUS_ERROR:
+		case SYSCHAR_STATUS_ERROR:
 			break;
-		case OVAL_STATUS_NOTCOLLECTED:
+		case SYSCHAR_STATUS_NOT_COLLECTED:
 			{
 				SEXP_t *r0, *r1;
 
@@ -738,7 +617,7 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 	SEXP_t *op_val;
 	int op_num;
 
-	SEXP_t *r0, *r1, *result, *msg = NULL;
+	SEXP_t *r0, *r1, *result, *Omsg = NULL;
 
 	_LOGCALL_;
 
@@ -786,115 +665,119 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 
 	probe_set_foreach(member, set) {
 		if (probe_ent_getname_r(member, member_name, sizeof member_name) == 0) {
-                        msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, "probe_set_eval: Invalid set element: ptr=%p, type=%s.", member, SEXP_strtype(member));
+                        Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+						"probe_set_eval: Invalid set element: ptr=%p, type=%s.", member, SEXP_strtype(member));
 			goto eval_fail;
 		}
 
-		switch (member_name[0]) {
-			CASE('s', "et") {
-				if (s_subset_i < 2) {
-					s_subset[s_subset_i] = probe_set_eval(probe, member, depth + 1);
+		if (strcmp("set", member_name) == 0) {
+			/*
+			 * Handle a (sub)set entity
+			 */
+			if (s_subset_i < 2) {
+				s_subset[s_subset_i] = probe_set_eval(probe, member, depth + 1);
 
-					if (s_subset[s_subset_i] == NULL) {
-                                                msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                                       "probe_set_eval: Recursive set evaluation failed: m=%p, d=%zu.",
-                                                                       member, depth + 1);
-						goto eval_fail;
-					}
+				if (s_subset[s_subset_i] == NULL) {
+					Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+								"probe_set_eval: Recursive set evaluation failed: m=%p, d=%zu.",
+								member, depth + 1);
+					goto eval_fail;
+				}
 
-					++s_subset_i;
+				++s_subset_i;
+			} else {
+				Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+							"probe_set_eval: More than 2 \"set\".");
+				goto eval_fail;
+			}
+		} else if (strcmp("obj_ref", member_name) == 0) {
+			/*
+			 * Handle an object reference
+			 */
+			SEXP_t *OID; /**< OVAL object ID */
+			SEXP_t *objres; /**< Result of the evaluation */
+			char    OID_cstr[128];
+
+			dI("Handling object_reference\n");
+
+			if ((OID = probe_ent_getval(member)) == NULL) {
+				Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+							"%s: missing object_reference entity value!", __FUNCTION__);
+				goto eval_fail;
+			}
+#ifndef NDEBUG
+			SEXP_string_cstr_r(OID, OID_cstr, sizeof OID_cstr);
+			dI("Looking for the result in cache: OID=%s\n", OID_cstr);
+#endif
+			if ((objres = probe_rcache_sexp_get(probe->rcache, OID)) == NULL) {
+				dI("MISS => requesting object evaluation from the library\n");
+
+				objres = probe_obj_eval(probe, OID);
+
+				dI("EVAL: result=%p\n", objres);
+				if (objres != NULL) {
+					dO(OSCAP_DEBUGOBJ_SEXP, objres);
 				} else {
-                                        msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                               "probe_set_eval: More than 2 \"set\".");
+					SEXP_string_cstr_r(OID, OID_cstr, sizeof OID_cstr);
+					Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+								"%s: evaluation failed: OID=%s", __FUNCTION__, OID_cstr);
+					SEXP_free(OID);
 					goto eval_fail;
 				}
 			}
-			break;
 
-			CASE('o', "bj_ref") {
-				SEXP_t *id, *res;
+			SEXP_free(OID);
 
-				id = probe_ent_getval(member);
-
-				if (id == NULL) {
-                                        msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                              "probe_set_eval: set=%p: Missing obj_ref value.",
-                                                              set);
-					goto eval_fail;
-				}
-
-				res = probe_rcache_sexp_get(probe->rcache, id);
-
-				if (res == NULL) {
-					/* cache miss */
-					res = probe_obj_eval(probe, id);
-
-					if (res == NULL) {
-						char *tmp = SEXP_string_cstr(id);
-
-                                                msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                                       "probe_set_eval: obj=%s: Evaluation failed.",
-                                                                       tmp);
-						oscap_free(tmp);
-						SEXP_free(id);
-						goto eval_fail;
-					}
-				}
-
-				SEXP_free(id);
-
-				if (o_subset_i < 2) {
-					o_subset[o_subset_i] = res;
-					++o_subset_i;
-				} else {
-                                        msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                               "probe_set_eval: More than 2 obj_refs.");
-					SEXP_free(res);
-					goto eval_fail;
-				}
+			if (o_subset_i < 2) {
+				o_subset[o_subset_i] = objres;
+				++o_subset_i;
+			} else {
+				Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+							"%s: more than 2 obj_refs.", __FUNCTION__);
+				SEXP_free(objres);
+				goto eval_fail;
 			}
-			break;
+		} else if (strcmp("filter", member_name) == 0) {
+			/*
+			 * Retrieve cached filters (states), remember unavailable ones so that
+			 * we can fetch them all in one request.
+			 */
+			SEXP_t *SID, *action, *state;
 
-			CASE('f', "ilter") {
-				SEXP_t *id, *act, *ste;
-
-				id = probe_ent_getval(member);
-				if (id == NULL) {
-					msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                               "probe_set_eval: set=%p: Missing filter value.", set);
-					goto eval_fail;
-				}
-
-				act = probe_ent_getattrval(member, "action");
-				if (act == NULL) {
-					msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                               "probe_set_eval: set=%p: Missing filter action.", set);
-					SEXP_free(id);
-					goto eval_fail;
-				}
-
-				ste = probe_rcache_sexp_get(probe->rcache, id);
-
-				if (ste == NULL) {
-					SEXP_list_add(filters_req, id);
-					r0 = SEXP_list_new(act, id, NULL);
-					SEXP_list_add(filters_u, r0);
-				} else {
-					r0 = SEXP_list_new(act, ste, NULL);
-					SEXP_list_add(filters_a, r0);
-					SEXP_free(ste);
-				}
-
-				SEXP_vfree(id, act, r0, NULL);
+			if ((SID = probe_ent_getval(member)) == NULL) {
+				Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+							"%s: missing filter entity value.", __FUNCTION__);
+				goto eval_fail;
 			}
-			break;
-		default:
-			msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                               "probe_set_eval: Unexpected set element: %s.", member_name);
-			goto eval_fail;
+
+			if ((action = probe_ent_getattrval(member, "action")) == NULL) {
+				Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+							"%s: missing filter action.", __FUNCTION__);
+				SEXP_free(SID);
+				goto eval_fail;
+			}
+
+			if ((state = probe_rcache_sexp_get(probe->rcache, SID)) == NULL) {
+				/* MISS - remember the ID */
+				SEXP_list_add(filters_req, SID);
+				r0 = SEXP_list_new(action, SID, NULL);
+				SEXP_list_add(filters_u, r0);
+			} else {
+				/* HIT */
+				r0 = SEXP_list_new(action, state, NULL);
+				SEXP_list_add(filters_a, r0);
+				SEXP_free(state);
+			}
+
+			SEXP_free(SID);
+			SEXP_free(action);
+			SEXP_free(r0);
+		} else {
+			/*
+			 * Unexpected entity
+			 */
+			abort();
 		}
-#undef CASE
-
 	}
 
 	member = NULL;
@@ -903,8 +786,8 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 	result = probe_ste_fetch(probe, filters_req);
 
 	if (result == NULL) {
-                msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR,
-                                      "probe_set_eval: Can't get unavailable filters.");
+                Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+					"%s: Can't get unavailable filters.", __FUNCTION__);
 		goto eval_fail;
 	}
 	SEXP_vfree(filters_req, result, NULL);
@@ -930,17 +813,41 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 
 #ifndef NDEBUG
 			if (s_subset[s_subset_i] == NULL) {
-                                msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
-                                                       "probe_set_eval: apply_filters returned NULL: set=%p, filters=%p.",
-                                                       o_subset[s_subset_i], filters_a);
+                                Omsg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
+							"%s: apply_filters returned NULL: set=%p, filters=%p.",
+							__FUNCTION__, o_subset[s_subset_i], filters_a);
 				goto eval_fail;
 			}
 #endif
 			SEXP_free(o_subset[s_subset_i]);
+                        o_subset[s_subset_i] = NULL;
 		}
-
 	}
 
+#ifndef NDEBUG
+        {
+                unsigned int i;
+
+                for (i = 0; i < s_subset_i; ++i) {
+                        if (s_subset[i] != NULL) {
+                                _I("=== s_subset[%d] ===\n", i);
+                                _SE(s_subset[i]);
+                                _I("\n");
+                        }
+                }
+
+                for (i = 0; i < o_subset_i; ++i) {
+                        if (o_subset[i] != NULL) {
+                                _I("=== o_subset[%d] ===\n", i);
+                                _SE(o_subset[i]);
+                                _I("\n");
+                        }
+                }
+        }
+
+        _I("OP= %d\n", op_num);
+#endif
+
 	SEXP_free(filters_a);
 	result = probe_set_combine(s_subset[0], s_subset[1], op_num);
 
@@ -949,6 +856,10 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 	SEXP_free(s_subset[0]);
 	SEXP_free(s_subset[1]);
 
+        _I("=== RESULT ===\n");
+        _SE(result);
+        _I("\n");
+
 	return (result);
  eval_fail:
 	SEXP_free(member);
@@ -964,9 +875,9 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 	SEXP_free(filters_req);
 	SEXP_free(result);
 
-        r1 = SEXP_list_new(msg, NULL);
+        r1 = SEXP_list_new(Omsg, NULL);
 	result = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
-	SEXP_vfree(msg, r1, NULL);
+	SEXP_vfree(Omsg, r1, NULL);
 	return result;
 }
 
@@ -1001,10 +912,12 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
 		// todo: in case of an internal error set probe_ret accordingly
 		*ret = 0;
 	} else {
-		SEXP_t *varrefs, *filters;
+                struct probe_ctx pctx;
+		SEXP_t *varrefs;
 
 		/* simple object */
-		filters = probe_prepare_filters(probe, probe_in);
+                pctx.icache  = probe->icache;
+		pctx.filters = probe_prepare_filters(probe, probe_in);
 
 		if (OSCAP_GSYM(varref_handling))
 			varrefs = probe_obj_getent(probe_in, "varrefs", 1);
@@ -1012,8 +925,23 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
                         varrefs = NULL;
 
 		if (varrefs == NULL || !OSCAP_GSYM(varref_handling)) {
+                        /*
+                         * Prepare the collected object
+                         */
 			probe_out = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
-			*ret = probe_main(probe_in, probe_out, probe->probe_arg, filters);
+
+                        pctx.probe_in  = probe_in;
+                        pctx.probe_out = probe_out;
+                        /*
+                         * Run the main function of the probe implementation
+                         */
+			*ret = probe_main(&pctx, probe->probe_arg);
+
+                        /*
+                         * Synchronize
+                         */
+                        probe_icache_nop(probe->icache);
+
 			probe_cobj_compute_flag(probe_out);
 		} else {
 			/*
@@ -1025,7 +953,7 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
 			dI("handling varrefs in object\n");
 
 			if (probe_varref_create_ctx(probe_in, varrefs, &ctx) != 0) {
-				SEXP_vfree(varrefs, filters, probe_in, NULL);
+				SEXP_vfree(varrefs, pctx.filters, probe_in, NULL);
 				*ret = PROBE_EUNKNOWN;
 				return (NULL);
 			}
@@ -1034,9 +962,23 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
 
 			do {
 				SEXP_t *cobj, *r0;
-
+                                /*
+                                 * Prepare the collected object
+                                 */
 				cobj = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
-				*ret = probe_main(ctx->pi2, cobj, probe->probe_arg, filters);
+
+                                pctx.probe_in  = ctx->pi2;
+                                pctx.probe_out = cobj;
+                                /*
+                                 * Run the main function of the probe implementation
+                                 */
+				*ret = probe_main(&pctx, probe->probe_arg);
+
+                                /*
+                                 * Synchronize
+                                 */
+                                probe_icache_nop(probe->icache);
+
 				probe_cobj_compute_flag(cobj);
 				r0 = probe_out;
 				probe_out = probe_set_combine(r0, cobj, OVAL_SET_OPERATION_UNION);
@@ -1047,7 +989,7 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
 			probe_varref_destroy_ctx(ctx);
 		}
 
-                SEXP_free(filters);
+                SEXP_free(pctx.filters);
 	}
 
 	SEXP_free(probe_in);
