@@ -73,7 +73,7 @@
  */
 
 /**
- * Build a new item according to the scpecified format.
+ * Build a new item according to the specified format.
  * @param fmt the desired format
  */
 SEXP_t *probe_item_build(const char *fmt, ...);
@@ -172,7 +172,7 @@ SEXP_t *probe_attr_creat(const char *name, const SEXP_t * val, ...);
  */
 
 /**
- * Build a new object according to the scpecified format.
+ * Build a new object according to the specified format.
  * @param fmt the desired format
  */
 SEXP_t *probe_obj_build(const char *fmt, ...);
@@ -463,29 +463,50 @@ void probe_tfc54behaviors_canonicalize(SEXP_t **behaviors);
 #define PROBE_EFATAL   254	/**< Unrecoverable error */
 #define PROBE_EUNKNOWN 255	/**< Unknown/Unexpected error */
 
-#define OVAL_STATUS_UNKNOWN      SYSCHAR_STATUS_UNKNOWN
-#define OVAL_STATUS_ERROR        SYSCHAR_STATUS_ERROR
-#define OVAL_STATUS_EXISTS       SYSCHAR_STATUS_EXISTS
-#define OVAL_STATUS_DOESNOTEXIST SYSCHAR_STATUS_DOES_NOT_EXIST
-#define OVAL_STATUS_NOTCOLLECTED SYSCHAR_STATUS_NOT_COLLECTED
-
 #define PROBECMD_STE_FETCH 1 /**< State fetch command code */
 #define PROBECMD_OBJ_EVAL  2 /**< Object eval command code */
 #define PROBECMD_RESET     3 /**< Reset command code */
 
 void *probe_init(void) __attribute__ ((unused));
 void probe_fini(void *) __attribute__ ((unused));
-int probe_main(SEXP_t *, SEXP_t *, void *, SEXP_t *) __attribute__ ((nonnull(1, 2)));
+
+typedef struct probe_ctx probe_ctx;
+
+int probe_main(probe_ctx *, void *) __attribute__ ((nonnull(1)));
 
 #define PROBE_VARREF_HANDLING 0
 #define PROBE_RESULT_CACHING  1
 
 int probe_setoption(int option, ...);
 
-bool probe_item_filtered(SEXP_t *item, SEXP_t *filters);
+bool probe_item_filtered(const SEXP_t *item, const SEXP_t *filters);
 
 int probe_result_additem(SEXP_t *result, SEXP_t *item);
 
+/**
+ * Collect generated item (i.e. add it to the collected object)
+ * The function takes ownership of the item reference and takes
+ * care of freeing the item (i.e. don't call SEXP_free(item) after
+ * calling this function). The implementation of this function
+ * is placed in the `probe/icache.c' file.
+ */
+int probe_item_collect(probe_ctx *ctx, SEXP_t *item);
+
+/**
+ * Return reference to the input object. The reference counter
+ * is NOT incremented by this operation (i.e. don't call SEXP_free
+ * on the return value of this function). Implementation of this
+ * function is placed in the `' file.
+ */
+SEXP_t *probe_ctx_getobject(probe_ctx *ctx);
+
+/**
+ * Return reference to the output object (aka collected object).
+ * Reference counter is NOT incremented (see the description of
+ * probe_ctx_getobject above).
+ */
+SEXP_t *probe_ctx_getresult(probe_ctx *ctx);
+
 typedef struct {
         oval_datatype_t type;
         void           *value;
@@ -493,4 +514,58 @@ typedef struct {
 
 SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attributes[], ...);
 
+#define PROBE_ENT_AREF(ent, dst, attr_name, invalid_exp)		\
+	do {								\
+		if (((dst) = probe_ent_getattrval(ent, attr_name)) == NULL) { \
+			_F("Attribute `%s' is missing!\n", attr_name);	\
+			invalid_exp					\
+		}               					\
+	} while(0)
+
+#define PROBE_ENT_STRVAL(ent, dst, dstlen, invalid_exp, zerolen_exp)	\
+	do {							\
+		SEXP_t *___r;					\
+								\
+		if ((___r = probe_ent_getval(ent)) == NULL) {	\
+			_W("Entity has no value!\n");		\
+			invalid_exp				\
+		} else {					\
+			if (!SEXP_stringp(___r)) {		\
+				_F("Invalid type\n");		\
+				SEXP_free(___r);		\
+				invalid_exp			\
+			}					\
+			if (SEXP_string_length(___r) == 0) {	\
+				SEXP_free(___r);		\
+				zerolen_exp			\
+			} else {				\
+				SEXP_string_cstr_r(___r, dst, dstlen); \
+				SEXP_free(___r);		\
+			}					\
+		}						\
+	} while (0)
+
+#define PROBE_ENT_I32VAL(ent, dst, invalid_exp, nil_exp)        \
+	do {							\
+		SEXP_t *___r;					\
+								\
+		if ((___r = probe_ent_getval(ent)) == NULL) {	\
+			_W("Entity has no value!\n");		\
+			nil_exp;				\
+		} else {					\
+			if (!SEXP_numberp(___r)) {		\
+				_F("Invalid type\n");		\
+				SEXP_free(___r);		\
+				invalid_exp;			\
+			} else {				\
+				dst = SEXP_number_geti_32(___r);	\
+				SEXP_free(___r);                	\
+			}						\
+		}							\
+	} while (0)
+
 #endif				/* PROBE_API_H */
+
+oval_operation_t probe_ent_getoperation(SEXP_t *entity, oval_operation_t op);
+
+int probe_item_add_msg(SEXP_t *item, oval_message_level_t msglvl, char *msgfmt, ...);
