@@ -3,14 +3,16 @@
 #include "common/debug_priv.h"
 #include "common/assume.h"
 
-int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *unused, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *unused)
 {
         SEXP_t *item;
 
+        (void)unused;
+
         item = probe_item_create(OVAL_UNIX_DNSCACHE, NULL, NULL);
-        probe_item_setstatus(item, OVAL_STATUS_NOTCOLLECTED);
-        probe_cobj_add_item(probe_out, item);
-        SEXP_free(item);
+
+        probe_item_setstatus(item, SYSCHAR_STATUS_NOT_COLLECTED);
+        probe_item_collect(ctx, item);
 
         return (0);
 }
