@@ -70,6 +70,7 @@
 #include <string.h>
 #include <net/if.h>
 #include <net/if_arp.h>
+#include <arpa/inet.h>
 
 static int fd=-1;
 
@@ -116,20 +117,99 @@ static void get_l2_info(const struct ifaddrs *ifa, char **mp, char **tp)
 		mac_buf[0] = 0;
 }
 
-static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
+
+static void get_flags(const struct ifaddrs *ifa, char ***fp) {
+	static char *flags_buf[17];
+	int i = 0;
+
+	*fp = flags_buf;
+
+	if (ifa != NULL) {
+		/* follow values from net/if.h */
+		if (ifa->ifa_flags & IFF_UP) {
+			flags_buf[i] = "UP";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_BROADCAST) {
+			flags_buf[i] = "BROADCAST";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_DEBUG) {
+			flags_buf[i] = "DEBUG";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_LOOPBACK) {
+			flags_buf[i] = "LOOPBACK";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_POINTOPOINT) {
+			flags_buf[i] = "POINTOPOINT";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_NOTRAILERS) {
+			flags_buf[i] = "NOTRAILERS";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_RUNNING) {
+			flags_buf[i] = "RUNNING";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_NOARP) {
+			flags_buf[i] = "NOAPP";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_PROMISC) {
+			flags_buf[i] = "PROMISC";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_ALLMULTI) {
+			flags_buf[i] = "ALLMULTI";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_MASTER) {
+			flags_buf[i] = "MASTER";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_SLAVE) {
+			flags_buf[i] = "SLAVE";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_MULTICAST) {
+			flags_buf[i] = "MULTICAST";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_PORTSEL) {
+			flags_buf[i] = "PORTSEL";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_AUTOMEDIA) {
+			flags_buf[i] = "AUTOMEDIA";
+			i++;
+		}
+		if (ifa->ifa_flags & IFF_DYNAMIC) {
+			flags_buf[i] = "DYNAMIC";
+			i++;
+		}
+	}
+	flags_buf[i] = NULL;
+
+}
+
+static int get_ifs(SEXP_t *name_ent, probe_ctx *ctx)
 {
 	struct ifaddrs *ifaddr, *ifa;
 	int family, rc=1;
-	char host[NI_MAXHOST], broad[NI_MAXHOST], mask[NI_MAXHOST], *mac, *type;
+	char host[NI_MAXHOST], broad[NI_MAXHOST], mask[NI_MAXHOST], *mac, *type, **flags;
+	oval_datatype_t address_type;
 	SEXP_t *item;
 
 	if (getifaddrs(&ifaddr) == -1) {
 		SEXP_t *msg;
 
 		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "getifaddrs() failed.");
-		probe_cobj_add_msg(probe_out, msg);
+		probe_cobj_add_msg(probe_ctx_getresult(ctx), msg);
 		SEXP_free(msg);
-		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
+		probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
 
 		return rc;
 	}
@@ -139,9 +219,9 @@ static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
 		SEXP_t *msg;
 
 		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "socket() failed.");
-		probe_cobj_add_msg(probe_out, msg);
+		probe_cobj_add_msg(probe_ctx_getresult(ctx), msg);
 		SEXP_free(msg);
-		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
+		probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
 
 		goto leave1;
 	}
@@ -166,36 +246,82 @@ static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
 		SEXP_free(sname);
 
 		get_l2_info(ifa, &mac, &type);
-		rc = getnameinfo(ifa->ifa_addr, (family == AF_INET) ?
-			sizeof(struct sockaddr_in) :
-			sizeof(struct sockaddr_in6), host, NI_MAXHOST,
-			NULL, 0, NI_NUMERICHOST);
-		if (rc) {
-			SEXP_t *msg;
+		get_flags(ifa, &flags);
 
-			msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "getnameinfo() failed.");
-			probe_cobj_add_msg(probe_out, msg);
-			SEXP_free(msg);
-			probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
-			rc = 1;
+/* The inet_addr entity is the IP address of the specific interface.
+ * Note that the IP address can be IPv4 or IPv6. If the IP address is an IPv6 address,
+ * this entity should be expressed as an IPv6 address prefix using CIDR notation and
+ * the netmask entity should not be collected.
+ */
+		if (family == AF_INET) {
+			rc = getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in),
+					host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
+			if (rc) {
+				SEXP_t *msg;
+
+				msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "getnameinfo() failed.");
+				probe_cobj_add_msg(probe_ctx_getresult(ctx), msg);
+				SEXP_free(msg);
+				probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
+				rc = 1;
+
+				goto leave2;
+			}
+			address_type = OVAL_DATATYPE_IPV4ADDR;
+
+			rc = getnameinfo(ifa->ifa_netmask, sizeof(struct sockaddr_in),
+					mask, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
+			if (rc) {
+				SEXP_t *msg;
+
+				msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "getnameinfo() failed.");
+				probe_cobj_add_msg(probe_ctx_getresult(ctx), msg);
+				SEXP_free(msg);
+				probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
+				rc = 1;
+
+				goto leave2;
+			}
 
-			goto leave2;
 		}
-		rc = getnameinfo(ifa->ifa_netmask, (family == AF_INET) ?
-			sizeof(struct sockaddr_in) :
-			sizeof(struct sockaddr_in6), mask, NI_MAXHOST,
-			NULL, 0, NI_NUMERICHOST);
-		if (rc) {
-			SEXP_t *msg;
+		else {
+			struct sockaddr_in6 *sin6p;
+			char host_tmp[NI_MAXHOST];
+			int bit, byte, prefix = 0;
+			u_int32_t tmp;
 
-			msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "getnameinfo() failed.");
-			probe_cobj_add_msg(probe_out, msg);
-			SEXP_free(msg);
-			probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
-			rc = 1;
+			sin6p = (struct sockaddr_in6 *) ifa->ifa_addr;
 
-			goto leave2;
+			if (! inet_ntop(family, (const void *)&sin6p->sin6_addr, host, NI_MAXHOST)) {
+				SEXP_t *msg;
+
+				msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "inet_ntop() failed.");
+				probe_cobj_add_msg(probe_ctx_getresult(ctx), msg);
+				SEXP_free(msg);
+				probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
+				rc = 1;
+
+				goto leave2;
+			}
+
+			address_type = OVAL_DATATYPE_IPV6ADDR;
+			*mask = '\0';
+
+			/* count prefix */
+			sin6p = (struct sockaddr_in6 *) ifa->ifa_netmask;
+
+			for (byte = 0; byte < 4 && sin6p->sin6_addr.s6_addr32[byte] == 0xffffffff; byte++) {
+			        prefix += 32;
+			}
+			if (byte < 4) {
+			        tmp = ntohl(sin6p->sin6_addr.s6_addr32[byte]);
+			        for (bit = 31; tmp & (1 << bit); bit--)
+			                prefix++;
+			}
+			snprintf(host_tmp, NI_MAXHOST, "%s/%d", host, prefix);
+			strncpy(host, host_tmp, NI_MAXHOST);
 		}
+
 		if (family == AF_INET && ifa->ifa_flags & IFF_BROADCAST) {
 			rc = getnameinfo(ifa->ifa_broadaddr, (family == AF_INET) ?
 					 sizeof(struct sockaddr_in) :
@@ -205,9 +331,9 @@ static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
 				SEXP_t *msg;
 
 				msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "getnameinfo() failed.");
-				probe_cobj_add_msg(probe_out, msg);
+				probe_cobj_add_msg(probe_ctx_getresult(ctx), msg);
 				SEXP_free(msg);
-				probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
+				probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
 				rc = 1;
 
 				goto leave2;
@@ -219,13 +345,13 @@ static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
                                          "name",           OVAL_DATATYPE_STRING, ifa->ifa_name,
                                          "type",           OVAL_DATATYPE_STRING, type,
                                          "hardware_addr",  OVAL_DATATYPE_STRING, mac,
-                                         "inet_addr",      OVAL_DATATYPE_STRING, host,
-                                         "broadcast_addr", OVAL_DATATYPE_STRING, broad,
-                                         "netmask",        OVAL_DATATYPE_STRING, mask,
+                                         "inet_addr",      address_type, host,
+                                         "broadcast_addr", address_type, broad,
+                                         "netmask",        address_type, mask,
+                                         "flag",          OVAL_DATATYPE_STRING_M, flags,
                                          NULL);
 
-		probe_cobj_add_item(probe_out, item);
-                SEXP_free(item);
+                probe_item_collect(ctx, item);
 	}
 
 	rc = 0;
@@ -236,7 +362,7 @@ leave1:
 	return rc;
 }
 #else
-static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
+static int get_ifs(SEXP_t *name_ent, probe_ctx *ctx)
 {
 	/* todo */
 	SEXP_t *item;
@@ -249,29 +375,24 @@ static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
                                  "netmask",        OVAL_DATATYPE_STRING, "255.255.255.0",
                                  NULL);
 
-	probe_cobj_add_item(probe_out, item);
-        SEXP_free(item);
+        probe_item_collect(ctx, item);
 
 	return (0);
 }
 #endif
 
-int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *arg)
 {
 	SEXP_t *name_ent;
 
-        (void)filters;
+        (void)arg;
 
-	if (probe_in == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
-
-	name_ent = probe_obj_getent(probe_in, "name", 1);
+	name_ent = probe_obj_getent(probe_ctx_getobject(ctx), "name", 1);
 	if (name_ent == NULL) {
 		return PROBE_ENOELM;
 	}
 
-	get_ifs(name_ent, probe_out);
+	get_ifs(name_ent, ctx);
 	SEXP_free(name_ent);
 
 	return 0;
