@@ -331,7 +331,7 @@ static int eval_data(const char *type, const char *local_address,
 	return 1;
 }
 
-static void report_finding(struct result_info *res, llist *l, SEXP_t *probe_out)
+static void report_finding(struct result_info *res, llist *l, probe_ctx *ctx)
 {
         SEXP_t *item;
         SEXP_t se_lport_mem, se_rport_mem, se_lfull_mem, se_ffull_mem, se_uid_mem;
@@ -340,23 +340,20 @@ static void report_finding(struct result_info *res, llist *l, SEXP_t *probe_out)
         item = probe_item_create(OVAL_LINUX_INET_LISTENING_SERVERS, NULL,
                                  "protocol",             OVAL_DATATYPE_STRING,  res->proto,
                                  "local_address",        OVAL_DATATYPE_STRING,  res->laddr,
-                                 "local_port",           OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_lport_mem,
-                                                                                                   "%u", res->lport),
+				 "local_port",           OVAL_DATATYPE_SEXP, SEXP_number_newu_64_r(&se_lport_mem, res->lport),
                                  "local_full_address",   OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_lfull_mem,
                                                                                                    "%s:%u", res->laddr, res->lport),
                                  "program_name",         OVAL_DATATYPE_STRING,  n->cmd,
                                  "foreign_address",      OVAL_DATATYPE_STRING,  res->raddr,
-                                 "foreign_port",         OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_rport_mem,
-                                                                                                   "%u", res->rport),
+				 "foreign_port",         OVAL_DATATYPE_SEXP, SEXP_number_newu_64_r(&se_rport_mem, res->rport),
                                  "foreign_full_address", OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_ffull_mem,
                                                                                                    "%s:%u", res->raddr, res->rport),
                                  "pid",                  OVAL_DATATYPE_INTEGER, (int64_t)n->pid,
-                                 "user_id",              OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_uid_mem,
-                                                                                                   "%u", n->uid),
+				 "user_id",              OVAL_DATATYPE_SEXP, SEXP_number_newu_64_r(&se_uid_mem, n->uid),
                                  NULL);
 
-	probe_cobj_add_item(probe_out, item);
-	SEXP_free(item);
+        probe_item_collect(ctx, item);
+
         SEXP_free_r(&se_lport_mem);
         SEXP_free_r(&se_rport_mem);
         SEXP_free_r(&se_lfull_mem);
@@ -380,8 +377,7 @@ static void addr_convert(const char *src, char *dest, int size)
 }
 
 
-static int read_tcp(const char *proc, const char *type, llist *l,
-		SEXP_t *probe_out)
+static int read_tcp(const char *proc, const char *type, llist *l, probe_ctx *ctx)
 {
 	int line = 0;
 	FILE *f;
@@ -422,7 +418,7 @@ static int read_tcp(const char *proc, const char *type, llist *l,
 				r.lport = local_port;
 				r.raddr = dest;
 				r.rport = rem_port;
-				report_finding(&r, l, probe_out);
+				report_finding(&r, l, ctx);
 			}
 		}
 	}
@@ -430,8 +426,7 @@ static int read_tcp(const char *proc, const char *type, llist *l,
 	return 0;
 }
 
-static int read_udp(const char *proc, const char *type, llist *l,
-		SEXP_t *probe_out)
+static int read_udp(const char *proc, const char *type, llist *l, probe_ctx *ctx)
 {
 	int line = 0;
 	FILE *f;
@@ -471,7 +466,7 @@ static int read_udp(const char *proc, const char *type, llist *l,
 				r.lport = local_port;
 				r.raddr = dest;
 				r.rport = rem_port;
-				report_finding(&r, l, probe_out);
+				report_finding(&r, l, ctx);
 			}
 		}
 	}
@@ -479,8 +474,7 @@ static int read_udp(const char *proc, const char *type, llist *l,
 	return 0;
 }
 
-static int read_raw(const char *proc, const char *type, llist *l,
-		SEXP_t *probe_out)
+static int read_raw(const char *proc, const char *type, llist *l, probe_ctx *ctx)
 {
 	int line = 0;
 	FILE *f;
@@ -520,7 +514,7 @@ static int read_raw(const char *proc, const char *type, llist *l,
 				r.lport = local_port;
 				r.raddr = dest;
 				r.rport = rem_port;
-				report_finding(&r, l, probe_out);
+				report_finding(&r, l, ctx);
 			}
 		}
 	}
@@ -528,16 +522,13 @@ static int read_raw(const char *proc, const char *type, llist *l,
 	return 0;
 }
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *arg)
 {
+        SEXP_t *object;
 	int err;
 	llist ll;
 
-        (void)filters;
-
-	if (object == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
+        object = probe_ctx_getobject(ctx);
 
 	req.protocol_ent = probe_obj_getent(object, "protocol", 1);
 	if (req.protocol_ent == NULL) {
@@ -563,26 +554,26 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 		SEXP_t *msg;
 
 		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "Permission error.");
-		probe_cobj_add_msg(probe_out, msg);
+		probe_cobj_add_msg(probe_ctx_getresult(ctx), msg);
 		SEXP_free(msg);
-		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
+		probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
 
 		err = PROBE_EUNKNOWN;
 		goto cleanup;
 	}
 
 	// Now we check the tcp socket list...
-	read_tcp("/proc/net/tcp", "tcp", &ll, probe_out);
-	read_tcp("/proc/net/tcp6", "tcp", &ll, probe_out);
+	read_tcp("/proc/net/tcp", "tcp", &ll, ctx);
+	read_tcp("/proc/net/tcp6", "tcp", &ll, ctx);
 
 	// Next udp sockets...
-	read_udp("/proc/net/udp", "udp", &ll, probe_out);
-	read_udp("/proc/net/udp6", "udp", &ll, probe_out);
+	read_udp("/proc/net/udp", "udp", &ll, ctx);
+	read_udp("/proc/net/udp6", "udp", &ll, ctx);
 
 	// Next, raw sockets...not exactly part of standard yet. They
 	// can be used to send datagrams, so we will pretend they are udp
-	read_raw("/proc/net/raw", "udp", &ll, probe_out);
-	read_raw("/proc/net/raw6", "udp", &ll, probe_out);
+	read_raw("/proc/net/raw", "udp", &ll, ctx);
+	read_raw("/proc/net/raw6", "udp", &ll, ctx);
 
 	list_clear(&ll);
 
