@@ -62,6 +62,20 @@
 #include <rpm/rpmts.h>
 #include <rpm/rpmmacro.h>
 #include <rpm/rpmlog.h>
+#include <rpm/header.h>
+
+#ifndef HAVE_HEADERFORMAT
+# define HAVE_LIBRPM44 1 /* hack */
+# define headerFormat(_h, _fmt, _emsg) headerSprintf((_h),( _fmt), rpmTagTable, rpmHeaderFormats, (_emsg))
+#endif
+
+#ifndef HAVE_RPMFREECRYPTO
+# define rpmFreeCrypto() while(0)
+#endif
+
+#ifndef HAVE_RPMFREEFILESYSTEMS
+# define rpmFreeFilesystems() while(0)
+#endif
 
 /* SEAP */
 #include <seap.h>
@@ -70,12 +84,6 @@
 #include <alloc.h>
 #include <common/assume.h>
 
-#ifdef HAVE_LIBRPM44
-#include <rpm/header.h>
-#define headerFormat(_h, _fmt, _emsg) headerSprintf((_h),( _fmt), rpmTagTable, rpmHeaderFormats, (_emsg))
-#define rpmFreeCrypto() while(0)
-#define rpmFreeFilesystems() while(0)
-#endif
 
 #ifndef _A
 #define _A(x) assert(x)
@@ -149,7 +157,7 @@ static void pkgh2rep (Header h, struct rpminfo_rep *r)
 
         r->evr = str;
 
-        str = headerFormat (h, "%{SIGGPG:pgpsig}", &rpmerr);
+        str = headerFormat (h, "%|SIGGPG?{%{SIGGPG:pgpsig}}:{%{SIGPGP:pgpsig}}|", &rpmerr);
         sid = strrchr (str, ' ');
         r->signature_keyid = (sid != NULL ? strdup (sid+1) : strdup ("0"));
         oscap_free (str);
@@ -185,6 +193,20 @@ static int get_rpminfo (struct rpminfo_req *req, struct rpminfo_rep **rep)
 
                 ret = rpmdbGetIteratorCount (match);
 
+                break;
+	case OVAL_OPERATION_NOT_EQUAL:
+                match = rpmtsInitIterator (g_rpm.rpmts, RPMDBI_PACKAGES, NULL, 0);
+                if (match == NULL) {
+                        ret = 0;
+                        goto ret;
+                }
+
+                if (rpmdbSetIteratorRE (match, RPMTAG_NAME, RPMMIRE_GLOB, "*") != 0)
+                {
+                        ret = -1;
+                        goto ret;
+                }
+
                 break;
         case OVAL_OPERATION_PATTERN_MATCH:
                 match = rpmtsInitIterator (g_rpm.rpmts, RPMDBI_PACKAGES, NULL, 0);
@@ -269,7 +291,7 @@ void probe_fini (void *ptr)
         return;
 }
 
-int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main (probe_ctx *ctx, void *arg)
 {
         SEXP_t *val, *item, *ent;
 	int rpmret, i;
@@ -277,13 +299,7 @@ int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
         struct rpminfo_req request_st;
         struct rpminfo_rep *reply_st;
 
-        (void)filters;
-
-	if (object == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
-
-        ent = probe_obj_getent (object, "name", 1);
+        ent = probe_obj_getent (probe_ctx_getobject(ctx), "name", 1);
 
         if (ent == NULL) {
                 return (PROBE_ENOENT);
@@ -309,6 +325,7 @@ int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 
                 switch (request_st.op) {
                 case OVAL_OPERATION_EQUALS:
+		case OVAL_OPERATION_NOT_EQUAL:
                 case OVAL_OPERATION_PATTERN_MATCH:
                         break;
                 default:
@@ -351,10 +368,8 @@ int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
                                          "name", OVAL_DATATYPE_STRING, request_st.name,
                                          NULL);
 
-                probe_item_setstatus (item, OVAL_STATUS_ERROR);
-
-		probe_cobj_add_item(probe_out, item);
-                SEXP_free (item);
+                probe_item_setstatus (item, SYSCHAR_STATUS_ERROR);
+                probe_item_collect(ctx, item);
                 break;
         default: /* Ok */
                 _A(rpmret   >= 0);
@@ -380,10 +395,9 @@ int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
                                                          "signature_keyid", OVAL_DATATYPE_STRING, reply_st[i].signature_keyid,
                                                          NULL);
 
-				/* only for >= 5.8: probe_itement_setdatatype(item_sexp, "evr", OVAL_DATATYPE_EVR_STRING); */
-                                probe_cobj_add_item(probe_out, item);
-				SEXP_vfree(item, name, NULL);
+                                probe_item_collect(ctx, item);
 
+				SEXP_free(name);
                                 __rpminfo_rep_free (&(reply_st[i]));
                         }
 
