@@ -38,6 +38,7 @@
  * pid
  * ppid
  * priority
+ * ruid
  * scheduling_class
  * start_time
  * tty
@@ -66,7 +67,6 @@
 #include "probe/entcmp.h"
 #include "alloc.h"
 
-#if defined(__linux__)
 /* Convenience structure for the results being reported */
 struct result_info {
         const char *command;
@@ -74,40 +74,39 @@ struct result_info {
         unsigned pid;
         unsigned ppid;
         long priority;
+	int ruid;
         const char *scheduling_class;
         const char *start_time;
         int tty;
-        unsigned user_id;
+	int user_id;
 };
 
-unsigned long ticks, boot;
-
-static void report_finding(struct result_info *res, SEXP_t *probe_out)
+static void report_finding(struct result_info *res, probe_ctx *ctx)
 {
-        SEXP_t *item;
-        SEXP_t se_pid_mem, se_ppid_mem, se_tty_mem, se_pri_mem, se_uid_mem;
+        SEXP_t *item, se_tty_mem;
 
         item = probe_item_create(OVAL_UNIX_PROCESS, NULL,
                                  "command",   OVAL_DATATYPE_STRING, res->command,
                                  "exec_time", OVAL_DATATYPE_STRING, res->exec_time,
-                                 "pid",       OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_pid_mem, "%u", res->pid),
-                                 "ppid",      OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_ppid_mem, "%u", res->ppid),
-                                 "priority",  OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_pri_mem, "%ld", res->priority),
+                                 "pid",       OVAL_DATATYPE_INTEGER, (int64_t)res->pid,
+                                 "ppid",      OVAL_DATATYPE_INTEGER, (int64_t)res->ppid,
+                                 "priority",  OVAL_DATATYPE_INTEGER, (int64_t)res->priority,
+				 "ruid", OVAL_DATATYPE_INTEGER, (int64_t) res->ruid,
                                  "scheduling_class", OVAL_DATATYPE_STRING, res->scheduling_class,
                                  "start_time", OVAL_DATATYPE_STRING, res->start_time,
                                  "tty",        OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_tty_mem, "%d", res->tty),
-                                 "user_id",    OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_uid_mem, "%u", res->user_id),
+                                 "user_id",    OVAL_DATATYPE_INTEGER, (int64_t)res->user_id,
                                  NULL);
 
-	probe_cobj_add_item(probe_out, item);
-	SEXP_free(item);
-        SEXP_free_r(&se_pid_mem);
-        SEXP_free_r(&se_ppid_mem);
+        probe_item_collect(ctx, item);
+
         SEXP_free_r(&se_tty_mem);
-        SEXP_free_r(&se_pri_mem);
-        SEXP_free_r(&se_uid_mem);
 }
 
+#if defined(__linux__)
+
+unsigned long ticks, boot;
+
 static void get_boot_time(void)
 {
 	char buf[100];
@@ -134,12 +133,14 @@ static void get_boot_time(void)
 	fclose(sf);
 }
 
-static unsigned get_effective_id(int pid)
+static int get_uids(int pid, struct result_info *r)
 {
 	char buf[100];
-	unsigned euid = 0;
 	FILE *sf;
 
+	r->ruid = -1;
+	r->user_id = -1;
+
 	snprintf(buf, sizeof(buf), "/proc/%d/status", pid);
 	sf = fopen(buf, "rt");
 	if (sf) {
@@ -151,14 +152,14 @@ static unsigned get_effective_id(int pid)
 				continue;
 			}
 			if (memcmp(buf, "Uid:", 4) == 0) {
-				int id;
-				sscanf(buf, "Uid: %d %u", &id, &euid);
+				sscanf(buf, "Uid: %d %d", &r->ruid, &r->user_id);
 				break;
 			}
 		}
 		fclose(sf);
 	}
-	return euid;
+
+	return 0;
 }
 
 static char *convert_time(unsigned long long t, char *tbuf, int tb_size)
@@ -179,7 +180,7 @@ static char *convert_time(unsigned long long t, char *tbuf, int tb_size)
 	return tbuf;
 }
 
-static int read_process(SEXP_t *cmd_ent, SEXP_t *probe_out)
+static int read_process(SEXP_t *cmd_ent, probe_ctx *ctx)
 {
 	int err = 1;
 	DIR *d;
@@ -292,7 +293,7 @@ static int read_process(SEXP_t *cmd_ent, SEXP_t *probe_out)
 			// Select format based on how long we've been running
 			fmt = "%H:%M";
 			if (tday != proc->tm_yday)
-				fmt = "%b%d";
+				fmt = "%b_%d";
 			if (tyear != proc->tm_year)
 				fmt = "%Y";
 			strftime(sbuf, sizeof(sbuf), fmt, proc);
@@ -304,8 +305,8 @@ static int read_process(SEXP_t *cmd_ent, SEXP_t *probe_out)
 			r.priority = priority;
 			r.start_time = sbuf;
 			r.tty = tty_nr;
-			r.user_id = get_effective_id(pid);
-			report_finding(&r, probe_out);
+			get_uids(pid, &r);
+			report_finding(&r, ctx);
 		}
 		SEXP_free(cmd_sexp);
 	}
@@ -314,22 +315,16 @@ static int read_process(SEXP_t *cmd_ent, SEXP_t *probe_out)
 	return err;
 }
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *arg)
 {
 	SEXP_t *ent;
 
-        (void)filters;
-
-	if (object == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
-
-	ent = probe_obj_getent(object, "command", 1);
+	ent = probe_obj_getent(probe_ctx_getobject(ctx), "command", 1);
 	if (ent == NULL) {
 		return PROBE_ENOVAL;
 	}
 
-	if (read_process(ent, probe_out)) {
+	if (read_process(ent, ctx)) {
 		SEXP_free(ent);
 		return PROBE_EACCESS;
 	}
@@ -338,19 +333,160 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 
 	return 0;
 }
+#elif defined (__SVR4) && defined (__sun)
+
+#include <procfs.h>
+#include <unistd.h>
+
+static char *convert_time(time_t t, char *tbuf, int tb_size)
+{
+	unsigned d,h,m,s;
+
+	s = t%60;
+	t /= 60;
+	m = t%60;
+	t /=60;
+	h = t%24;
+	t /= 24;
+	d = t;
+	if (d)
+		snprintf(tbuf, tb_size, "%u-%02u:%02u:%02u", d, h, m, s);
+	else
+		snprintf(tbuf, tb_size,	"%02u:%02u:%02u", h, m, s);
+	return tbuf;
+}
+
+static int read_process(SEXP_t *cmd_ent, probe_ctx *ctx)
+{
+	int err = 1;
+	DIR *d;
+	struct dirent *ent;
+
+	d = opendir("/proc");
+	if (d == NULL)
+		return err;
+
+	psinfo_t *psinfo;
+
+	// Scan the directories
+	while (( ent = readdir(d) )) {
+		int fd, len;
+		char buf[336];
+		int pid;
+		unsigned sched_policy;
+		SEXP_t *cmd_sexp;
+
+
+		// Skip non-process dir entries
+		if(*ent->d_name<'0' || *ent->d_name>'9')
+			continue;
+		errno = 0;
+		pid = strtol(ent->d_name, NULL, 10);
+		if (errno || pid == 2) // skip err & kthreads
+			continue;
+
+		// Parse up the stat file for the proc
+		snprintf(buf, 32, "/proc/%d/psinfo", pid);
+		fd = open(buf, O_RDONLY, 0);
+		if (fd < 0)
+			continue;
+		len = read(fd, buf, sizeof buf);
+		close(fd);
+		if (len < 336)
+			continue;
+
+		// The psinfo file contains a psinfo struct; this typecast gets us the struct directly
+		psinfo = (psinfo_t *) buf;
+
+
+		err = 0; // If we get this far, no permission problems
+		_D("Have command: %s\n", psinfo->pr_fname);
+		cmd_sexp = SEXP_string_newf("%s", psinfo->pr_fname);
+		if (probe_entobj_cmp(cmd_ent, cmd_sexp) == OVAL_RESULT_TRUE) {
+			struct result_info r;
+			char tbuf[32], sbuf[32];
+			int tday,tyear;
+			time_t s_time;
+			struct tm *proc, *now;
+			const char *fmt;
+			int fixfmt_year;
+
+			r.scheduling_class = malloc(PRCLSZ);
+			strncpy(r.scheduling_class, (psinfo->pr_lwp).pr_clname, sizeof(r.scheduling_class));
+
+			// Get the start time
+			s_time = time(NULL);
+			now = localtime(&s_time);
+			tyear = now->tm_year;
+			tday = now->tm_yday;
+
+			// Get current time
+			s_time = psinfo->pr_start.tv_sec;
+			proc = localtime(&s_time);
+
+			// Select format based on how long we've been running
+			fixfmt_year = 0;
+			fmt = "%H:%M";
+			if (tday != proc->tm_yday)
+				fmt = "%b%d";
+			if (tyear != proc->tm_year)
+				fmt = "%Y";
+				fixfmt_year = 1;
+			strftime(sbuf, sizeof(sbuf), fmt, proc);
+
+			/* Fixing format from MMMYY to MMM_YY */
+			if (fixfmt_year == 1) {
+				sbuf[6] = '\0';
+				sbuf[5] = sbuf[4];
+				sbuf[4] = sbuf[3];
+				sbuf[3] = '_';
+			}
+
+			r.command = psinfo->pr_fname;
+			r.exec_time = convert_time(psinfo->pr_time.tv_sec, tbuf, sizeof(tbuf));
+			r.pid = psinfo->pr_pid;
+			r.ppid = psinfo->pr_ppid;
+			r.priority = (psinfo->pr_lwp).pr_pri;
+			r.ruid = psinfo->pr_uid;
+			r.start_time = sbuf;
+			r.tty = psinfo->pr_ttydev;
+			r.user_id = psinfo->pr_euid;
+			report_finding(&r, ctx);
+		}
+		SEXP_free(cmd_sexp);
+	}
+        closedir(d);
+
+	return err;
+}
+
+int probe_main(probe_ctx *ctx, void *arg)
+{
+	SEXP_t *ent;
+
+	ent = probe_obj_getent(probe_ctx_getobject(ctx), "command", 1);
+	if (ent == NULL) {
+		return PROBE_ENOVAL;
+	}
+
+	if (read_process(ent, ctx)) {
+		SEXP_free(ent);
+		return PROBE_EACCESS;
+	}
+
+	SEXP_free(ent);
+
+	return 0;
+}
+
 #else
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
+int probe_main(probe_ctx *ctx, void *arg)
 {
         SEXP_t *item_sexp;
 
-	if (object == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
-
 	item_sexp = probe_item_creat ("process_item", NULL, NULL);
-        probe_item_setstatus (item_sexp, OVAL_STATUS_NOTCOLLECTED);
-	probe_cobj_add_item(probe_out, item_sexp);
-        SEXP_free (item_sexp);
+        probe_item_setstatus (item_sexp, SYSCHAR_STATUS_NOT_COLLECTED);
+        probe_item_collect(ctx, item_sexp);
 
 	return 0;
 }
