@@ -59,18 +59,15 @@
 #include "alloc.h"
 
 #ifndef HAVE_SHADOW_H
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
+int probe_main(probe_ctx *ctx, void *arg)
 {
         SEXP_t *item_sexp;
 
-	if (object == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
+        (void)arg;
 
 	item_sexp = probe_item_creat ("shadow_item", NULL, NULL);
-        probe_item_setstatus (item_sexp, OVAL_STATUS_NOTCOLLECTED);
-	probe_cobj_add_item(probe_out, item_sexp);
-        SEXP_free (item_sexp);
+        probe_item_setstatus (item_sexp, SYSCHAR_STATUS_NOT_COLLECTED);
+        probe_item_collect(ctx, item_sexp);
 
         return 0;
 }
@@ -91,9 +88,56 @@ struct result_info {
         unsigned long flag;
 };
 
-static void report_finding(struct result_info *res, SEXP_t *probe_out)
+static SEXP_t *parse_enc_mth(const char *pwd)
 {
-        SEXP_t *item;
+	char *mth_str;
+
+	switch (*pwd) {
+	case '_':
+		return SEXP_string_newf("BSDi");
+	case '$':
+		pwd++;
+		switch (*pwd) {
+		case '1':
+			mth_str = "MD5";
+			pwd++;
+			break;
+		case '2':
+			mth_str = "Blowfish";
+			pwd++;
+			if (*pwd == 'a')
+				pwd++;
+			break;
+		case '5':
+			mth_str = "SHA-256";
+			pwd++;
+			break;
+		case '6':
+			mth_str = "SHA-512";
+			pwd++;
+			break;
+		default:
+			if (strncmp(pwd, "md5", 3))
+				goto fail;
+			mth_str = "Sun MD5";
+			pwd += 3;
+		}
+
+		if (*pwd != '$')
+			goto fail;
+
+		return SEXP_string_newf(mth_str);
+	default:
+		return SEXP_string_newf("DES");
+	}
+
+ fail:
+	return NULL;
+}
+
+static void report_finding(struct result_info *res, probe_ctx *ctx)
+{
+	SEXP_t *item, *enc_mth;
         SEXP_t se_chl_mem, se_cha_mem, se_chr_mem;
         SEXP_t se_exw_mem, se_exi_mem, se_exd_mem;
         SEXP_t se_flg_mem;
@@ -101,17 +145,22 @@ static void report_finding(struct result_info *res, SEXP_t *probe_out)
         item = probe_item_create(OVAL_UNIX_SHADOW, NULL,
                                  "username",  OVAL_DATATYPE_STRING, res->username,
                                  "password",  OVAL_DATATYPE_STRING, res->password,
-                                 "chg_lst",   OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_chl_mem, "%li", res->chg_lst),
-                                 "chg_allow", OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_cha_mem, "%li", res->chg_allow),
-                                 "chg_req",   OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_chr_mem, "%li", res->chg_req),
-                                 "exp_warn",  OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_exw_mem, "%li", res->exp_warn),
-                                 "exp_inact", OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_exi_mem, "%li", res->exp_inact),
-                                 "exp_date",  OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_exd_mem, "%li", res->exp_date),
+                                 "chg_lst",   OVAL_DATATYPE_SEXP, SEXP_number_newi_64_r(&se_chl_mem, res->chg_lst),
+                                 "chg_allow", OVAL_DATATYPE_SEXP, SEXP_number_newi_64_r(&se_cha_mem, res->chg_allow),
+                                 "chg_req",   OVAL_DATATYPE_SEXP, SEXP_number_newi_64_r(&se_chr_mem, res->chg_req),
+                                 "exp_warn",  OVAL_DATATYPE_SEXP, SEXP_number_newi_64_r(&se_exw_mem, res->exp_warn),
+                                 "exp_inact", OVAL_DATATYPE_SEXP, SEXP_number_newi_64_r(&se_exi_mem, res->exp_inact),
+                                 "exp_date",  OVAL_DATATYPE_SEXP, SEXP_number_newi_64_r(&se_exd_mem, res->exp_date),
                                  "flag",      OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_flg_mem, "%lu", res->flag),
                                  NULL);
+	enc_mth = parse_enc_mth(res->password);
+	if (enc_mth) {
+		probe_item_ent_add(item, "encrypt_method", NULL, enc_mth);
+		SEXP_free(enc_mth);
+	}
+
+        probe_item_collect(ctx, item);
 
-	probe_cobj_add_item(probe_out, item);
-	SEXP_free(item);
         SEXP_free_r(&se_chl_mem);
         SEXP_free_r(&se_cha_mem);
         SEXP_free_r(&se_chr_mem);
@@ -121,7 +170,7 @@ static void report_finding(struct result_info *res, SEXP_t *probe_out)
         SEXP_free_r(&se_flg_mem);
 }
 
-static int read_shadow(SEXP_t *un_ent, SEXP_t *probe_out)
+static int read_shadow(SEXP_t *un_ent, probe_ctx *ctx)
 {
 	int err = 1;
 	struct spwd *pw;
@@ -145,7 +194,7 @@ static int read_shadow(SEXP_t *un_ent, SEXP_t *probe_out)
 			r.exp_date = pw->sp_expire;
 			r.flag = pw->sp_flag;
 
-			report_finding(&r, probe_out);
+			report_finding(&r, ctx);
 		}
 		SEXP_free(un);
 	}
@@ -153,23 +202,17 @@ static int read_shadow(SEXP_t *un_ent, SEXP_t *probe_out)
 	return err;
 }
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *arg)
 {
 	SEXP_t *ent;
 
-        (void)filters;
-
-	if (object == NULL || probe_out == NULL) {
-		return (PROBE_EINVAL);
-	}
-
-	ent = probe_obj_getent(object, "username", 1);
+	ent = probe_obj_getent(probe_ctx_getobject(ctx), "username", 1);
 	if (ent == NULL) {
 		return PROBE_ENOVAL;
 	}
 
 	// Now we check the file...
-	read_shadow(ent, probe_out);
+	read_shadow(ent, ctx);
 	SEXP_free(ent);
 
 	return 0;
