@@ -32,6 +32,7 @@
 #endif
 
 #include <probe-api.h>
+#include <probe/entcmp.h>
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -50,6 +51,7 @@
 #include <alloc.h>
 #include <bfind.h>
 #include <common/debug_priv.h>
+#include <netdb.h>
 #include "../SEAP/generic/rbt/rbt.h"
 
 #define dI(...) oscap_dlprintf(DBG_I, __VA_ARGS__)
@@ -124,10 +126,7 @@ typedef struct {
 	bool unlisted;
 	bool rpc;
 
-	/*
-	 * Unused attributes; Maybe we will need them in some future version.
-	 */
-#if 0
+#if 0 /* Unused attributes; Maybe we will need them in some future version. */
 	bool groups;
 	bool mdns;
 	bool enabled;
@@ -201,6 +200,8 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 int xiconf_parse_service(xiconf_file_t *file, xiconf_service_t *service);
 int xiconf_parse_defaults(xiconf_file_t *file, xiconf_service_t *defaults, rbt_t *stree);
 xiconf_strans_t *xiconf_getservice(xiconf_t *xiconf, char *name, char *prot);
+xiconf_strans_t *xiconf_dump(xiconf_t *xiconf);
+void xiconf_strans_free(xiconf_strans_t *strans);
 
 int op_assign_bool(void *var, char *val);
 int op_assign_u16(void *var, char *val);
@@ -401,28 +402,69 @@ static xiconf_service_t *xiconf_service_new(void)
 	return (service);
 }
 
+static void xiconf_service_free(xiconf_service_t *service)
+{
+        if (service == NULL)
+                return;
+
+#define NONNULL_FREE(p) if (service->p != NULL) oscap_free(service->p)
+
+        if (service->id != service->name)
+                NONNULL_FREE(id);
+
+        NONNULL_FREE(name);
+        NONNULL_FREE(type);
+        NONNULL_FREE(flags);
+        NONNULL_FREE(socket_type);
+        NONNULL_FREE(protocol);
+        NONNULL_FREE(user);
+        NONNULL_FREE(server);
+        NONNULL_FREE(server_args);
+        NONNULL_FREE(only_from);
+        NONNULL_FREE(no_access);
+	oscap_free(service);
+}
+
+static void xiconf_stree_free_cb(struct rbt_str_node *n)
+{
+        xiconf_service_free(n->data);
+}
+
+static void xiconf_ttree_free_cb(struct rbt_str_node *n)
+{
+        xiconf_strans_free(n->data);
+	oscap_free(n->key);
+}
+
 void xiconf_free(xiconf_t *xiconf)
 {
 	register size_t i;
 
+	if (xiconf == NULL)
+		return;
+
 	for (i = 0; i < xiconf->count; ++i) {
-		oscap_free(xiconf->cfile[i]->cpath);
-		/* TODO: rbt_del(); */
+		if (xiconf->cfile[i]) {
+			if (xiconf->cfile[i]->cpath)
+				oscap_free(xiconf->cfile[i]->cpath);
+			if (xiconf->cfile[i]->inmem)
+				oscap_free(xiconf->cfile[i]->inmem);
+			oscap_free(xiconf->cfile[i]);
+		}
 	}
 
 	oscap_free(xiconf->cfile);
-	/* TODO: rbt_free();   */
-	/* TODO: free defaults */
+
+        rbt_str_free_cb(xiconf->stree, xiconf_stree_free_cb);
+        rbt_str_free_cb(xiconf->ttree, xiconf_ttree_free_cb);
+
+	oscap_free(xiconf->defaults->name);
+	oscap_free(xiconf->defaults);
 	oscap_free(xiconf);
 
 	return;
 }
 
-static void xiconf_service_free(xiconf_service_t *service)
-{
-	return;
-}
-
 static xiconf_file_t *xiconf_read(const char *path, int flags)
 {
 	int fd;
@@ -832,10 +874,9 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 {
 	xiconf_service_t *snew, *scur;
 	char *buffer;
-	register size_t bufidx;
+	register size_t bufidx, keyidx;
 	char  *l_pbeg;
 	char  *l_pend;
-	char  *l_next;
 	size_t l_size;
 	char *key, *op, *opval;
 	void **opvar;
@@ -902,9 +943,6 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 
 		memcpy (buffer, l_pbeg, l_size);
 
-		buffer[l_size] = ' ';
-		l_next = strchr(buffer,  ' ');
-		*l_next = '\0';
 		buffer[l_size] = '\0';
 
 		/* skip whitespaces in the line buffer */
@@ -914,7 +952,12 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 		 * now there should be a attribute name, comment
 		 * or the end-of-line.
 		 */
-		key = buffer + bufidx;
+		key = strdup(buffer + bufidx);
+		if (key == NULL)
+			exit(ENOMEM);
+
+		for (keyidx = 0; ! isspace(key[keyidx]) && key[keyidx]; keyidx++);
+		key[keyidx] = '\0';
 
 		switch (*key) {
 		case  '#':
@@ -996,6 +1039,7 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 			} else
 				goto fail;
 
+			while (*opval && isspace(*opval)) ++opval;
 			if (opfun == NULL)
 				break;
 			if (opfun(opvar, opval) != 0)
@@ -1003,6 +1047,8 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 		}
 
 		tmpbuf_free(buffer);
+		free(key);
+		key = NULL;
 	}
 
 finish_section:
@@ -1011,6 +1057,11 @@ finish_section:
 		tmpbuf_free(buffer);
 	}
 
+	if (key != NULL) {
+		dW("key not freed; freeing now...\n");
+		free(key);
+	}
+
 	switch (type) {
 	case XICONF_SECTION_DEFAULTS:
 		xiconf->defaults = snew;
@@ -1019,7 +1070,6 @@ finish_section:
 	{
 		xiconf_strans_t *st;
 		char   st_key[XICFG_STRANS_MAXKEYLEN+1];
-		size_t st_ksz;
 
 		if (snew->id == NULL)
 			snew->id = snew->name;
@@ -1042,8 +1092,19 @@ finish_section:
 			scur = snew;
 
 			if (scur->protocol == NULL) {
-				dW("FIXME: protocol == NULL!\n");
-				scur->protocol = strdup("foo");
+
+				struct servent *service;
+
+				dI("protocol is empty, trying to guess from /etc/services for %s\n", scur->name);
+				if ((service = getservbyname(scur->name, NULL)) != NULL) {
+					scur->protocol = strdup(service->s_proto);
+					dI("service %s has default protocol=%s\n", scur->name, scur->protocol);
+					endservent();
+				}
+				if (scur->protocol == NULL) {
+					dW("FIXME: protocol == NULL!\n");
+					scur->protocol = strdup("foo");
+				}
 			}
 		} else {
 			dI("Merge(%p, %p)\n", scur, snew);
@@ -1061,7 +1122,6 @@ finish_section:
 		 * (in case it's not already there)
 		 */
 		st = NULL;
-		st_ksz = strlen(scur->name) + strlen(scur->protocol);
 		strcpy(st_key, scur->name);
 		strcat(st_key, scur->protocol);
 
@@ -1109,6 +1169,17 @@ finish_section:
 	return (0);
 fail:
 	tmpbuf_free(buffer);
+	if (key) {
+		dW("fail: key not freed; freeing now...\n");
+		free(key);
+	}
+
+	if (snew->name) {
+		dW("fail: snew->name not freed; freeing now...\n");
+		free(snew->name);
+	}
+	free(snew);
+
 	return (-1);
 }
 
@@ -1133,6 +1204,45 @@ xiconf_strans_t *xiconf_getservice(xiconf_t *xiconf, char *name, char *prot)
 	return (strans);
 }
 
+static int xiconf_dump_cb(struct rbt_str_node *node, void *user)
+{
+	xiconf_strans_t *res = (xiconf_strans_t *)user;
+
+	assume_d(res->cnt > 0, -1);
+
+	res->srv[res->cnt - 1] = (xiconf_service_t *)node->data;
+	--res->cnt;
+
+	return (0);
+}
+
+xiconf_strans_t *xiconf_dump(xiconf_t *xiconf)
+{
+	xiconf_strans_t *res;
+
+	assume_d(xiconf != NULL, NULL);
+
+	res = oscap_talloc(xiconf_strans_t);
+	res->cnt = rbt_str_size(xiconf->stree);
+	res->srv = oscap_alloc(sizeof(xiconf_service_t *) * res->cnt);
+
+	rbt_str_walk_inorder2(xiconf->stree, xiconf_dump_cb, (void *)res, 0);
+
+	res->cnt = rbt_str_size(xiconf->stree);
+
+	return (res);
+}
+
+void xiconf_strans_free(xiconf_strans_t *strans)
+{
+        if (strans == NULL)
+                return;
+
+        oscap_free(strans->srv);
+        oscap_free(strans);
+        return;
+}
+
 int op_assign_bool(void *var, char *val)
 {
 	assume_d(var != NULL, -1);
@@ -1422,9 +1532,9 @@ void probe_fini(void *arg)
 		xiconf_free(arg);
 }
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
+int probe_main(probe_ctx *ctx, void *arg)
 {
-	SEXP_t *eval;
+	SEXP_t *service_name, *protocol, *eval, *object;
 	char    srv_name[256];
 	char    srv_prot[256];
 	int     err;
@@ -1432,40 +1542,44 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 	xiconf_strans_t  *xres;
 	xiconf_t         *xcfg = (xiconf_t *)arg;
 
-        (void)filters;
-
-	if (object == NULL || probe_out == NULL) {
-		err = PROBE_EINVAL;
-		goto fail;
-	}
-
 	if (arg == NULL) {
-		err = PROBE_EINIT;
-		goto fail;
+		probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_NOT_APPLICABLE);
+		return (0);
 	}
 
-	eval = probe_obj_getentval(object, "service_name", 1);
+        object = probe_ctx_getobject(ctx);
 
-	if (eval == NULL) {
+	service_name = probe_obj_getent(object, "service_name", 1);
+
+	if (service_name == NULL) {
 		err = PROBE_ENOVAL;
 		goto fail;
 	}
 
+	eval = probe_ent_getval(service_name);
+
 	if (SEXP_string_cstr_r (eval, srv_name, sizeof srv_name) == (size_t)-1) {
+		SEXP_free (service_name);
 		SEXP_free (eval);
 		err = PROBE_ERANGE;
 		goto fail;
 	}
 
 	SEXP_free (eval);
-	eval = probe_obj_getentval(object, "protocol", 1);
 
-	if (eval == NULL) {
+	protocol = probe_obj_getent(object, "protocol", 1);
+
+	if (protocol == NULL) {
+		SEXP_free (service_name);
 		err = PROBE_ENOVAL;
 		goto fail;
 	}
 
+	eval = probe_ent_getval(protocol);
+
 	if (SEXP_string_cstr_r (eval, srv_prot, sizeof srv_prot) == (size_t)-1) {
+		SEXP_free (service_name);
+		SEXP_free (protocol);
 		SEXP_free (eval);
 		err = PROBE_ERANGE;
 		goto fail;
@@ -1480,17 +1594,19 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 		goto fail;
 	}
 
-	_D("Looking for service(s) that match: (%s && %s)\n", srv_name, srv_prot);
-	xres = xiconf_getservice(xcfg, srv_name, srv_prot);
-
-	/* TODO: distinguish between an error and "not found" result */
+	xres = xiconf_dump(xcfg);
 
 	if (xres != NULL) {
-		SEXP_t *item;
+		SEXP_t *item, *xres_service_name, *xres_protocol;
 		register unsigned int l;
 
 		for (l = 0; l < xres->cnt; ++l) {
-                        item = probe_item_create(OVAL_UNIX_XINETD, NULL,
+			xres_service_name = SEXP_string_new( xres->srv[l]->name, strlen( xres->srv[l]->name));
+			xres_protocol = SEXP_string_new( xres->srv[l]->protocol, strlen( xres->srv[l]->protocol));
+			if (probe_entobj_cmp(service_name, xres_service_name) == OVAL_RESULT_TRUE &&
+			    probe_entobj_cmp(protocol, xres_protocol) == OVAL_RESULT_TRUE
+			   ) {
+				item = probe_item_create(OVAL_UNIX_XINETD, NULL,
 						"protocol",         OVAL_DATATYPE_STRING,  xres->srv[l]->protocol,
 						"service_name",     OVAL_DATATYPE_STRING,  xres->srv[l]->name,
 						"flags",            OVAL_DATATYPE_STRING,  xres->srv[l]->flags,
@@ -1506,14 +1622,20 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 						"disabled",         OVAL_DATATYPE_BOOLEAN, xres->srv[l]->disable,
 						NULL);
 
-			probe_cobj_add_item(probe_out, item);
-			SEXP_free(item);
+				probe_item_collect(ctx, item);
+			}
+			SEXP_free(xres_service_name);
+			SEXP_free(xres_protocol);
 		}
+		oscap_free(xres->srv);
+		oscap_free(xres);
+
 	}
+	SEXP_vfree(service_name, protocol, NULL);
 
 	return (0);
  fail:
-	probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
+	probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_ERROR);
 	return err;
 }
 #endif
