@@ -157,7 +157,7 @@ struct oval_sysitem_iterator;
 struct oval_sysent;
 /**
  * @struct oval_sysent_iterator
- * @see: oval_sysitem_get_items
+ * @see: oval_sysitem_get_sysents
  */
 struct oval_sysent_iterator;
 
@@ -198,7 +198,7 @@ struct oval_syschar_model *oval_syschar_model_new(struct oval_definition_model *
  * If imported content specifies a model entity that is already registered within the model its content is overwritten.
  * @param model the merge target model
  * @param file filename
- * @return -1 if an error occurred
+ * @return zero on success or non zero value if an error occurred
  * @memberof oval_syschar_model
  */
 int oval_syschar_model_import(struct oval_syschar_model *model, const char *file);
@@ -235,17 +235,6 @@ int oval_syschar_model_bind_variable_model(struct oval_syschar_model *, struct o
  * @memberof oval_syschar_model
  */
 void oval_syschar_model_set_sysinfo(struct oval_syschar_model *model, struct oval_sysinfo *sysinfo);
-/**
- * @memberof oval_syschar_model
- */
-void oval_syschar_model_add_variable_binding(struct oval_syschar_model *model, struct oval_variable_binding *binding);
-/**
- * Lock the syschar_model instance.
- * The state of a locked instance cannot be changed.
- * This operation has no effect if the model is already locked.
- * @memberof oval_syschar_model
- */
-void oval_syschar_model_lock(struct oval_syschar_model *syschar_model);
 /** @} */
 
 /**
@@ -289,28 +278,12 @@ oval_syschar_collection_flag_t oval_variable_get_collection_flag(struct oval_var
  * @memberof oval_syschar_model
  */
 struct oval_sysitem *oval_syschar_model_get_sysitem(struct oval_syschar_model *, const char *);
-/**
- * Return <b>true</b> if the syschar_model instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_syschar_model
- */
-bool oval_syschar_model_is_locked(struct oval_syschar_model *syschar_model);
 /** @} */
 
 /**
  * @name Evaluators
  * @{
  */
-/**
- * Return <b>true</b> if the syschar_model instance is valid
- * An oval_syschar_model is valid if all the following are true
- * 	- The sysinfo attribute is valid
- * 	- The associated oval_definition_model is valid
- * 	- All included syschars are valid
- * 	- All included variable_bindings are valid
- * @memberof oval_syschar_model
- */
-bool oval_syschar_model_is_valid(struct oval_syschar_model *syschar_model);
 /** @} */
 
 
@@ -391,12 +364,6 @@ char *oval_sysinfo_get_primary_host_name(struct oval_sysinfo *);
  * @memberof oval_sysinfo
  */
 struct oval_sysint_iterator *oval_sysinfo_get_interfaces(struct oval_sysinfo *);
-/**
- * Return <b>true</b> if the sysinfo instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_sysinfo
- */
-bool oval_sysinfo_is_locked(struct oval_sysinfo *sysinfo);
 /** @} */
 
 /**
@@ -421,11 +388,6 @@ void oval_sysinfo_iterator_free(struct oval_sysinfo_iterator *);
  * @name Evaluators
  * @{
  */
-/**
- * Return <b>true</b> if the sysinfo instance is valid
- * @memberof oval_sysinfo
- */
-bool oval_sysinfo_is_valid(struct oval_sysinfo *sysinfo);
 /** @} */
 
 
@@ -510,12 +472,6 @@ struct oval_variable_binding_iterator *oval_syschar_get_variable_bindings(struct
  * @memberof oval_syschar
  */
 struct oval_sysitem_iterator *oval_syschar_get_sysitem(struct oval_syschar *);
-/**
- * Return <b>true</b> if the syschar instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_syschar
- */
-bool oval_syschar_is_locked(struct oval_syschar *syschar);
 /** @} */
 
 /**
@@ -540,15 +496,6 @@ void oval_syschar_iterator_free(struct oval_syschar_iterator *);
  * @name Evaluators
  * @{
  */
-/**
- * Return <b>true</b> if the syschar instance is valid
- * An oval_syschar is valid if all the following are true
- * 	- The object attribute is valid
- * 	- All included variable_bindings are valid
- * 	- All included sysitem are valid
- * @memberof oval_syschar
- */
-bool oval_syschar_is_valid(struct oval_syschar *syschar);
 /** @} */
 
 
@@ -609,12 +556,6 @@ char *oval_sysint_get_ip_address(struct oval_sysint *);
  * @memberof oval_sysint
  */
 char *oval_sysint_get_mac_address(struct oval_sysint *);
-/**
- * Return <b>true</b> if the sysint instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_sysint
- */
-bool oval_sysint_is_locked(struct oval_sysint *sysint);
 /** @} */
 
 /**
@@ -639,11 +580,6 @@ void oval_sysint_iterator_free(struct oval_sysint_iterator *);
  * @name Evaluators
  * @{
  */
-/**
- * Return <b>true</b> if the sysint instance is valid
- * @memberof oval_sysint
- */
-bool oval_sysint_is_valid(struct oval_sysint *sysint);
 /** @} */
 
 
@@ -684,7 +620,7 @@ void oval_sysitem_add_message(struct oval_sysitem *, struct oval_message *);
 /**
  * @memberof oval_sysitem
  */
-void oval_sysitem_add_item(struct oval_sysitem *, struct oval_sysent *);
+void oval_sysitem_add_sysent(struct oval_sysitem *, struct oval_sysent *);
 /** @} */
 
 /**
@@ -705,7 +641,7 @@ oval_syschar_status_t oval_sysitem_get_status(struct oval_sysitem *);
  * Get system data individual items.
  * @memberof oval_sysitem
  */
-struct oval_sysent_iterator *oval_sysitem_get_items(struct oval_sysitem *);
+struct oval_sysent_iterator *oval_sysitem_get_sysents(struct oval_sysitem *);
 /**
  * Get system data message.
  * @memberof oval_sysitem
@@ -716,12 +652,6 @@ struct oval_message_iterator *oval_sysitem_get_messages(struct oval_sysitem *);
  * @memberof oval_sysitem
  */
 oval_subtype_t oval_sysitem_get_subtype(struct oval_sysitem *);
-/**
- * Return <b>true</b> if the sysitem instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_sysitem
- */
-bool oval_sysitem_is_locked(struct oval_sysitem *sysitem);
 /** @} */
 
 /**
@@ -746,13 +676,6 @@ void oval_sysitem_iterator_free(struct oval_sysitem_iterator *);
  * @name Evaluators
  * @{
  */
-/**
- * Return <b>true</b> if the sysitem instance is valid
- * An oval_sysitem is valid if all the following are true
- * 	- All included sysents are valid
- * @memberof oval_sysitem
- */
-bool oval_sysitem_is_valid(struct oval_sysitem *sysitem);
 /** @} */
 
 
@@ -785,6 +708,10 @@ void oval_sysent_set_name(struct oval_sysent *sysent, char *name);
  * @memberof oval_sysent
  */
 void oval_sysent_set_value(struct oval_sysent *sysent, char *value);
+/**
+ * @memberof oval_sysent
+ */
+void oval_sysent_add_record_field(struct oval_sysent *, struct oval_record_field *);
 /**
  * @memberof oval_sysent
  */
@@ -815,6 +742,11 @@ char *oval_sysent_get_name(struct oval_sysent *);
  */
 char *oval_sysent_get_value(struct oval_sysent *);
 
+/**
+ * @memberof oval_sysent
+ */
+struct oval_record_field_iterator *oval_sysent_get_record_fields(struct oval_sysent *);
+
 /**
  * Get system data item status.
  * @memberof oval_sysent
@@ -832,12 +764,6 @@ oval_datatype_t oval_sysent_get_datatype(struct oval_sysent *);
  * @memberof oval_sysent
  */
 int oval_sysent_get_mask(struct oval_sysent *);
-/**
- * Return <b>true</b> if the sysent instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_sysent
- */
-bool oval_sysent_is_locked(struct oval_sysent *sysent);
 /** @} */
 
 /**
@@ -862,16 +788,26 @@ void oval_sysent_iterator_free(struct oval_sysent_iterator *);
  * @name Evaluators
  * @{
  */
-/**
- * Return <b>true</b> if the sysent instance is valid
- * @memberof oval_sysent
- */
-bool oval_sysent_is_valid(struct oval_sysent *sysent);
 /** @} */
 
-
-
-
+/**
+ * @name Setters
+ * @{
+ */
+/**
+ * @memberof oval_record_field
+ */
+void oval_record_field_set_status(struct oval_record_field *, oval_syschar_status_t);
+/** @} */
+/**
+ * @name Getters
+ * @{
+ */
+/**
+ * @memberof oval_record_field
+ */
+oval_syschar_status_t oval_record_field_get_status(struct oval_record_field *);
+/** @} */
 
 /**
  * @memberof oval_message
@@ -915,12 +851,6 @@ char *oval_message_get_text(struct oval_message *message);
  * @memberof oval_message
  */
 oval_message_level_t oval_message_get_level(struct oval_message *message);
-/**
- * return <b>true</b> if the message instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_message
- */
-bool oval_message_is_locked(struct oval_message *message);
 /** @} */
 
 /**
@@ -941,19 +871,6 @@ struct oval_message *oval_message_iterator_next(struct oval_message_iterator *oc
 void oval_message_iterator_free(struct oval_message_iterator *oc_message);
 /** @} */
 
-/**
- * @name Evaluators
- * @{
- */
-/**
- * Return <b>true</b> if the message instance is valid
- * @memberof oval_message
- */
-bool oval_message_is_valid(struct oval_message *message);
-
-
-
-
 
 /**
  * @memberof oval_variable_binding
