@@ -107,31 +107,6 @@ void xccdf_policy_free(struct xccdf_policy *);
  */
 void xccdf_value_binding_free(struct xccdf_value_binding *);
 
-/**
- * Import/Export function
- * Not yet implemented
- * @memberof xccdf_policy
- */
-void xccdf_policy_export_variables(struct xccdf_policy *, char *export_namespace, const char *file);
-/**
- * Import/Export function
- * Not yet implemented 
- * @memberof xccdf_policy
- */
-void xccdf_policy_export_controls (struct xccdf_policy *, char *export_namespace, const char *file);
-/**
- * Import/Export function
- * Not yet implemented
- * @memberof xccdf_policy
- */
-void xccdf_policy_import_results(struct xccdf_policy *, char *import_namespace, const char *file);
-/**
- * Import/Export function
- * Not yet implemented
- * @memberof xccdf_policy
- */
-void xccdf_policy_export_results(struct xccdf_policy *, char *scoring_model_namespace, const char *file);
-
 /**
  * Function to register callback for checking system
  * @param model XCCDF Policy Model
@@ -439,6 +414,13 @@ struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struc
  */
 struct oscap_stringlist * xccdf_policy_model_get_files(struct xccdf_policy_model * policy_model);
 
+/**
+ * Return names of files that are used in checks of particular rules. Every check needs this file to be
+ * evaluated properly. If this file will not be imported and bind to the XCCDF Policy system the result
+ * of rule after evaluation will be "Not checked"
+ */
+struct oscap_stringlist * xccdf_item_get_files(struct xccdf_item * item);
+
 /************************************************************/
 /** @} End of Evaluators group */
 
@@ -505,6 +487,13 @@ void xccdf_value_binding_iterator_reset(struct xccdf_value_binding_iterator *it)
  */
 struct xccdf_score * xccdf_policy_get_score(struct xccdf_policy * policy, struct xccdf_result * test_result, const char * system);
 
+/**
+ * Perform textual substitution of cdf:sub elements with respect to given XCCDF policy.
+ * @param text text to be substituted
+ * @param policy policy to be used
+ */
+char* xccdf_policy_substitute(const char *text, struct xccdf_policy *policy);
+
 /************************************************************/
 /** @} End of Iterators group */
 
