@@ -20,13 +20,17 @@
  *      Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
-#include "list.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <assert.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdarg.h>
 
+#include "list.h"
+
 struct oscap_list *oscap_list_new(void)
 {
 	struct oscap_list *list = oscap_calloc(1, sizeof(struct oscap_list));
@@ -200,7 +204,7 @@ static inline void oscap_iterator_find_nearest(struct oscap_iterator *it)
 
 	do {
 		it->cur = (it->cur ? it->cur->next : it->list->first);
-	} while (!it->filter(it->cur->data, it->user_data) && oscap_iterator_has_more(it));
+	} while (it->cur && !it->filter(it->cur->data, it->user_data) && oscap_iterator_has_more(it));
 }
 
 void *oscap_iterator_new(struct oscap_list *list)
@@ -322,6 +326,7 @@ OSCAP_ITERATOR_GEN_T(const char *, oscap_string)
 OSCAP_ITERATOR_REMOVE_T(const char *, oscap_string, free)
 OSCAP_ITERATOR_GEN(oscap_stringlist)
 OSCAP_ITERATOR_REMOVE(oscap_stringlist, oscap_stringlist_free)
+    /*OSCAP_ITERATOR_RESET(oscap_string)*/
 
 
 #define OSCAP_DEFAULT_HSIZE 256
@@ -337,6 +342,9 @@ static inline unsigned int oscap_htable_hash(const char *str, size_t htable_size
 struct oscap_htable *oscap_htable_new1(oscap_compare_func cmp, size_t hsize)
 {
 	struct oscap_htable *t;
+    
+    assert(hsize > 0);
+
 	t = oscap_alloc(sizeof(struct oscap_htable));
 	if (t == NULL)
 		return NULL;
