@@ -41,38 +41,37 @@
 #include <stdlib.h>
 
 /// @cond
-#undef  __P
-#define __P __attribute__ ((unused)) static
+#define __ATTRIB __attribute__ ((unused)) static
 /// @endcond
 
 #if defined(NDEBUG)
 /// @cond
 void *__oscap_alloc(size_t s);
-__P void *oscap_alloc(size_t s)
+__ATTRIB void *oscap_alloc(size_t s)
 {
 	return __oscap_alloc(s);
 }
 
 void *__oscap_calloc(size_t n, size_t s);
-__P void *oscap_calloc(size_t n, size_t s)
+__ATTRIB void *oscap_calloc(size_t n, size_t s)
 {
 	return __oscap_calloc(n, s);
 }
 
 void *__oscap_realloc(void *p, size_t s);
-__P void *oscap_realloc(void *p, size_t s)
+__ATTRIB void *oscap_realloc(void *p, size_t s)
 {
 	return __oscap_realloc(p, s);
 }
 
 void *__oscap_reallocf(void *p, size_t s);
-__P void *oscap_reallocf(void *p, size_t s)
+__ATTRIB void *oscap_reallocf(void *p, size_t s)
 {
 	return __oscap_reallocf(p, s);
 }
 
 void __oscap_free(void *p);
-__P void oscap_free(void *p)
+__ATTRIB void oscap_free(void *p)
 {
 	__oscap_free(p);
 }
@@ -101,31 +100,31 @@ __P void oscap_free(void *p)
 
 #else
 void *__oscap_alloc_dbg(size_t s, const char *f, size_t l);
-__P void *oscap_alloc(size_t s)
+__ATTRIB void *oscap_alloc(size_t s)
 {
 	return __oscap_alloc_dbg(s, __FUNCTION__, 0);
 }
 
 void *__oscap_calloc_dbg(size_t n, size_t s, const char *f, size_t l);
-__P void *oscap_calloc(size_t n, size_t s)
+__ATTRIB void *oscap_calloc(size_t n, size_t s)
 {
 	return __oscap_calloc_dbg(n, s, __FUNCTION__, 0);
 }
 
 void *__oscap_realloc_dbg(void *p, size_t s, const char *f, size_t l);
-__P void *oscap_realloc(void *p, size_t s)
+__ATTRIB void *oscap_realloc(void *p, size_t s)
 {
 	return __oscap_realloc_dbg(p, s, __FUNCTION__, 0);
 }
 
 void *__oscap_reallocf_dbg(void *p, size_t s, const char *f, size_t l);
-__P void *oscap_reallocf(void *p, size_t s)
+__ATTRIB void *oscap_reallocf(void *p, size_t s)
 {
 	return __oscap_reallocf_dbg(p, s, __FUNCTION__, 0);
 }
 
 void __oscap_free_dbg(void **p, const char *f, size_t l);
-__P void oscap_free(void *p)
+__ATTRIB void oscap_free(void *p)
 {
 	__oscap_free_dbg(&p, __FUNCTION__, 0);
 }
