@@ -28,13 +28,16 @@
  *      Author: david.niemoller
  */
 
-#include "util.h"
-#include "list.h"
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
 #include <stdio.h>
 
 #include "text_priv.h"
+#include "util.h"
+#include "list.h"
 
 const char * const OSCAP_LANG_ENGLISH    = "en";
 const char * const OSCAP_LANG_ENGLISH_US = "en_US";
@@ -51,9 +54,16 @@ OSCAP_GENERIC_GETTER(bool, oscap_text, can_substitute, traits.can_substitute)
 OSCAP_GENERIC_GETTER(bool, oscap_text, can_override, traits.can_override)
 OSCAP_GENERIC_GETTER(bool, oscap_text, overrides, traits.overrides)
 OSCAP_ITERATOR_GEN_T(struct oscap_text *, oscap_text)
+    /*OSCAP_ITERATOR_RESET(oscap_text)*/
 OSCAP_ITERATOR_REMOVE_T(struct oscap_text *, oscap_text, oscap_text_free)
 
 
+bool oscap_text_set_overrides(struct oscap_text *text, bool overrides)
+{
+	text->traits.overrides = overrides;
+	return text->traits.can_override;
+}
+
 void oscap_text_free(struct oscap_text *text)
 {
     if (text != NULL) {
