@@ -9,7 +9,6 @@
 #
 # Authors:
 #      Peter Vrabec, <pvrabec@redhat.com>
-#      David Niemoller
 #      Ondrej Moris, <omoris@redhat.com>
 
 . ${srcdir}/../../test_common.sh
@@ -29,7 +28,14 @@ function test_api_oval_syschar {
 }
 
 function test_api_oval_results {
-    ./test_api_results $srcdir/results.xml $srcdir/exported-results.xml
+    ./test_api_results $srcdir/results.xml exported-results.xml
+    cmp $srcdir/results-good.xml exported-results.xml
+    return $?
+}
+
+function test_api_oval_directives {
+    ./test_api_directives $srcdir/directives.xml exported-directives.xml
+    cmp $srcdir/directives.xml exported-directives.xml
     return $?
 }
 
@@ -40,5 +46,6 @@ test_init "test_api_oval.log"
 test_run "test_api_oval_definition" test_api_oval_definition
 test_run "test_api_oval_syschar" test_api_oval_syschar
 test_run "test_api_oval_results" test_api_oval_results
+test_run "test_api_oval_directives" test_api_oval_directives
 
 test_exit
