@@ -34,10 +34,14 @@
 static int app_collect_oval(const struct oscap_action *action);
 static int app_evaluate_oval(const struct oscap_action *action);
 static int app_oval_xslt(const struct oscap_action *action);
+static int app_oval_list_probes(const struct oscap_action *action);
 static bool getopt_oval(int argc, char **argv, struct oscap_action *action);
+static bool getopt_oval_list_probes(int argc, char **argv, struct oscap_action *action);
 static bool getopt_oval_validate(int argc, char **argv, struct oscap_action *action);
 static int app_analyse_oval(const struct oscap_action *action);
 
+static bool valid_inputs(const struct oscap_action *action);
+
 static struct oscap_module* OVAL_SUBMODULES[];
 static struct oscap_module* OVAL_GEN_SUBMODULES[];
 
@@ -73,6 +77,7 @@ static struct oscap_module OVAL_EVAL = {
         "Options:\n"
 	"   --id <definition-id>\r\t\t\t\t - ID of the definition we want to evaluate.\n"
 	"   --variables <file>\r\t\t\t\t - Provide external variables expected by OVAL Definitions.\n"
+        "   --directives <file>\r\t\t\t\t - Use OVAL Directives content to specify desired results content.\n"
         "   --results <file>\r\t\t\t\t - Write OVAL Results into file.\n"
         "   --report <file>\r\t\t\t\t - Create human readable (HTML) report from OVAL Results.\n"
         "   --skip-valid\r\t\t\t\t - Skip validation.\n",
@@ -87,7 +92,10 @@ static struct oscap_module OVAL_COLLECT = {
     .usage = "[options] oval-definitions.xml",
     .help =
 	"Options:\n"
-	"   --variables <file>\r\t\t\t\t - Provide external variables expected by OVAL Definitions.\n",
+	"   --id <object>\r\t\t\t\t - Collect system characteristics ONLY for specified OVAL Object.\n"
+        "   --syschar <file>\r\t\t\t\t - Write OVAL System Characteristic into file.\n"
+	"   --variables <file>\r\t\t\t\t - Provide external variables expected by OVAL Definitions.\n"
+        "   --skip-valid\r\t\t\t\t - Skip validation.\n",
     .opt_parser = getopt_oval,
     .func = app_collect_oval
 };
@@ -96,10 +104,12 @@ static struct oscap_module OVAL_ANALYSE = {
     .name = "analyse",
     .parent = &OSCAP_OVAL_MODULE,
     .summary = "Evaluate provided system characteristics file",
-    .usage = "[options] oval-definitions.xml system-characteristics.xml" ,
+    .usage = "[options] --results FILE oval-definitions.xml system-characteristics.xml" ,
     .help =
 	"Options:\n"
-	"   --variables <file>\r\t\t\t\t - Provide external variables expected by OVAL Definitions.\n",
+	"   --variables <file>\r\t\t\t\t - Provide external variables expected by OVAL Definitions.\n"
+        "   --directives <file>\r\t\t\t\t - Use OVAL Directives content to specify desired results content.\n"
+        "   --skip-valid\r\t\t\t\t - Skip validation.\n",
     .opt_parser = getopt_oval,
     .func = app_analyse_oval
 };
@@ -125,6 +135,19 @@ static struct oscap_module OVAL_REPORT = {
     .func = app_oval_xslt
 };
 
+static struct oscap_module OVAL_LIST_PROBES = {
+    .name = "list-probes",
+    .parent = &OSCAP_OVAL_MODULE,
+    .summary = "List supported object types (i.e. probes)",
+    .usage = "[options]",
+    .help = "Options:\n"
+            "   --static\r\t\t\t\t - List all probes defined in the internal tables.\n"
+            "   --dynamic\r\t\t\t\t - List all probes supported on the current system (this is default behavior).\n"
+            "   --verbose\r\t\t\t\t - Be verbose.",
+    .opt_parser = getopt_oval_list_probes,
+    .func = app_oval_list_probes
+};
+
 static struct oscap_module* OVAL_GEN_SUBMODULES[] = {
     &OVAL_REPORT,
     NULL
@@ -135,6 +158,7 @@ static struct oscap_module* OVAL_SUBMODULES[] = {
     &OVAL_ANALYSE,
     &OVAL_VALIDATE,
     &OVAL_GENERATE,
+    &OVAL_LIST_PROBES,
     NULL
 };
 
@@ -195,15 +219,19 @@ int app_collect_oval(const struct oscap_action *action)
 	struct oval_syschar_model	*sys_model = NULL;
 	struct oval_sysinfo		*sysinfo   = NULL;
 	struct oval_probe_session	*pb_sess   = NULL;
-	int ret;
+	int ret = OSCAP_ERROR;
+
+	/* validate inputs */
+	if (action->validate) {
+		if (!valid_inputs(action)) {
+			goto cleanup;
+		}
+	}
 
 	/* import definitions */
 	def_model = oval_definition_model_import(action->f_oval);
 	if (def_model == NULL) {
 		fprintf(stderr, "Failed to import the OVAL Definitions from (%s).\n", action->f_oval);
-		if(oscap_err())
-			fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-		ret = OSCAP_ERROR;
 		goto cleanup;
 	}
 
@@ -212,17 +240,11 @@ int app_collect_oval(const struct oscap_action *action)
 		var_model = oval_variable_model_import(action->f_variables);
 		if (var_model == NULL) {
 			fprintf(stderr, "Failed to import the OVAL Variables from (%s).\n", action->f_oval);
-			if(oscap_err())
-				fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-			ret = OSCAP_ERROR;
 			goto cleanup;
 		}
 
 		if (oval_definition_model_bind_variable_model(def_model, var_model)) {
 			fprintf(stderr, "Failed to bind Variables to Definitions\n");
-			if(oscap_err())
-				fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-			ret = OSCAP_ERROR;
 			goto cleanup;
 		}
 	}
@@ -237,9 +259,6 @@ int app_collect_oval(const struct oscap_action *action)
 	ret = oval_probe_query_sysinfo(pb_sess, &sysinfo);
 	if (ret != 0) {
 		fprintf(stderr, "Failed to query sysinfo\n");
-		if(oscap_err())
-			fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-		ret = OSCAP_ERROR;
 		goto cleanup;
 	}
 	oval_syschar_model_set_sysinfo(sys_model, sysinfo);
@@ -248,52 +267,55 @@ int app_collect_oval(const struct oscap_action *action)
 	struct oval_object *object;
 	struct oval_syschar *syschar;
 	oval_syschar_collection_flag_t sc_flg;
-	struct oval_message_iterator *messages;
 	if (action->id) {
 		object = oval_definition_model_get_object(def_model, action->id);
-		if (VERBOSE >= 0) fprintf(stderr, "Collected: \"%s\" : ", oval_object_get_id(object));
+		if (!object) {
+			fprintf(stderr, "Object ID(%s) does not exist in %s.\n", action->id, action->f_oval);
+			goto cleanup;
+		}
+		if (VERBOSE >= 0)
+			fprintf(stderr, "Collected: \"%s\" : ", oval_object_get_id(object));
 		oval_probe_query_object(pb_sess, object, 0, &syschar);
 		sc_flg = oval_syschar_get_flag(syschar);
-		if (VERBOSE >= 0) {
+		if (VERBOSE >= 0)
 			fprintf(stderr, "%s\n", oval_syschar_collection_flag_get_text(sc_flg));
-			if (sc_flg == SYSCHAR_FLAG_ERROR) {
-				messages = oval_syschar_get_messages(syschar);
-				while (oval_message_iterator_has_more(messages)) {
-					struct oval_message *message = oval_message_iterator_next(messages);
-					fprintf(stderr, "\t%s\n", oval_message_get_text(message));
-				}
-				oval_message_iterator_free(messages);
-			}
-		}
 	}
 	else {
 	        struct oval_object_iterator *objects = oval_definition_model_get_objects(def_model);
 		while (oval_object_iterator_has_more(objects)) {
 			object = oval_object_iterator_next(objects);
-			if (VERBOSE >= 0) fprintf(stderr, "Collected: \"%s\" : ", oval_object_get_id(object));
+			if (VERBOSE >= 0)
+				fprintf(stderr, "Collected: \"%s\" : ", oval_object_get_id(object));
 			oval_probe_query_object(pb_sess, object, 0, &syschar);
 			sc_flg = oval_syschar_get_flag(syschar);
-			if (VERBOSE >= 0) {
+			if (VERBOSE >= 0)
 				fprintf(stderr, "%s\n", oval_syschar_collection_flag_get_text(sc_flg));
-				if (sc_flg == SYSCHAR_FLAG_ERROR) {
-					messages = oval_syschar_get_messages(syschar);
-					while (oval_message_iterator_has_more(messages)) {
-						struct oval_message *message = oval_message_iterator_next(messages);
-						fprintf(stderr, "\t%s\n", oval_message_get_text(message));
-					}
-					oval_message_iterator_free(messages);
-				}
-			}
 		}
 		oval_object_iterator_free(objects);
 	}
 
 	/* output */
-	oval_syschar_model_export(sys_model, action->f_syschar);
+	if (action->f_syschar != NULL) {
+		/* export OVAL System Characteristics */
+		oval_syschar_model_export(sys_model, action->f_syschar);
+
+		/* validate OVAL System Characteristics */
+		if (action->validate) {
+			if (!oscap_validate_document(action->f_syschar, OSCAP_DOCUMENT_OVAL_SYSCHAR, NULL,
+			    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+				fprintf(stdout, "OVAL System Characteristics are NOT exported correctly.\n");
+				goto cleanup;
+			}
+			fprintf(stdout, "OVAL System Characteristics are exported correctly.\n");
+		}
+	}
 
 	ret = OSCAP_OK;
 
 cleanup:
+	if(oscap_err())
+		fprintf(stderr, "%s %s\n", OSCAP_ERR_MSG, oscap_err_desc());
+
 	if (sysinfo) oval_sysinfo_free(sysinfo);
 	if (pb_sess) oval_probe_session_destroy(pb_sess);
 	if (sys_model) oval_syschar_model_free(sys_model);
@@ -308,44 +330,22 @@ int app_evaluate_oval(const struct oscap_action *action)
 
 	struct oval_definition_model	*def_model = NULL;
 	struct oval_variable_model	*var_model = NULL;
+	struct oval_directives_model	*dir_model = NULL;
 	struct oval_usr			*usr       = NULL;
 	oval_agent_session_t		*sess      = NULL;
-	int ret;
+	int ret = OSCAP_ERROR;
 
-	/* validate OVAL Definitions & Variables*/
-	if ( action->validate ) {
-		if (!oscap_validate_document(action->f_oval, OSCAP_DOCUMENT_OVAL_DEFINITIONS, NULL,
-		    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
-			if (oscap_err()) {
-				fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-				return OSCAP_FAIL;
-			}
-			fprintf(stdout, "%s\n", INVALID_DOCUMENT_MSG);
-			ret = OSCAP_ERROR;
+	/* validate inputs */
+	if (action->validate) {
+		if (!valid_inputs(action)) {
 			goto cleanup;
 		}
-
-		if (action->f_variables) {
-			if (!oscap_validate_document(action->f_variables, OSCAP_DOCUMENT_OVAL_VARIABLES, NULL,
-			    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
-				if (oscap_err()) {
-					fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-					return OSCAP_FAIL;
-				}
-				fprintf(stdout, "%s\n", INVALID_DOCUMENT_MSG);
-				ret = OSCAP_ERROR;
-				goto cleanup;
-			}
-		}
 	}
 
 	/* import OVAL Definitions */
 	def_model = oval_definition_model_import(action->f_oval);
 	if (def_model == NULL) {
 		fprintf(stderr, "Failed to import the OVAL Definitions from (%s).\n", action->f_oval);
-		if(oscap_err())
-			fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-		ret = OSCAP_ERROR;
 		goto cleanup;
 	}
 
@@ -354,27 +354,18 @@ int app_evaluate_oval(const struct oscap_action *action)
 		var_model = oval_variable_model_import(action->f_variables);
 		if (var_model == NULL) {
 			fprintf(stderr, "Failed to import the OVAL Variables from (%s).\n", action->f_oval);
-			if(oscap_err())
-				fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-			ret = OSCAP_ERROR;
 			goto cleanup;
 		}
 
 		if (oval_definition_model_bind_variable_model(def_model, var_model)) {
 			fprintf(stderr, "Failed to bind Variables to Definitions\n");
-			if(oscap_err())
-				fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-			ret = OSCAP_ERROR;
 			goto cleanup;
 		}
 	}
 
 	sess = oval_agent_new_session(def_model, basename(action->f_oval));
 	if (sess == NULL) {
-		if (oscap_err())
-			fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
 		fprintf(stderr, "Failed to create new agent session.\n");
-		ret = OSCAP_ERROR;
 		goto cleanup;
 	}
 
@@ -384,16 +375,14 @@ int app_evaluate_oval(const struct oscap_action *action)
 
 	/* Evaluation */
 	if (action->id)
-		ret = oval_agent_eval_definition(sess, action->id);
+		oval_agent_eval_definition(sess, action->id);
 	else
-		ret = oval_agent_eval_system(sess, app_oval_callback, usr);
+		oval_agent_eval_system(sess, app_oval_callback, usr);
 
-	if (ret && (oscap_err())) {
-		fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-		ret = OSCAP_ERROR;
+	if (oscap_err())
 		goto cleanup;
-	} else
-		printf("Evaluation done.\n");
+
+	printf("Evaluation done.\n");
 
 
 	/* export results to file */
@@ -401,25 +390,24 @@ int app_evaluate_oval(const struct oscap_action *action)
 		/* get result model */
 		struct oval_results_model *res_model = oval_agent_get_results_model(sess);
 
-		/* set up directives */
-		struct oval_result_directives *res_direct = oval_result_directives_new(res_model);
-		oval_result_directives_set_reported(res_direct, OVAL_RESULT_TRUE | OVAL_RESULT_FALSE |
-						    OVAL_RESULT_UNKNOWN | OVAL_RESULT_NOT_EVALUATED |
-						    OVAL_RESULT_ERROR | OVAL_RESULT_NOT_APPLICABLE, true);
+		/* import directives */
+		if (action->f_directives != NULL) {
+			dir_model = oval_directives_model_new();
+			oval_directives_model_import(dir_model, action->f_directives);
+		}
 
-		oval_result_directives_set_content(res_direct,
-						   OVAL_RESULT_TRUE |
-						   OVAL_RESULT_FALSE |
-						   OVAL_RESULT_UNKNOWN |
-						   OVAL_RESULT_NOT_EVALUATED |
-						   OVAL_RESULT_NOT_APPLICABLE |
-						   OVAL_RESULT_ERROR,
-						   OVAL_DIRECTIVE_CONTENT_FULL);
-
-		
 		/* export result model to XML */
-		oval_results_model_export(res_model, res_direct, action->f_results);
-		oval_result_directives_free(res_direct);
+		oval_results_model_export(res_model, dir_model, action->f_results);
+
+		/* validate OVAL Results */
+		if (action->validate) {
+			if (!oscap_validate_document(action->f_results, OSCAP_DOCUMENT_OVAL_RESULTS, NULL,
+			    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+				fprintf(stdout, "OVAL Results are NOT exported correctly.\n");
+				goto cleanup;
+			}
+			fprintf(stdout, "OVAL Results are exported correctly.\n");
+		}
 
 		/* generate report */
 	        if (action->f_report != NULL)
@@ -430,7 +418,10 @@ int app_evaluate_oval(const struct oscap_action *action)
 	if (action->id) {
 		oval_result_t res;
 
-		res = oval_agent_get_definition_result(sess, action->id);
+		if (oval_agent_get_definition_result(sess, action->id, &res)==-1) {
+			goto cleanup;
+		}
+
 		if (VERBOSE >= 0)
 			printf("Definition %s: %s\n", action->id, oval_result_get_text(res));
 		ret = (res == OVAL_RESULT_FALSE) ? OSCAP_FAIL : OSCAP_OK;
@@ -440,9 +431,13 @@ int app_evaluate_oval(const struct oscap_action *action)
 
 	/* clean up */
 cleanup:
+	if(oscap_err())
+		fprintf(stderr, "%s %s\n", OSCAP_ERR_MSG, oscap_err_desc());
+
 	if (usr) free(usr);
 	if (sess) oval_agent_destroy_session(sess);
 	if (def_model) oval_definition_model_free(def_model);
+	if (dir_model) oval_directives_model_free(dir_model);
 
 	return ret;
 }
@@ -452,15 +447,21 @@ static int app_analyse_oval(const struct oscap_action *action) {
 	struct oval_syschar_model	*sys_model = NULL;
 	struct oval_results_model	*res_model = NULL;
 	struct oval_variable_model	*var_model = NULL;
-	struct oval_result_directives	*res_direct = NULL;
+	struct oval_directives_model	*dir_model = NULL;
  	struct oval_syschar_model	*sys_models[2];
-	int ret;
+	int ret = OSCAP_ERROR;
+
+	/* validate inputs */
+	if (action->validate) {
+		if (!valid_inputs(action)) {
+			goto cleanup;
+		}
+	}
 
 	/* load defnitions */
 	def_model = oval_definition_model_import(action->f_oval);
         if (def_model == NULL) {
                 fprintf(stderr, "Failed to import the OVAL Definitions from (%s).\n", action->f_oval);
-                ret = OSCAP_ERROR;
 		goto cleanup;
         }
 
@@ -469,15 +470,11 @@ static int app_analyse_oval(const struct oscap_action *action) {
 		var_model = oval_variable_model_import(action->f_variables);
 		if (var_model == NULL) {
 			fprintf(stderr, "Failed to import the OVAL Variables from (%s).\n", action->f_oval);
-			ret = OSCAP_ERROR;
 			goto cleanup;
 		}
 
 		if (oval_definition_model_bind_variable_model(def_model, var_model)) {
 			fprintf(stderr, "Failed to bind Variables to Definitions\n");
-			if(oscap_err())
-				fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-			ret = OSCAP_ERROR;
 			goto cleanup;
 		}
 	}
@@ -486,9 +483,6 @@ static int app_analyse_oval(const struct oscap_action *action) {
 	sys_model = oval_syschar_model_new(def_model);
         if (oval_syschar_model_import(sys_model, action->f_syschar) ==  -1 ) {
                 fprintf(stderr, "Failed to import the System Characteristics from (%s).\n", action->f_syschar);
-		if(oscap_err())
-			fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-		ret = OSCAP_ERROR;
                 goto cleanup;
         }
 
@@ -499,28 +493,38 @@ static int app_analyse_oval(const struct oscap_action *action) {
 	oval_results_model_eval(res_model);
 
 	/* export results */
-	res_direct = oval_result_directives_new(res_model);
-	oval_result_directives_set_reported(res_direct, OVAL_RESULT_TRUE | OVAL_RESULT_FALSE |
-					    OVAL_RESULT_UNKNOWN | OVAL_RESULT_NOT_EVALUATED |
-					    OVAL_RESULT_ERROR | OVAL_RESULT_NOT_APPLICABLE, true);
-	oval_result_directives_set_content(res_direct,
-					   OVAL_RESULT_TRUE |
-					   OVAL_RESULT_FALSE |
-					   OVAL_RESULT_UNKNOWN |
-					   OVAL_RESULT_NOT_EVALUATED |
-					   OVAL_RESULT_NOT_APPLICABLE |
-					   OVAL_RESULT_ERROR,
-					   OVAL_DIRECTIVE_CONTENT_FULL);
-	oval_results_model_export(res_model, res_direct, "/dev/stdout");
+	if (action->f_results != NULL) {
+		/* import directives */
+		if (action->f_directives != NULL) {
+			dir_model = oval_directives_model_new();
+			oval_directives_model_import(dir_model, action->f_directives);
+		}
+
+		/* export result model to XML */
+		oval_results_model_export(res_model, dir_model, action->f_results);
+
+		/* validate OVAL Results */
+		if (action->validate) {
+			if (!oscap_validate_document(action->f_results, OSCAP_DOCUMENT_OVAL_RESULTS, NULL,
+			    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+				fprintf(stdout, "OVAL Results are NOT exported correctly.\n");
+				goto cleanup;
+			}
+			fprintf(stdout, "OVAL Results are exported correctly.\n");
+		}
+	}
 
 	ret = OSCAP_OK;
 
 	/* clean up */
 cleanup:
-	if(res_direct) oval_result_directives_free(res_direct);
+	if(oscap_err())
+		fprintf(stderr, "%s %s\n", OSCAP_ERR_MSG, oscap_err_desc());
+
 	if(res_model) oval_results_model_free(res_model);
 	if(sys_model) oval_syschar_model_free(sys_model);
 	if(def_model) oval_definition_model_free(def_model);
+	if(dir_model) oval_directives_model_free(dir_model);
 
 	return ret;
 }
@@ -531,6 +535,18 @@ static int app_oval_xslt(const struct oscap_action *action)
     return app_xslt(action->f_oval, action->module->user, action->f_results, NULL);
 }
 
+static int app_oval_list_probes(const struct oscap_action *action)
+{
+    int flags = 0;
+
+    if (action->list_dynamic)
+	flags |= OVAL_PROBEMETA_LIST_DYNAMIC;
+    if (action->verbosity >= 10)
+	flags |= OVAL_PROBEMETA_LIST_VERBOSE;
+
+    oval_probe_meta_list(stdout, flags);
+    return (0);
+}
 
 enum oval_opt {
     OVAL_OPT_RESULT_FILE = 1,
@@ -538,6 +554,7 @@ enum oval_opt {
     OVAL_OPT_ID,
     OVAL_OPT_VARIABLES,
     OVAL_OPT_SYSCHAR,
+    OVAL_OPT_DIRECTIVES,
     OVAL_OPT_OUTPUT = 'o'
 };
 
@@ -556,6 +573,7 @@ bool getopt_oval(int argc, char **argv, struct oscap_action *action)
 		{ "output",    	required_argument, NULL, OVAL_OPT_OUTPUT       },
 		{ "variables",	required_argument, NULL, OVAL_OPT_VARIABLES    },
 		{ "syschar",	required_argument, NULL, OVAL_OPT_SYSCHAR      },
+		{ "directives",	required_argument, NULL, OVAL_OPT_DIRECTIVES   },
         // flags
 		{ "skip-valid",	no_argument, &action->validate, 0 },
         // end
@@ -571,6 +589,7 @@ bool getopt_oval(int argc, char **argv, struct oscap_action *action)
 		case OVAL_OPT_ID: action->id = optarg; break;
 		case OVAL_OPT_VARIABLES: action->f_variables = optarg; break;
 		case OVAL_OPT_SYSCHAR: action->f_syschar = optarg; break;
+		case OVAL_OPT_DIRECTIVES: action->f_directives = optarg; break;
         	case 0: break;
 		default: return oscap_module_usage(action->module, stderr, NULL);
 		}
@@ -578,19 +597,56 @@ bool getopt_oval(int argc, char **argv, struct oscap_action *action)
 
 	/* We should have Definitions file here */
 	if (optind >= argc)
-        	return oscap_module_usage(action->module, stderr, "Definitions file needs to be specified!");
+		return oscap_module_usage(action->module, stderr, "Definitions file is not specified!");
 	action->f_oval = argv[optind];
 
-	/* We should have System Characteristics file here */
 	if (action->module == &OVAL_ANALYSE) {
+		/* We should have System Characteristics file here */
 		if ((optind+1) > argc)
-			return oscap_module_usage(action->module, stderr, "System characteristics file needs to be specified!");
+			return oscap_module_usage(action->module, stderr, "System characteristics file is not specified");
 		action->f_syschar = argv[optind + 1];
+
+		if (action->f_results == NULL) {
+			return oscap_module_usage(action->module, stderr, "OVAL Results file is not specified(--results parameter)");
+		}
 	}
 
 	return true;
 }
 
+bool getopt_oval_list_probes(int argc, char **argv, struct oscap_action *action)
+{
+#define PROBE_LIST_STATIC  0
+#define PROBE_LIST_DYNAMIC 1
+
+        int list_type = PROBE_LIST_DYNAMIC;
+	VERBOSE = action->verbosity;
+
+	action->doctype = OSCAP_DOCUMENT_OVAL_DEFINITIONS;
+
+	/* Command-options */
+	struct option long_options[] = {
+        // flags
+		{ "static",	no_argument, &list_type, PROBE_LIST_STATIC  },
+		{ "dynamic",	no_argument, &list_type, PROBE_LIST_DYNAMIC },
+		{ "verbose",    no_argument, &action->verbosity, 10},
+        // end
+		{ 0, 0, 0, 0 }
+	};
+
+	int c;
+	while ((c = getopt_long(argc, argv, "", long_options, NULL)) != -1) {
+		switch (c) {
+        	case 0: break;
+		default: return oscap_module_usage(action->module, stderr, NULL);
+		}
+	}
+
+	action->list_dynamic = list_type == PROBE_LIST_DYNAMIC ? true : false;
+
+	return true;
+}
+
 bool getopt_oval_validate(int argc, char **argv, struct oscap_action *action)
 {
 	VERBOSE = action->verbosity;
@@ -604,6 +660,7 @@ bool getopt_oval_validate(int argc, char **argv, struct oscap_action *action)
 		{ "variables",		no_argument, &action->doctype, OSCAP_DOCUMENT_OVAL_VARIABLES   },
 		{ "syschar",		no_argument, &action->doctype, OSCAP_DOCUMENT_OVAL_SYSCHAR     },
 		{ "results",		no_argument, &action->doctype, OSCAP_DOCUMENT_OVAL_RESULTS     },
+		{ "directives",		no_argument, &action->doctype, OSCAP_DOCUMENT_OVAL_DIRECTIVES  },
 		// force schematron validation
 		{ "schematron",		no_argument, &action->force, 1 },
         // end
@@ -626,3 +683,44 @@ bool getopt_oval_validate(int argc, char **argv, struct oscap_action *action)
 	return true;
 }
 
+static bool valid_inputs(const struct oscap_action *action) {
+        bool ret = false;
+
+	/* validate OVAL Definitions & Variables & Syschars */
+	if (!oscap_validate_document(action->f_oval, OSCAP_DOCUMENT_OVAL_DEFINITIONS, NULL,
+	    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+		fprintf(stdout, "Invalid OVAL Definition content in %s\n", action->f_oval);
+		goto cleanup;
+	}
+
+	if (action->f_variables) {
+		if (!oscap_validate_document(action->f_variables, OSCAP_DOCUMENT_OVAL_VARIABLES, NULL,
+		    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+			fprintf(stdout, "Invalid OVAL Variables content in %s\n", action->f_variables);
+			goto cleanup;
+		}
+	}
+
+	if (action->f_directives) {
+		if (!oscap_validate_document(action->f_directives, OSCAP_DOCUMENT_OVAL_DIRECTIVES, NULL,
+		    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+			fprintf(stdout, "Invalid OVAL Directives content in %s\n", action->f_directives);
+			goto cleanup;
+		}
+	}
+
+	if (action->module == &OVAL_ANALYSE && action->f_syschar) {
+		if (!oscap_validate_document(action->f_syschar, OSCAP_DOCUMENT_OVAL_SYSCHAR, NULL,
+		    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+			fprintf(stdout, "Invalid OVAL System Characteristics content in %s\n", action->f_syschar);
+			goto cleanup;
+		}
+	}
+
+	ret = true;
+
+cleanup:
+
+	return ret;
+}
+
