@@ -22,6 +22,7 @@
 
 #include <oval_probe.h>
 #include <oval_agent_api.h>
+#include <oval_agent_xccdf_api.h>
 #include <oval_results.h>
 #include <oval_variables.h>
 
@@ -36,10 +37,12 @@
 #include <unistd.h>
 
 #include "oscap-tool.h"
-
+#include "oscap.h"
+#include "alloc.h"
 
 static int app_evaluate_xccdf(const struct oscap_action *action);
 static int app_xccdf_resolve(const struct oscap_action *action);
+static int app_xccdf_export_oval_variables(const struct oscap_action *action);
 static bool getopt_xccdf(int argc, char **argv, struct oscap_action *action);
 static bool getopt_generate(int argc, char **argv, struct oscap_action *action);
 static int xccdf_gen_report(const char *infile, const char *id, const char *outfile, const char *show, const char *oval_template);
@@ -77,6 +80,15 @@ static struct oscap_module XCCDF_VALIDATE = {
     .func = app_validate_xml
 };
 
+static struct oscap_module XCCDF_EXPORT_OVAL_VARIABLES = {
+    .name = "export-oval-variables",
+    .parent = &OSCAP_XCCDF_MODULE,
+    .summary = "Export XCCDF values as OVAL external-variables document(s)",
+    .usage = "[options] <xccdf benchmark file> [oval definitions files]",
+    .opt_parser = getopt_xccdf,
+    .func = app_xccdf_export_oval_variables,
+};
+
 static struct oscap_module XCCDF_EVAL = {
     .name = "eval",
     .parent = &OSCAP_XCCDF_MODULE,
@@ -166,6 +178,7 @@ static struct oscap_module* XCCDF_SUBMODULES[] = {
     &XCCDF_EVAL,
     &XCCDF_RESOLVE,
     &XCCDF_VALIDATE,
+    &XCCDF_EXPORT_OVAL_VARIABLES,
     &XCCDF_GENERATE,
     NULL
 };
@@ -235,40 +248,35 @@ int app_evaluate_xccdf(const struct oscap_action *action)
         void **sessions = NULL;
 	char ** oval_files = NULL;
 	int idx = 0;
+
+	int retval = OSCAP_ERROR;
 	
 	/* Validate documents */
 	if( action->validate ) {
-        	if (!oscap_validate_document(action->f_xccdf, OSCAP_DOCUMENT_XCCDF, NULL, 
-					     (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
-        	        if (oscap_err()) {
-        	                fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-        	                return OSCAP_FAIL;
-        	        }
-			fprintf(stdout, "%s\n", INVALID_DOCUMENT_MSG);
-        	        return OSCAP_ERROR;
+		if (!oscap_validate_document(action->f_xccdf, OSCAP_DOCUMENT_XCCDF, NULL, (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+			fprintf(stdout, "Invalid XCCDF content in %s\n", action->f_xccdf);
+			goto cleanup;
         	}
 	}
 
 	/* Load XCCDF model and XCCDF Policy model */
 	benchmark = xccdf_benchmark_import(action->f_xccdf);
 	if (benchmark == NULL) {
-		if (oscap_err())
-			fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
-                return OSCAP_ERROR;
+		fprintf(stderr, "Failed to import the XCCDF content from (%s).\n", action->f_xccdf);
+		goto cleanup;
         }
 
+	/* Create policy model */
 	policy_model = xccdf_policy_model_new(benchmark);
 
-	/* Get the first policy, just for prototype - if there is no Policy 
-	 * report error and return -1 */
+	/* Select profile */
 	if (action->profile != NULL) {
 		policy = xccdf_policy_model_get_policy_by_id(policy_model, action->profile);
-
 		if(policy == NULL) {
                         fprintf(stderr, "Profile \"%s\" was not found.\n", action->profile);
-                        return OSCAP_ERROR;
+                        goto cleanup;
                 }
-	} else {
+	} else { /* Take first policy */
 		policy_it = xccdf_policy_model_get_policies(policy_model);
 		if (xccdf_policy_iterator_has_more(policy_it)) {
 			policy = xccdf_policy_iterator_next(policy_it);
@@ -277,16 +285,15 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 
 		if (policy == NULL) {
 			fprintf(stderr, "No Policy to evaluate. \n");
-			return OSCAP_ERROR;
+			goto cleanup;
 		}
 	}
 
-
 	/* Register callback */
 	xccdf_policy_model_register_start_callback(policy_model, scallback, (void*) action);
 	xccdf_policy_model_register_output_callback(policy_model, callback, (void*) action);
 
-	/* NO OVAL files? get ones from policy model */
+	/* Use OVAL files from policy model */
 	if (action->f_ovals == NULL) {
 		struct stat sb;
 		char * tmp_path;
@@ -325,46 +332,55 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 	/* Validate OVAL files */
 	if (action->validate) {
         	for (idx=0; oval_files[idx]; idx++) {
-			if (!oscap_validate_document(oval_files[idx], OSCAP_DOCUMENT_OVAL_DEFINITIONS, NULL,
-						     (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
-				if (oscap_err()) {
-					fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-					return OSCAP_FAIL;
-				}
-				fprintf(stdout, "%s\n", INVALID_DOCUMENT_MSG);
-				return OSCAP_ERROR;
+			if (!oscap_validate_document(oval_files[idx],
+						     OSCAP_DOCUMENT_OVAL_DEFINITIONS,
+						     NULL,
+						     (action->verbosity >= 0 ? oscap_reporter_fd : NULL),
+						     stdout)) {
+				fprintf(stdout, "Invalid OVAL Definition content in %s\n", oval_files[idx]);
+				goto cleanup;
 			}
 		}
 	}
 
-	/* Create OVAL sessions */
-        def_models = malloc((idx+1) * sizeof(struct oval_definition_model *));
-        sessions = malloc((idx+1) * sizeof(struct oval_agent_session *));
+	if (!oval_files[0]) {
+		fprintf(stderr, "No OVAL definition files present, aborting.\n");
+		goto cleanup;
+	}
+
+	/* Register checking engines */
         for (idx=0; oval_files[idx]; idx++) {
+		/* file -> def_model */
 	        struct oval_definition_model *tmp_def_model = oval_definition_model_import(oval_files[idx]);
 		if (tmp_def_model == NULL) {
-			if (oscap_err())
-				fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
-			return OSCAP_ERROR;
+			fprintf(stderr, "Failed to create definition model from: %s.\n", oval_files[idx]);
+			goto cleanup;
 		}
+		/* remember def_models */
+		def_models = realloc(def_models, (idx + 2) * sizeof(struct oval_definition_model *));
+		def_models[idx] = tmp_def_model;
+		def_models[idx+1] = NULL;
+
+		/* def_model -> session */
                 struct oval_agent_session *tmp_sess = oval_agent_new_session(tmp_def_model, basename(oval_files[idx]));
 		if (tmp_sess == NULL) {
-			if (oscap_err())
-				fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
-			fprintf(stderr, "Failed to create new agent session.\n");
-			return OSCAP_ERROR;
+			fprintf(stderr, "Failed to create new agent session for: %s.\n", oval_files[idx]);
+			goto cleanup;
 		}
+		/* remember sessions */
+		sessions = realloc(sessions, (idx + 2) * sizeof(struct oval_agent_session *));
+		sessions[idx] = tmp_sess;
+		sessions[idx+1] = NULL;
+
+		/* register session */
 	        xccdf_policy_model_register_engine_oval(policy_model, tmp_sess);
-                def_models[idx] = tmp_def_model;
-                sessions[idx] = tmp_sess;
 	}
-	def_models[idx] = NULL;
-	sessions[idx] = NULL;
-	
-	
+
 	/* Perform evaluation */
 	struct xccdf_result *ritem = xccdf_policy_evaluate(policy);
-        if (ritem == NULL) return OSCAP_ERROR;
+        if (ritem == NULL) {
+		goto cleanup;
+	}
 
 	/* Write results into XCCDF Test Result model */
 	xccdf_result_set_benchmark_uri(ritem, action->f_xccdf);
@@ -377,8 +393,8 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 			xccdf_result_set_profile(ritem, id);
 	}
 
-	/* Use sysinfo from the first oval session, if exist */
-	if( sessions[0] )
+	/* Use sysinfo from the first is the historical update frequenct oval session, if exist */
+	if (sessions && sessions[0])
 		oval_agent_export_sysinfo_to_xccdf_result(sessions[0], ritem);
 
 	struct xccdf_model_iterator *model_it = xccdf_benchmark_get_models(benchmark);
@@ -397,25 +413,8 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 			char * name =  malloc(PATH_MAX * sizeof(char));
 			sprintf(name, "%s.result.xml", oval_agent_get_filename(sessions[i]));
 
-			/* set up directives */
-			struct oval_result_directives *res_direct = oval_result_directives_new(res_model);
-			oval_result_directives_set_reported(res_direct, OVAL_RESULT_TRUE | OVAL_RESULT_FALSE |
-							    OVAL_RESULT_UNKNOWN | OVAL_RESULT_NOT_EVALUATED |
-							    OVAL_RESULT_ERROR | OVAL_RESULT_NOT_APPLICABLE, true);
-
-			oval_result_directives_set_content(res_direct,
-							   OVAL_RESULT_TRUE |
-							   OVAL_RESULT_FALSE |
-							   OVAL_RESULT_UNKNOWN |
-							   OVAL_RESULT_NOT_EVALUATED |
-							   OVAL_RESULT_NOT_APPLICABLE |
-							   OVAL_RESULT_ERROR,
-							   OVAL_DIRECTIVE_CONTENT_FULL);
-
-			
 			/* export result model to XML */
-			oval_results_model_export(res_model, res_direct, name);
-			oval_result_directives_free(res_direct);
+			oval_results_model_export(res_model, NULL, name);
 			free(name);
 		}
 	}
@@ -423,6 +422,18 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 	if (action->f_results != NULL) {
 		xccdf_benchmark_add_result(benchmark, xccdf_result_clone(ritem));
 		xccdf_benchmark_export(benchmark, action->f_results);
+
+		/* validate XCCDF Results */
+		if (action->validate) {
+			if (!oscap_validate_document(action->f_results, OSCAP_DOCUMENT_XCCDF, NULL,
+			    (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+				fprintf(stdout, "XCCDF Results are NOT exported correctly.\n");
+				goto cleanup;
+			}
+			fprintf(stdout, "XCCDF Results are exported correctly.\n");
+		}
+
+		/* generate report */
 		if (action->f_report != NULL)
 			xccdf_gen_report(action->f_results, xccdf_result_get_id(ritem), action->f_report, "", (action->oval_results ? "%.result.xml" : ""));
 	}
@@ -457,7 +468,7 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 	}
 
 	/* Get the result from TestResult model and decide if end with error or with correct return code */
-	int retval = OSCAP_OK;
+	retval = OSCAP_OK;
 	struct xccdf_rule_result_iterator *res_it = xccdf_result_get_rule_results(ritem);
 	while (xccdf_rule_result_iterator_has_more(res_it)) {
 		struct xccdf_rule_result *res = xccdf_rule_result_iterator_next(res_it);
@@ -466,23 +477,215 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 			retval = OSCAP_FAIL;
 	}
 	xccdf_rule_result_iterator_free(res_it);
-	/* Clear & End */
-        for (int i=0; def_models[i]; i++) {
-            oval_definition_model_free(def_models[i]);
-	    oval_agent_destroy_session(sessions[i]);
-        }
-	free(def_models);
-        free(sessions);
-	if (oval_files != action->f_ovals) {
+
+
+cleanup:
+	if (oscap_err())
+		fprintf(stderr, "%s %s\n", OSCAP_ERR_MSG, oscap_err_desc());
+
+	/* Definition Models */
+	if (def_models) {
+		for (int i=0; def_models[i]; i++)
+			oval_definition_model_free(def_models[i]);
+		free(def_models);
+	}
+
+	/* Sessions */
+	if (sessions) {
+		for (int i=0; sessions[i]; i++)
+			oval_agent_destroy_session(sessions[i]);
+		free(sessions);
+	}
+
+	/* OVAL files imported from XCCDF */
+	if (oval_files && (oval_files != action->f_ovals)) {
 		for(int i=0; oval_files[i]; i++) {
 			free(oval_files[i]);
 		}
 		free(oval_files);
 	}
-       	xccdf_policy_model_free(policy_model);
+
+	if (policy_model)
+		xccdf_policy_model_free(policy_model);
+
 	return retval;
 }
 
+static xccdf_test_result_type_t resolve_variables_wrapper(struct xccdf_policy *policy, const char *rule_id,
+	const char *id, const char *href, struct xccdf_value_binding_iterator *bnd_itr, void *usr)
+{
+	if (0 != oval_agent_resolve_variables((struct oval_agent_session *) usr, bnd_itr))
+		return XCCDF_RESULT_UNKNOWN;
+
+	return XCCDF_RESULT_PASS;
+}
+
+// todo: consolidate with app_evaluate_xccdf()
+static int app_xccdf_export_oval_variables(const struct oscap_action *action)
+{
+	struct xccdf_benchmark *benchmark;
+	struct xccdf_policy_model *policy_model = NULL;
+	struct xccdf_policy *policy = NULL;
+	struct xccdf_policy_iterator *policy_itr;
+	struct oval_definition_model **def_mod_lst = NULL;
+	struct oval_agent_session **ag_ses_lst = NULL;
+	struct xccdf_result *xres;
+	char **oval_file_lst = NULL;
+	int of_cnt = 0, i, ret = OSCAP_ERROR;
+
+	/* validate the XCCDF document */
+	if (action->validate) {
+		if (!oscap_validate_document(action->f_xccdf, OSCAP_DOCUMENT_XCCDF,
+			NULL, (action->verbosity >= 0) ? oscap_reporter_fd : NULL, stderr)) {
+			fprintf(stderr, "Ivalid XCCDF content in '%s'.\n", action->f_xccdf);
+			goto cleanup;
+		}
+	}
+
+	/* import the XCCDF document */
+	benchmark = xccdf_benchmark_import(action->f_xccdf);
+	if (benchmark == NULL) {
+		fprintf(stderr, "Failed to import the XCCDF document from '%s'.\n", action->f_xccdf);
+		goto cleanup;
+	}
+
+	/* create the policy model */
+	policy_model = xccdf_policy_model_new(benchmark);
+
+	/* select a profile */
+	if (action->profile != NULL) {
+		policy = xccdf_policy_model_get_policy_by_id(policy_model, action->profile);
+		if (policy == NULL) {
+			fprintf(stderr, "Unable to find profile '%s'.\n", action->profile);
+			goto cleanup;
+		}
+	} else {
+		/* use the first one if none specified */
+		policy_itr = xccdf_policy_model_get_policies(policy_model);
+		if (xccdf_policy_iterator_has_more(policy_itr))
+			policy = xccdf_policy_iterator_next(policy_itr);
+		xccdf_policy_iterator_free(policy_itr);
+		if (policy == NULL) {
+			fprintf(stderr, "No profile to evaluate.\n");
+			goto cleanup;
+		}
+	}
+
+	if (action->f_ovals != NULL) {
+		oval_file_lst = action->f_ovals;
+		for (of_cnt = 0; oval_file_lst[of_cnt]; of_cnt++);
+	} else {
+		char *xccdf_path_cpy, *dir_path;
+		struct oscap_stringlist *files;
+		struct oscap_string_iterator *files_itr;
+
+		oval_file_lst = oscap_talloc(char *);
+		oval_file_lst[0] = NULL;
+		of_cnt = 0;
+
+		xccdf_path_cpy = strdup(action->f_xccdf);
+		dir_path = dirname(xccdf_path_cpy);
+
+		files = xccdf_policy_model_get_files(policy_model);
+		files_itr = oscap_stringlist_get_strings(files);
+		while (oscap_string_iterator_has_more(files_itr)) {
+			char *filename;
+			char oval_path[PATH_MAX + 1];
+			struct stat sb;
+
+			filename = (char *) oscap_string_iterator_next(files_itr);
+			snprintf(oval_path, sizeof(oval_path), "%s/%s", dir_path, filename);
+			if (stat(oval_path, &sb)) {
+				fprintf(stderr, "warning: can't find file '%s' (referenced from XCCDF).\n", oval_path);
+			} else {
+				oval_file_lst[of_cnt++] = strdup(oval_path);
+				oval_file_lst = realloc(oval_file_lst, (of_cnt + 1) * sizeof(char *));
+				oval_file_lst[of_cnt] = NULL;
+			}
+		}
+		oscap_string_iterator_free(files_itr);
+		oscap_stringlist_free(files);
+		oscap_free(xccdf_path_cpy);
+	}
+
+	if (!oval_file_lst[0]) {
+		fprintf(stderr, "No OVAL definition files present, aborting.\n");
+		goto cleanup;
+	}
+
+	def_mod_lst = oscap_calloc(of_cnt, sizeof(struct oval_definition_model *));
+	ag_ses_lst = oscap_calloc(of_cnt, sizeof(struct oval_agent_session *));
+
+	for (i = 0; i < of_cnt; i++) {
+		def_mod_lst[i] = oval_definition_model_import(oval_file_lst[i]);
+		if (def_mod_lst[i] == NULL) {
+			fprintf(stderr, "Failed to import definitions model from '%s'.\n", oval_file_lst[i]);
+			goto cleanup;
+		}
+
+		ag_ses_lst[i] = oval_agent_new_session(def_mod_lst[i], basename(oval_file_lst[i]));
+		if (ag_ses_lst[i] == NULL) {
+			fprintf(stderr, "Failed to create new agent session for '%s'.\n", oval_file_lst[i]);
+			goto cleanup;
+		}
+
+		xccdf_policy_model_register_engine_callback(policy_model,
+			"http://oval.mitre.org/XMLSchema/oval-definitions-5",
+			resolve_variables_wrapper, ag_ses_lst[i]);
+	}
+
+	/* perform evaluation */
+	xres = xccdf_policy_evaluate(policy);
+	if (xres == NULL)
+		goto cleanup;
+
+	for (i = 0; i < of_cnt; i++) {
+		int j;
+		char *ses_name;
+		struct oval_variable_model_iterator *var_mod_itr;
+
+		j = 0;
+		ses_name = (char *) oval_agent_get_filename(ag_ses_lst[i]);
+
+		var_mod_itr = oval_definition_model_get_variable_models(def_mod_lst[i]);
+		while (oval_variable_model_iterator_has_more(var_mod_itr)) {
+			struct oval_variable_model *var_mod;
+			char fname[strlen(ses_name) + 32];
+
+			var_mod = oval_variable_model_iterator_next(var_mod_itr);
+			snprintf(fname, sizeof(fname), "%s-%d.variables-%d.xml", ses_name, i, j++);
+			oval_variable_model_export(var_mod, fname);
+		}
+		oval_variable_model_iterator_free(var_mod_itr);
+	}
+
+	ret = OSCAP_OK;
+
+ cleanup:
+	if (oscap_err())
+		fprintf(stderr, "%s %s\n", OSCAP_ERR_MSG, oscap_err_desc());
+
+	if (def_mod_lst != NULL) {
+		for (i = 0; i < of_cnt; i++) {
+			oval_agent_destroy_session(ag_ses_lst[i]);
+			oval_definition_model_free(def_mod_lst[i]);
+		}
+		oscap_free(ag_ses_lst);
+		oscap_free(def_mod_lst);
+	}
+
+	if (oval_file_lst && oval_file_lst != action->f_ovals) {
+		for (i = 0; i < of_cnt; i++)
+			oscap_free(oval_file_lst[i]);
+		oscap_free(oval_file_lst);
+	}
+
+	if (policy_model)
+		xccdf_policy_model_free(policy_model);
+
+	return ret;
+}
+
 int app_xccdf_resolve(const struct oscap_action *action)
 {
 	if (action->f_xccdf == NULL) {
